@@ -43,9 +43,8 @@ PALETTE = [
 
 class RootWidget(urwid.Frame):
 
-    def __init__(self, *args, ui: 'UI' = None, **kwargs):
+    def __init__(self, *args, ui: 'UI', **kwargs):
         super().__init__(*args, **kwargs)
-        assert ui is not None
         self.ui = ui
 
     def keypress(self, size: int, key: str) -> typing.Optional[str]:
@@ -67,16 +66,20 @@ class CommitWidget(urwid.Text):
     _selectable = True
 
     def __init__(self, ui: 'UI', commit: 'core.Commit'):
-        super().__init__(f'{commit.sha[:10]} {commit.description}')
+        assert commit.nomination_type is not None
+        reason = commit.nomination_type.name.ljust(6)
+        super().__init__(f'{commit.date()} {reason} {commit.sha[:10]} {commit.description}')
         self.ui = ui
         self.commit = commit
 
     async def apply(self) -> None:
         async with self.ui.git_lock:
-            result, err = await self.commit.apply(self.ui)
+            result, err = await self.commit.apply()
             if not result:
                 self.ui.chp_failed(self, err)
             else:
+                self.ui.feedback(f'{self.commit.sha} ({self.commit.description}) applied successfully.')
+                self.ui.save()
                 self.ui.remove_commit(self)
 
     async def denominate(self) -> None:
@@ -106,8 +109,8 @@ class UI:
 
     """Main management object.
 
-    :previous_commits: A list of commits to master since this branch was created
-    :new_commits: Commits added to master since the last time this script was run
+    :previous_commits: A list of commits to main since this branch was created
+    :new_commits: Commits added to main since the last time this script was run
     """
 
     commit_list: typing.List['urwid.Button'] = attr.ib(factory=lambda: urwid.SimpleFocusListWalker([]), init=False)
@@ -170,7 +173,7 @@ class UI:
             self.mainloop.widget = o
 
         for commit in reversed(list(itertools.chain(self.new_commits, self.previous_commits))):
-            if commit.nominated and commit.resolution is core.Resolution.UNRESOLVED:
+            if commit.nominated and commit.resolution in {core.Resolution.UNRESOLVED, core.Resolution.MANUAL_RESOLUTION}:
                 b = urwid.AttrMap(CommitWidget(self, commit), None, focus_map='reversed')
                 self.commit_list.append(b)
         self.save()
