@@ -105,22 +105,22 @@ CHIPSET(0x983D, KABINI)
 CHIPSET(0x983E, KABINI)
 CHIPSET(0x983F, KABINI)
 
-CHIPSET(0x9850, MULLINS)
-CHIPSET(0x9851, MULLINS)
-CHIPSET(0x9852, MULLINS)
-CHIPSET(0x9853, MULLINS)
-CHIPSET(0x9854, MULLINS)
-CHIPSET(0x9855, MULLINS)
-CHIPSET(0x9856, MULLINS)
-CHIPSET(0x9857, MULLINS)
-CHIPSET(0x9858, MULLINS)
-CHIPSET(0x9859, MULLINS)
-CHIPSET(0x985A, MULLINS)
-CHIPSET(0x985B, MULLINS)
-CHIPSET(0x985C, MULLINS)
-CHIPSET(0x985D, MULLINS)
-CHIPSET(0x985E, MULLINS)
-CHIPSET(0x985F, MULLINS)
+CHIPSET(0x9850, KABINI)
+CHIPSET(0x9851, KABINI)
+CHIPSET(0x9852, KABINI)
+CHIPSET(0x9853, KABINI)
+CHIPSET(0x9854, KABINI)
+CHIPSET(0x9855, KABINI)
+CHIPSET(0x9856, KABINI)
+CHIPSET(0x9857, KABINI)
+CHIPSET(0x9858, KABINI)
+CHIPSET(0x9859, KABINI)
+CHIPSET(0x985A, KABINI)
+CHIPSET(0x985B, KABINI)
+CHIPSET(0x985C, KABINI)
+CHIPSET(0x985D, KABINI)
+CHIPSET(0x985E, KABINI)
+CHIPSET(0x985F, KABINI)
 
 CHIPSET(0x1304, KAVERI)
 CHIPSET(0x1305, KAVERI)
@@ -157,83 +157,3 @@ CHIPSET(0x67B8, HAWAII)
 CHIPSET(0x67B9, HAWAII)
 CHIPSET(0x67BA, HAWAII)
 CHIPSET(0x67BE, HAWAII)
-
-CHIPSET(0x6900, ICELAND)
-CHIPSET(0x6901, ICELAND)
-CHIPSET(0x6902, ICELAND)
-CHIPSET(0x6903, ICELAND)
-CHIPSET(0x6907, ICELAND)
-
-CHIPSET(0x6920, TONGA)
-CHIPSET(0x6921, TONGA)
-CHIPSET(0x6928, TONGA)
-CHIPSET(0x6929, TONGA)
-CHIPSET(0x692B, TONGA)
-CHIPSET(0x692F, TONGA)
-CHIPSET(0x6930, TONGA)
-CHIPSET(0x6938, TONGA)
-CHIPSET(0x6939, TONGA)
-
-CHIPSET(0x9870, CARRIZO)
-CHIPSET(0x9874, CARRIZO)
-CHIPSET(0x9875, CARRIZO)
-CHIPSET(0x9876, CARRIZO)
-CHIPSET(0x9877, CARRIZO)
-
-CHIPSET(0x7300, FIJI)
-
-CHIPSET(0x67E0, POLARIS11)
-CHIPSET(0x67E1, POLARIS11)
-CHIPSET(0x67E3, POLARIS11)
-CHIPSET(0x67E7, POLARIS11)
-CHIPSET(0x67E8, POLARIS11)
-CHIPSET(0x67E9, POLARIS11)
-CHIPSET(0x67EB, POLARIS11)
-CHIPSET(0x67EF, POLARIS11)
-CHIPSET(0x67FF, POLARIS11)
-
-CHIPSET(0x67C0, POLARIS10)
-CHIPSET(0x67C1, POLARIS10)
-CHIPSET(0x67C2, POLARIS10)
-CHIPSET(0x67C4, POLARIS10)
-CHIPSET(0x67C7, POLARIS10)
-CHIPSET(0x67C8, POLARIS10)
-CHIPSET(0x67C9, POLARIS10)
-CHIPSET(0x67CA, POLARIS10)
-CHIPSET(0x67CC, POLARIS10)
-CHIPSET(0x67CF, POLARIS10)
-CHIPSET(0x67D0, POLARIS10)
-CHIPSET(0x67DF, POLARIS10)
-
-CHIPSET(0x98E4, STONEY)
-
-CHIPSET(0x6980, POLARIS12)
-CHIPSET(0x6981, POLARIS12)
-CHIPSET(0x6985, POLARIS12)
-CHIPSET(0x6986, POLARIS12)
-CHIPSET(0x6987, POLARIS12)
-CHIPSET(0x6995, POLARIS12)
-CHIPSET(0x6997, POLARIS12)
-CHIPSET(0x699F, POLARIS12)
-
-CHIPSET(0x694C, VEGAM)
-CHIPSET(0x694E, VEGAM)
-
-CHIPSET(0x6860, VEGA10)
-CHIPSET(0x6861, VEGA10)
-CHIPSET(0x6862, VEGA10)
-CHIPSET(0x6863, VEGA10)
-CHIPSET(0x6864, VEGA10)
-CHIPSET(0x6867, VEGA10)
-CHIPSET(0x6868, VEGA10)
-CHIPSET(0x687F, VEGA10)
-CHIPSET(0x686C, VEGA10)
-
-CHIPSET(0x69A0, VEGA12)
-CHIPSET(0x69A1, VEGA12)
-CHIPSET(0x69A2, VEGA12)
-CHIPSET(0x69A3, VEGA12)
-CHIPSET(0x69AF, VEGA12)
-
-CHIPSET(0x15DD, RAVEN)
-CHIPSET(0x15D8, RAVEN)
