@@ -1,28 +1,30 @@
-/*
- * Copyright © 2007-2019 Advanced Micro Devices, Inc.
- * All Rights Reserved.
- *
- * Permission is hereby granted, free of charge, to any person obtaining
- * a copy of this software and associated documentation files (the
- * "Software"), to deal in the Software without restriction, including
- * without limitation the rights to use, copy, modify, merge, publish,
- * distribute, sub license, and/or sell copies of the Software, and to
- * permit persons to whom the Software is furnished to do so, subject to
- * the following conditions:
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NON-INFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS, AUTHORS
- * AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
- * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
- * USE OR OTHER DEALINGS IN THE SOFTWARE.
- *
- * The above copyright notice and this permission notice (including the
- * next paragraph) shall be included in all copies or substantial portions
- * of the Software.
- */
+/**
+************************************************************************************************************************
+*
+* Copyright © 2007-2021 Advanced Micro Devices, Inc.
+* All Rights Reserved.
+*
+* Permission is hereby granted, free of charge, to any person obtaining a
+* copy of this software and associated documentation files (the "Software"),
+* to deal in the Software without restriction, including without limitation
+* the rights to use, copy, modify, merge, publish, distribute, sublicense,
+* and/or sell copies of the Software, and to permit persons to whom the
+* Software is furnished to do so, subject to the following conditions:
+*
+* The above copyright notice and this permission notice shall be included in
+* all copies or substantial portions of the Software.
+*
+* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+* THE COPYRIGHT HOLDER(S) OR AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
+* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
+* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+* OTHER DEALINGS IN THE SOFTWARE
+*
+************************************************************************************************************************
+*/
+
 
 /**
 ************************************************************************************************************************
@@ -310,6 +312,9 @@ ADDR_E_RETURNCODE Lib::ComputeSurfaceInfo(
                 if (pOut->pStereoInfo != NULL)
                 {
                     ComputeQbStereoInfo(pOut);
+#if DEBUG
+                    ValidateStereoInfo(pIn, pOut);
+#endif
                 }
             }
         }
@@ -826,7 +831,12 @@ ADDR_E_RETURNCODE Lib::ComputeDccAddrFromCoord(
     }
     else
     {
-        returnCode = HwlComputeDccAddrFromCoord(pIn, pOut);
+        returnCode = HwlSupportComputeDccAddrFromCoord(pIn);
+
+        if (returnCode == ADDR_OK)
+        {
+            HwlComputeDccAddrFromCoord(pIn, pOut);
+        }
     }
 
     return returnCode;
@@ -940,6 +950,37 @@ ADDR_E_RETURNCODE Lib::ComputeSubResourceOffsetForSwizzlePattern(
     return returnCode;
 }
 
+/**
+************************************************************************************************************************
+*   Lib::ComputeNonBlockCompressedView
+*
+*   @brief
+*       Interface function stub of Addr2ComputeNonBlockCompressedView.
+*
+*   @return
+*       ADDR_E_RETURNCODE
+************************************************************************************************************************
+*/
+ADDR_E_RETURNCODE Lib::ComputeNonBlockCompressedView(
+    const ADDR2_COMPUTE_NONBLOCKCOMPRESSEDVIEW_INPUT* pIn,
+    ADDR2_COMPUTE_NONBLOCKCOMPRESSEDVIEW_OUTPUT*      pOut)
+{
+    ADDR_E_RETURNCODE returnCode;
+
+    if ((GetFillSizeFieldsFlags() == TRUE) &&
+        ((pIn->size != sizeof(ADDR2_COMPUTE_NONBLOCKCOMPRESSEDVIEW_INPUT)) ||
+         (pOut->size != sizeof(ADDR2_COMPUTE_NONBLOCKCOMPRESSEDVIEW_OUTPUT))))
+    {
+        returnCode = ADDR_INVALIDPARAMS;
+    }
+    else
+    {
+        returnCode = HwlComputeNonBlockCompressedView(pIn, pOut);
+    }
+
+    return returnCode;
+}
+
 /**
 ************************************************************************************************************************
 *   Lib::ExtractPipeBankXor
@@ -1971,7 +2012,7 @@ VOID Lib::FilterInvalidEqSwizzleMode(
         const UINT_32 rsrcTypeIdx         = static_cast<UINT_32>(resourceType) - 1;
         UINT_32       validSwModeSet      = allowedSwModeSetVal;
 
-        for (UINT_32 swModeIdx = 0; validSwModeSet != 0; swModeIdx++)
+        for (UINT_32 swModeIdx = 1; validSwModeSet != 0; swModeIdx++)
         {
             if (validSwModeSet & 1)
             {
@@ -1992,6 +2033,169 @@ VOID Lib::FilterInvalidEqSwizzleMode(
     }
 }
 
+/**
+************************************************************************************************************************
+*   Lib::IsBlockTypeAvaiable
+*
+*   @brief
+*       Determine whether a block type is allowed in a given blockSet
+*
+*   @return
+*       N/A
+************************************************************************************************************************
+*/
+BOOL_32 Lib::IsBlockTypeAvaiable(
+    ADDR2_BLOCK_SET blockSet,
+    AddrBlockType   blockType)
+{
+    BOOL_32 avail;
+
+    if (blockType == AddrBlockLinear)
+    {
+        avail = blockSet.linear ? TRUE : FALSE;
+    }
+    else
+    {
+        avail = blockSet.value & (1 << (static_cast<UINT_32>(blockType) - 1)) ? TRUE : FALSE;
+    }
+
+    return avail;
+}
+
+/**
+************************************************************************************************************************
+*   Lib::BlockTypeWithinMemoryBudget
+*
+*   @brief
+*       Determine whether a new block type is acceptible based on memory waste ratio
+*
+*   @return
+*       N/A
+************************************************************************************************************************
+*/
+BOOL_32 Lib::BlockTypeWithinMemoryBudget(
+    UINT_64 minSize,
+    UINT_64 newBlockTypeSize,
+    UINT_32 ratioLow,
+    UINT_32 ratioHi,
+    DOUBLE  memoryBudget,
+    BOOL_32 newBlockTypeBigger)
+{
+    BOOL_32 accept = FALSE;
+
+    if (memoryBudget >= 1.0)
+    {
+        if (newBlockTypeBigger)
+        {
+            if ((static_cast<DOUBLE>(newBlockTypeSize) / minSize) <= memoryBudget)
+            {
+                accept = TRUE;
+            }
+        }
+        else
+        {
+            if ((static_cast<DOUBLE>(minSize) / newBlockTypeSize) > memoryBudget)
+            {
+                accept = TRUE;
+            }
+        }
+    }
+    else
+    {
+        if (newBlockTypeBigger)
+        {
+            if ((newBlockTypeSize * ratioHi) <= (minSize * ratioLow))
+            {
+                accept = TRUE;
+            }
+        }
+        else
+        {
+            if ((newBlockTypeSize * ratioLow) < (minSize * ratioHi))
+            {
+                accept = TRUE;
+            }
+        }
+    }
+
+    return accept;
+}
+
+#if DEBUG
+/**
+************************************************************************************************************************
+*   Lib::ValidateStereoInfo
+*
+*   @brief
+*       Validate stereo info by checking a few typical cases
+*
+*   @return
+*       N/A
+************************************************************************************************************************
+*/
+VOID Lib::ValidateStereoInfo(
+    const ADDR2_COMPUTE_SURFACE_INFO_INPUT*  pIn,   ///< [in] input structure
+    const ADDR2_COMPUTE_SURFACE_INFO_OUTPUT* pOut   ///< [in] output structure
+    ) const
+{
+    ADDR2_COMPUTE_SURFACE_ADDRFROMCOORD_INPUT addrIn = {};
+    addrIn.size            = sizeof(addrIn);
+    addrIn.swizzleMode     = pIn->swizzleMode;
+    addrIn.flags           = pIn->flags;
+    addrIn.flags.qbStereo  = 0;
+    addrIn.resourceType    = pIn->resourceType;
+    addrIn.bpp             = pIn->bpp;
+    addrIn.unalignedWidth  = pIn->width;
+    addrIn.numSlices       = pIn->numSlices;
+    addrIn.numMipLevels    = pIn->numMipLevels;
+    addrIn.numSamples      = pIn->numSamples;
+    addrIn.numFrags        = pIn->numFrags;
+
+    // Call Addr2ComputePipeBankXor() and validate different pbXor value if necessary...
+    const UINT_32 pbXor = 0;
+
+    ADDR2_COMPUTE_SURFACE_ADDRFROMCOORD_OUTPUT addrOut = {};
+    addrOut.size = sizeof(addrOut);
+
+    // Make the array to be {0, 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096} for full test
+    const UINT_32 TestCoord[] = {0};
+
+    for (UINT_32 xIdx = 0; xIdx < sizeof(TestCoord) / sizeof(TestCoord[0]); xIdx++)
+    {
+        if (TestCoord[xIdx] < pIn->width)
+        {
+            addrIn.x = TestCoord[xIdx];
+
+            for (UINT_32 yIdx = 0; yIdx  < sizeof(TestCoord) / sizeof(TestCoord[0]); yIdx++)
+            {
+                if (TestCoord[yIdx] < pIn->height)
+                {
+                    addrIn.y               = TestCoord[yIdx] + pOut->pStereoInfo->eyeHeight;
+                    addrIn.pipeBankXor     = pbXor ^ pOut->pStereoInfo->rightSwizzle;
+                    addrIn.unalignedHeight = pIn->height + pOut->pStereoInfo->eyeHeight;
+
+                    ADDR_E_RETURNCODE ret = ComputeSurfaceAddrFromCoord(&addrIn, &addrOut);
+                    ADDR_ASSERT(ret == ADDR_OK);
+
+                    const UINT_64 rightEyeOffsetFromBase = addrOut.addr;
+
+                    addrIn.y               = TestCoord[yIdx];
+                    addrIn.pipeBankXor     = pbXor;
+                    addrIn.unalignedHeight = pIn->height;
+
+                    ret = ComputeSurfaceAddrFromCoord(&addrIn, &addrOut);
+                    ADDR_ASSERT(ret == ADDR_OK);
+
+                    const UINT_64 rightEyeOffsetRelative = addrOut.addr;
+
+                    ADDR_ASSERT(rightEyeOffsetFromBase == rightEyeOffsetRelative + pOut->pStereoInfo->rightOffset);
+                }
+            }
+        }
+    }
+}
+#endif
+
 } // V2
 } // Addr
 
