@@ -57,6 +57,8 @@ radv_meta_save(struct radv_meta_saved_state *state, struct radv_cmd_buffer *cmd_
       state->viewport.count = cmd_buffer->state.dynamic.viewport.count;
       typed_memcpy(state->viewport.viewports, cmd_buffer->state.dynamic.viewport.viewports,
                    MAX_VIEWPORTS);
+      typed_memcpy(state->viewport.xform, cmd_buffer->state.dynamic.viewport.xform,
+                   MAX_VIEWPORTS);
 
       /* Save all scissors. */
       state->scissor.count = cmd_buffer->state.dynamic.scissor.count;
@@ -149,6 +151,8 @@ radv_meta_restore(const struct radv_meta_saved_state *state, struct radv_cmd_buf
       cmd_buffer->state.dynamic.viewport.count = state->viewport.count;
       typed_memcpy(cmd_buffer->state.dynamic.viewport.viewports, state->viewport.viewports,
                    MAX_VIEWPORTS);
+      typed_memcpy(cmd_buffer->state.dynamic.viewport.xform, state->viewport.xform,
+                   MAX_VIEWPORTS);
 
       /* Restore all scissors. */
       cmd_buffer->state.dynamic.scissor.count = state->scissor.count;
@@ -215,8 +219,10 @@ radv_meta_restore(const struct radv_meta_saved_state *state, struct radv_cmd_buf
    }
 
    if (state->flags & RADV_META_SAVE_COMPUTE_PIPELINE) {
-      radv_CmdBindPipeline(radv_cmd_buffer_to_handle(cmd_buffer), VK_PIPELINE_BIND_POINT_COMPUTE,
-                           radv_pipeline_to_handle(state->old_pipeline));
+      if (state->old_pipeline) {
+         radv_CmdBindPipeline(radv_cmd_buffer_to_handle(cmd_buffer), VK_PIPELINE_BIND_POINT_COMPUTE,
+                              radv_pipeline_to_handle(state->old_pipeline));
+      }
    }
 
    if (state->flags & RADV_META_SAVE_DESCRIPTORS) {
@@ -284,7 +290,7 @@ radv_meta_get_iview_layer(const struct radv_image *dest_image,
    }
 }
 
-static void *
+static VKAPI_ATTR void * VKAPI_CALL
 meta_alloc(void *_device, size_t size, size_t alignment, VkSystemAllocationScope allocationScope)
 {
    struct radv_device *device = _device;
@@ -292,7 +298,7 @@ meta_alloc(void *_device, size_t size, size_t alignment, VkSystemAllocationScope
                                          VK_SYSTEM_ALLOCATION_SCOPE_DEVICE);
 }
 
-static void *
+static VKAPI_ATTR void * VKAPI_CALL
 meta_realloc(void *_device, void *original, size_t size, size_t alignment,
              VkSystemAllocationScope allocationScope)
 {
@@ -301,7 +307,7 @@ meta_realloc(void *_device, void *original, size_t size, size_t alignment,
                                            VK_SYSTEM_ALLOCATION_SCOPE_DEVICE);
 }
 
-static void
+static VKAPI_ATTR void VKAPI_CALL
 meta_free(void *_device, void *data)
 {
    struct radv_device *device = _device;
@@ -487,8 +493,20 @@ radv_device_init_meta(struct radv_device *device)
    if (result != VK_SUCCESS)
       goto fail_accel_struct_build;
 
+   result = radv_device_init_meta_fmask_copy_state(device);
+   if (result != VK_SUCCESS)
+      goto fail_fmask_copy;
+
+   result = radv_device_init_meta_etc_decode_state(device, on_demand);
+   if (result != VK_SUCCESS)
+      goto fail_etc_decode;
+
    return VK_SUCCESS;
 
+fail_etc_decode:
+   radv_device_finish_meta_fmask_copy_state(device);
+fail_fmask_copy:
+   radv_device_finish_accel_struct_build_state(device);
 fail_accel_struct_build:
    radv_device_finish_meta_fmask_expand_state(device);
 fail_fmask_expand:
@@ -522,6 +540,7 @@ fail_clear:
 void
 radv_device_finish_meta(struct radv_device *device)
 {
+   radv_device_finish_meta_etc_decode_state(device);
    radv_device_finish_accel_struct_build_state(device);
    radv_device_finish_meta_clear_state(device);
    radv_device_finish_meta_resolve_state(device);
@@ -537,12 +556,31 @@ radv_device_finish_meta(struct radv_device *device)
    radv_device_finish_meta_fmask_expand_state(device);
    radv_device_finish_meta_dcc_retile_state(device);
    radv_device_finish_meta_copy_vrs_htile_state(device);
+   radv_device_finish_meta_fmask_copy_state(device);
 
    radv_store_meta_pipeline(device);
    radv_pipeline_cache_finish(&device->meta_state.cache);
    mtx_destroy(&device->meta_state.mtx);
 }
 
+nir_builder PRINTFLIKE(2, 3) radv_meta_init_shader(gl_shader_stage stage, const char *name, ...)
+{
+   nir_builder b = nir_builder_init_simple_shader(stage, NULL, NULL);
+   if (name) {
+      va_list args;
+      va_start(args, name);
+      b.shader->info.name = ralloc_vasprintf(b.shader, name, args);
+      va_end(args);
+   }
+
+   b.shader->info.internal = true;
+   b.shader->info.workgroup_size[0] = 1;
+   b.shader->info.workgroup_size[1] = 1;
+   b.shader->info.workgroup_size[2] = 1;
+
+   return b;
+}
+
 nir_ssa_def *
 radv_meta_gen_rect_vertices_comp2(nir_builder *vs_b, nir_ssa_def *comp2)
 {
@@ -583,7 +621,7 @@ radv_meta_build_nir_vs_generate_vertices(void)
 
    nir_variable *v_position;
 
-   nir_builder b = nir_builder_init_simple_shader(MESA_SHADER_VERTEX, NULL, "meta_vs_gen_verts");
+   nir_builder b = radv_meta_init_shader(MESA_SHADER_VERTEX, "meta_vs_gen_verts");
 
    nir_ssa_def *outvec = radv_meta_gen_rect_vertices(&b);
 
@@ -598,9 +636,7 @@ radv_meta_build_nir_vs_generate_vertices(void)
 nir_shader *
 radv_meta_build_nir_fs_noop(void)
 {
-   nir_builder b = nir_builder_init_simple_shader(MESA_SHADER_FRAGMENT, NULL, "meta_noop_fs");
-
-   return b.shader;
+   return radv_meta_init_shader(MESA_SHADER_FRAGMENT, "meta_noop_fs").shader;
 }
 
 void
@@ -687,3 +723,32 @@ radv_meta_load_descriptor(nir_builder *b, unsigned desc_set, unsigned binding)
                                                  .binding = binding);
    return nir_channels(b, rsrc, 0x3);
 }
+
+nir_ssa_def *
+get_global_ids(nir_builder *b, unsigned num_components)
+{
+   unsigned mask = BITFIELD_MASK(num_components);
+
+   nir_ssa_def *local_ids = nir_channels(b, nir_load_local_invocation_id(b), mask);
+   nir_ssa_def *block_ids = nir_channels(b, nir_load_workgroup_id(b, 32), mask);
+   nir_ssa_def *block_size = nir_channels(
+      b,
+      nir_imm_ivec4(b, b->shader->info.workgroup_size[0], b->shader->info.workgroup_size[1],
+                    b->shader->info.workgroup_size[2], 0),
+      mask);
+
+   return nir_iadd(b, nir_imul(b, block_ids, block_size), local_ids);
+}
+
+void
+radv_break_on_count(nir_builder *b, nir_variable *var, nir_ssa_def *count)
+{
+   nir_ssa_def *counter = nir_load_var(b, var);
+
+   nir_push_if(b, nir_uge(b, counter, count));
+   nir_jump(b, nir_jump_break);
+   nir_pop_if(b, NULL);
+
+   counter = nir_iadd(b, counter, nir_imm_int(b, 1));
+   nir_store_var(b, var, counter, 0x1);
+}
