@@ -21,66 +21,49 @@
  * IN THE SOFTWARE.
  */
 
-#include "ac_shader_args.h"
-#include "radv_constants.h"
-#include "util/list.h"
 #include "compiler/shader_enums.h"
+#include "util/list.h"
+#include "util/macros.h"
+#include "ac_shader_args.h"
 #include "amd_family.h"
+#include "radv_constants.h"
+#include "radv_shader.h"
 
 struct radv_shader_args {
-	struct ac_shader_args ac;
-	struct radv_shader_info *shader_info;
-	const struct radv_nir_compiler_options *options;
+   struct ac_shader_args ac;
 
-	struct ac_arg descriptor_sets[MAX_SETS];
-	struct ac_arg ring_offsets;
-	struct ac_arg scratch_offset;
+   struct ac_arg descriptor_sets[MAX_SETS];
+   struct ac_arg ring_offsets;
 
-	struct ac_arg vertex_buffers;
-	struct ac_arg rel_auto_id;
-	struct ac_arg vs_prim_id;
-	struct ac_arg es2gs_offset;
+   /* Streamout */
+   struct ac_arg streamout_buffers;
 
-	struct ac_arg oc_lds;
-	struct ac_arg merged_wave_info;
-	struct ac_arg tess_factor_offset;
-	struct ac_arg tes_rel_patch_id;
-	struct ac_arg tes_u;
-	struct ac_arg tes_v;
+   /* NGG GS */
+   struct ac_arg ngg_gs_state;
+   struct ac_arg ngg_culling_settings;
+   struct ac_arg ngg_viewport_scale[2];
+   struct ac_arg ngg_viewport_translate[2];
 
-	/* HW GS */
-	/* On gfx10:
-	 *  - bits 0..11: ordered_wave_id
-	 *  - bits 12..20: number of vertices in group
-	 *  - bits 22..30: number of primitives in group
-	 */
-	struct ac_arg gs_tg_info;
-	struct ac_arg gs2vs_offset;
-	struct ac_arg gs_wave_id;
-	struct ac_arg gs_vtx_offset[6];
+   struct ac_arg prolog_inputs;
+   struct ac_arg vs_inputs[MAX_VERTEX_ATTRIBS];
 
-	/* Streamout */
-	struct ac_arg streamout_buffers;
-	struct ac_arg streamout_write_idx;
-	struct ac_arg streamout_config;
-	struct ac_arg streamout_offset[4];
+   struct radv_userdata_locations user_sgprs_locs;
+   unsigned num_user_sgprs;
 
-	/* NGG GS */
-	struct ac_arg ngg_gs_state;
-
-	bool is_gs_copy_shader;
-	bool is_trap_handler_shader;
+   bool is_gs_copy_shader;
+   bool is_trap_handler_shader;
 };
 
 static inline struct radv_shader_args *
 radv_shader_args_from_ac(struct ac_shader_args *args)
 {
-	struct radv_shader_args *radv_args = NULL;
-	return (struct radv_shader_args *) container_of(args, radv_args, ac);
+   return container_of(args, struct radv_shader_args, ac);
 }
 
-void radv_declare_shader_args(struct radv_shader_args *args,
-			      gl_shader_stage stage,
-			      bool has_previous_stage,
-			      gl_shader_stage previous_stage);
+struct radv_nir_compiler_options;
+struct radv_shader_info;
 
+void radv_declare_shader_args(const struct radv_nir_compiler_options *options,
+                              const struct radv_shader_info *info, gl_shader_stage stage,
+                              bool has_previous_stage, gl_shader_stage previous_stage,
+                              struct radv_shader_args *args);
