@@ -25,6 +25,8 @@
 #include "radv_private.h"
 #include "radv_shader.h"
 
+#include "ac_nir.h"
+
 static void
 mark_sampler_desc(const nir_variable *var, struct radv_shader_info *info)
 {
@@ -49,15 +51,6 @@ gather_intrinsic_load_input_info(const nir_shader *nir, const nir_intrinsic_inst
    }
 }
 
-static uint32_t
-widen_writemask(uint32_t wrmask)
-{
-   uint32_t new_wrmask = 0;
-   for (unsigned i = 0; i < 4; i++)
-      new_wrmask |= (wrmask & (1 << i) ? 0x3 : 0x0) << (i * 2);
-   return new_wrmask;
-}
-
 static void
 set_writes_memory(const nir_shader *nir, struct radv_shader_info *info)
 {
@@ -76,7 +69,7 @@ gather_intrinsic_store_output_info(const nir_shader *nir, const nir_intrinsic_in
    uint8_t *output_usage_mask = NULL;
 
    if (instr->src[0].ssa->bit_size == 64)
-      write_mask = widen_writemask(write_mask);
+      write_mask = util_widen_mask(write_mask, 2);
 
    switch (nir->info.stage) {
    case MESA_SHADER_VERTEX:
@@ -128,27 +121,42 @@ gather_intrinsic_info(const nir_shader *nir, const nir_intrinsic_instr *instr,
    switch (instr->intrinsic) {
    case nir_intrinsic_load_barycentric_sample:
    case nir_intrinsic_load_barycentric_pixel:
-   case nir_intrinsic_load_barycentric_centroid: {
+   case nir_intrinsic_load_barycentric_centroid:
+   case nir_intrinsic_load_barycentric_at_sample:
+   case nir_intrinsic_load_barycentric_at_offset: {
       enum glsl_interp_mode mode = nir_intrinsic_interp_mode(instr);
       switch (mode) {
-      case INTERP_MODE_NONE:
       case INTERP_MODE_SMOOTH:
+      case INTERP_MODE_NONE:
+         if (instr->intrinsic == nir_intrinsic_load_barycentric_pixel ||
+             instr->intrinsic == nir_intrinsic_load_barycentric_at_sample ||
+             instr->intrinsic == nir_intrinsic_load_barycentric_at_offset)
+            info->ps.reads_persp_center = true;
+         else if (instr->intrinsic == nir_intrinsic_load_barycentric_centroid)
+            info->ps.reads_persp_centroid = true;
+         else if (instr->intrinsic == nir_intrinsic_load_barycentric_sample)
+            info->ps.reads_persp_sample = true;
+         break;
       case INTERP_MODE_NOPERSPECTIVE:
-         info->ps.uses_persp_or_linear_interp = true;
+         if (instr->intrinsic == nir_intrinsic_load_barycentric_pixel ||
+             instr->intrinsic == nir_intrinsic_load_barycentric_at_sample ||
+             instr->intrinsic == nir_intrinsic_load_barycentric_at_offset)
+            info->ps.reads_linear_center = true;
+         else if (instr->intrinsic == nir_intrinsic_load_barycentric_centroid)
+            info->ps.reads_linear_centroid = true;
+         else if (instr->intrinsic == nir_intrinsic_load_barycentric_sample)
+            info->ps.reads_linear_sample = true;
          break;
       default:
          break;
       }
-      break;
-   }
-   case nir_intrinsic_load_barycentric_at_offset:
-   case nir_intrinsic_load_barycentric_at_sample:
-      if (nir_intrinsic_interp_mode(instr) != INTERP_MODE_FLAT)
-         info->ps.uses_persp_or_linear_interp = true;
-
       if (instr->intrinsic == nir_intrinsic_load_barycentric_at_sample)
          info->ps.needs_sample_positions = true;
       break;
+   }
+   case nir_intrinsic_load_barycentric_model:
+      info->ps.reads_barycentric_model = true;
+      break;
    case nir_intrinsic_load_draw_id:
       info->vs.needs_draw_id = true;
       break;
@@ -161,6 +169,9 @@ gather_intrinsic_info(const nir_shader *nir, const nir_intrinsic_instr *instr,
    case nir_intrinsic_load_num_workgroups:
       info->cs.uses_grid_size = true;
       break;
+   case nir_intrinsic_load_ray_launch_size:
+      info->cs.uses_ray_launch_size = true;
+      break;
    case nir_intrinsic_load_local_invocation_id:
    case nir_intrinsic_load_workgroup_id: {
       unsigned mask = nir_ssa_def_components_read(&instr->dest.ssa);
@@ -182,14 +193,23 @@ gather_intrinsic_info(const nir_shader *nir, const nir_intrinsic_instr *instr,
    case nir_intrinsic_load_sample_mask_in:
       info->ps.reads_sample_mask_in = true;
       break;
-   case nir_intrinsic_load_view_index:
-      info->needs_multiview_view_index = true;
-      if (nir->info.stage == MESA_SHADER_FRAGMENT)
-         info->ps.layer_input = true;
+   case nir_intrinsic_load_sample_id:
+      info->ps.reads_sample_id = true;
       break;
-   case nir_intrinsic_load_layer_id:
-      if (nir->info.stage == MESA_SHADER_FRAGMENT)
-         info->ps.layer_input = true;
+   case nir_intrinsic_load_frag_shading_rate:
+      info->ps.reads_frag_shading_rate = true;
+      break;
+   case nir_intrinsic_load_front_face:
+      info->ps.reads_front_face = true;
+      break;
+   case nir_intrinsic_load_frag_coord:
+      info->ps.reads_frag_coord_mask = nir_ssa_def_components_read(&instr->dest.ssa);
+      break;
+   case nir_intrinsic_load_sample_pos:
+      info->ps.reads_sample_pos_mask = nir_ssa_def_components_read(&instr->dest.ssa);
+      break;
+   case nir_intrinsic_load_view_index:
+      info->uses_view_index = true;
       break;
    case nir_intrinsic_load_invocation_id:
       info->uses_invocation_id = true;
@@ -216,7 +236,10 @@ gather_intrinsic_info(const nir_shader *nir, const nir_intrinsic_instr *instr,
    case nir_intrinsic_image_deref_atomic_xor:
    case nir_intrinsic_image_deref_atomic_exchange:
    case nir_intrinsic_image_deref_atomic_comp_swap:
-   case nir_intrinsic_image_deref_size: {
+   case nir_intrinsic_image_deref_atomic_fmin:
+   case nir_intrinsic_image_deref_atomic_fmax:
+   case nir_intrinsic_image_deref_size:
+   case nir_intrinsic_image_deref_samples: {
       nir_variable *var =
          nir_deref_instr_get_variable(nir_instr_as_deref(instr->src[0].ssa->parent_instr));
       mark_sampler_desc(var, info);
@@ -231,7 +254,9 @@ gather_intrinsic_info(const nir_shader *nir, const nir_intrinsic_instr *instr,
           instr->intrinsic == nir_intrinsic_image_deref_atomic_or ||
           instr->intrinsic == nir_intrinsic_image_deref_atomic_xor ||
           instr->intrinsic == nir_intrinsic_image_deref_atomic_exchange ||
-          instr->intrinsic == nir_intrinsic_image_deref_atomic_comp_swap) {
+          instr->intrinsic == nir_intrinsic_image_deref_atomic_comp_swap ||
+          instr->intrinsic == nir_intrinsic_image_deref_atomic_fmin ||
+          instr->intrinsic == nir_intrinsic_image_deref_atomic_fmax) {
          set_writes_memory(nir, info);
       }
       break;
@@ -247,6 +272,8 @@ gather_intrinsic_info(const nir_shader *nir, const nir_intrinsic_instr *instr,
    case nir_intrinsic_ssbo_atomic_xor:
    case nir_intrinsic_ssbo_atomic_exchange:
    case nir_intrinsic_ssbo_atomic_comp_swap:
+   case nir_intrinsic_ssbo_atomic_fmin:
+   case nir_intrinsic_ssbo_atomic_fmax:
    case nir_intrinsic_store_global:
    case nir_intrinsic_global_atomic_add:
    case nir_intrinsic_global_atomic_imin:
@@ -258,6 +285,8 @@ gather_intrinsic_info(const nir_shader *nir, const nir_intrinsic_instr *instr,
    case nir_intrinsic_global_atomic_xor:
    case nir_intrinsic_global_atomic_exchange:
    case nir_intrinsic_global_atomic_comp_swap:
+   case nir_intrinsic_global_atomic_fmin:
+   case nir_intrinsic_global_atomic_fmax:
       set_writes_memory(nir, info);
       break;
    case nir_intrinsic_load_input:
@@ -310,7 +339,7 @@ gather_info_block(const nir_shader *nir, const nir_block *block, struct radv_sha
 
 static void
 gather_info_input_decl_vs(const nir_shader *nir, const nir_variable *var,
-                          struct radv_shader_info *info, const struct radv_shader_variant_key *key)
+                          const struct radv_pipeline_key *key, struct radv_shader_info *info)
 {
    unsigned attrib_count = glsl_count_attribute_slots(var->type, true);
 
@@ -387,22 +416,28 @@ gather_info_input_decl_ps(const nir_shader *nir, const nir_variable *var,
 
    uint64_t mask = ((1ull << attrib_count) - 1);
 
-   if (var->data.interpolation == INTERP_MODE_FLAT)
-      info->ps.flat_shaded_mask |= mask << var->data.driver_location;
-   if (var->data.interpolation == INTERP_MODE_EXPLICIT)
-      info->ps.explicit_shaded_mask |= mask << var->data.driver_location;
+   if (!var->data.per_primitive) {
+      if (var->data.interpolation == INTERP_MODE_FLAT)
+         info->ps.flat_shaded_mask |= mask << var->data.driver_location;
+      else if (var->data.interpolation == INTERP_MODE_EXPLICIT)
+         info->ps.explicit_shaded_mask |= mask << var->data.driver_location;
+   }
 
-   if (var->data.location >= VARYING_SLOT_VAR0)
-      info->ps.input_mask |= mask << (var->data.location - VARYING_SLOT_VAR0);
+   if (var->data.location >= VARYING_SLOT_VAR0) {
+      if (var->data.per_primitive)
+         info->ps.input_per_primitive_mask |= mask << (var->data.location - VARYING_SLOT_VAR0);
+      else
+         info->ps.input_mask |= mask << (var->data.location - VARYING_SLOT_VAR0);
+   }
 }
 
 static void
 gather_info_input_decl(const nir_shader *nir, const nir_variable *var,
-                       struct radv_shader_info *info, const struct radv_shader_variant_key *key)
+                       const struct radv_pipeline_key *key, struct radv_shader_info *info)
 {
    switch (nir->info.stage) {
    case MESA_SHADER_VERTEX:
-      gather_info_input_decl_vs(nir, var, info, key);
+      gather_info_input_decl_vs(nir, var, key, info);
       break;
    case MESA_SHADER_FRAGMENT:
       gather_info_input_decl_ps(nir, var, info);
@@ -448,31 +483,47 @@ gather_info_output_decl_gs(const nir_shader *nir, const nir_variable *var,
    info->gs.output_streams[idx] = stream;
 }
 
+static struct radv_vs_output_info *
+get_vs_output_info(const nir_shader *nir, struct radv_shader_info *info)
+{
+
+   switch (nir->info.stage) {
+   case MESA_SHADER_VERTEX:
+      if (!info->vs.as_ls && !info->vs.as_es)
+         return &info->vs.outinfo;
+      break;
+   case MESA_SHADER_GEOMETRY:
+      return &info->vs.outinfo;
+      break;
+   case MESA_SHADER_TESS_EVAL:
+      if (!info->tes.as_es)
+         return &info->tes.outinfo;
+      break;
+   case MESA_SHADER_MESH:
+      return &info->ms.outinfo;
+   default:
+      break;
+   }
+
+   return NULL;
+}
+
 static void
 gather_info_output_decl(const nir_shader *nir, const nir_variable *var,
-                        struct radv_shader_info *info, const struct radv_shader_variant_key *key)
+                        struct radv_shader_info *info)
 {
-   struct radv_vs_output_info *vs_info = NULL;
+   struct radv_vs_output_info *vs_info = get_vs_output_info(nir, info);
 
    switch (nir->info.stage) {
    case MESA_SHADER_FRAGMENT:
       gather_info_output_decl_ps(nir, var, info);
       break;
    case MESA_SHADER_VERTEX:
-      if (!key->vs_common_out.as_ls && !key->vs_common_out.as_es)
-         vs_info = &info->vs.outinfo;
-
-      /* TODO: Adjust as_ls/as_nng. */
-      if (!key->vs_common_out.as_ls && key->vs_common_out.as_ngg)
-         gather_info_output_decl_gs(nir, var, info);
       break;
    case MESA_SHADER_GEOMETRY:
-      vs_info = &info->vs.outinfo;
       gather_info_output_decl_gs(nir, var, info);
       break;
    case MESA_SHADER_TESS_EVAL:
-      if (!key->vs_common_out.as_es)
-         vs_info = &info->tes.outinfo;
       break;
    default:
       break;
@@ -489,13 +540,22 @@ gather_info_output_decl(const nir_shader *nir, const nir_variable *var,
          vs_info->writes_pointsize = true;
          break;
       case VARYING_SLOT_VIEWPORT:
-         vs_info->writes_viewport_index = true;
+         if (var->data.per_primitive)
+            vs_info->writes_viewport_index_per_primitive = true;
+         else
+            vs_info->writes_viewport_index = true;
          break;
       case VARYING_SLOT_LAYER:
-         vs_info->writes_layer = true;
+         if (var->data.per_primitive)
+            vs_info->writes_layer_per_primitive = true;
+         else
+            vs_info->writes_layer = true;
          break;
       case VARYING_SLOT_PRIMITIVE_SHADING_RATE:
-         vs_info->writes_primitive_shading_rate = true;
+         if (var->data.per_primitive)
+            vs_info->writes_primitive_shading_rate_per_primitive = true;
+         else
+            vs_info->writes_primitive_shading_rate = true;
          break;
       default:
          break;
@@ -534,18 +594,40 @@ gather_xfb_info(const nir_shader *nir, struct radv_shader_info *info)
    ralloc_free(xfb);
 }
 
+static void
+assign_outinfo_param(struct radv_vs_output_info *outinfo, gl_varying_slot idx,
+                     unsigned *total_param_exports)
+{
+   if (outinfo->vs_output_param_offset[idx] == AC_EXP_PARAM_UNDEFINED)
+      outinfo->vs_output_param_offset[idx] = (*total_param_exports)++;
+}
+
+static void
+assign_outinfo_params(struct radv_vs_output_info *outinfo, uint64_t mask,
+                      unsigned *total_param_exports)
+{
+   u_foreach_bit64(idx, mask) {
+      if (idx >= VARYING_SLOT_VAR0 || idx == VARYING_SLOT_LAYER ||
+          idx == VARYING_SLOT_PRIMITIVE_ID || idx == VARYING_SLOT_VIEWPORT ||
+          ((idx == VARYING_SLOT_CLIP_DIST0 || idx == VARYING_SLOT_CLIP_DIST1) &&
+           outinfo->export_clip_dists))
+         assign_outinfo_param(outinfo, idx, total_param_exports);
+   }
+}
+
 void
 radv_nir_shader_info_init(struct radv_shader_info *info)
 {
    /* Assume that shaders only have 32-bit push constants by default. */
-   info->min_push_constant_used = UINT8_MAX;
+   info->min_push_constant_used = UINT16_MAX;
    info->has_only_32bit_push_constants = true;
 }
 
 void
 radv_nir_shader_info_pass(struct radv_device *device, const struct nir_shader *nir,
                           const struct radv_pipeline_layout *layout,
-                          const struct radv_shader_variant_key *key, struct radv_shader_info *info)
+                          const struct radv_pipeline_key *pipeline_key,
+                          struct radv_shader_info *info)
 {
    struct nir_function *func = (struct nir_function *)exec_list_get_head_const(&nir->functions);
 
@@ -556,96 +638,108 @@ radv_nir_shader_info_pass(struct radv_device *device, const struct nir_shader *n
    }
 
    if (nir->info.stage == MESA_SHADER_VERTEX) {
+      if (pipeline_key->vs.dynamic_input_state && nir->info.inputs_read) {
+         info->vs.has_prolog = true;
+         info->vs.dynamic_inputs = true;
+      }
+
       /* Use per-attribute vertex descriptors to prevent faults and
        * for correct bounds checking.
        */
-      info->vs.use_per_attribute_vb_descs = device->robust_buffer_access;
+      info->vs.use_per_attribute_vb_descs = device->robust_buffer_access || info->vs.dynamic_inputs;
    }
 
+   /* We have to ensure consistent input register assignments between the main shader and the
+    * prolog. */
+   info->vs.needs_instance_id |= info->vs.has_prolog;
+   info->vs.needs_base_instance |= info->vs.has_prolog;
+   info->vs.needs_draw_id |= info->vs.has_prolog;
+
    nir_foreach_shader_in_variable (variable, nir)
-      gather_info_input_decl(nir, variable, info, key);
+      gather_info_input_decl(nir, variable, pipeline_key, info);
 
    nir_foreach_block (block, func->impl) {
       gather_info_block(nir, block, info);
    }
 
-   nir_foreach_shader_out_variable(variable, nir) gather_info_output_decl(nir, variable, info, key);
+   nir_foreach_shader_out_variable(variable, nir) gather_info_output_decl(nir, variable, info);
 
    if (nir->info.stage == MESA_SHADER_VERTEX || nir->info.stage == MESA_SHADER_TESS_EVAL ||
        nir->info.stage == MESA_SHADER_GEOMETRY)
       gather_xfb_info(nir, info);
 
-   /* Make sure to export the LayerID if the fragment shader needs it. */
-   if (key->vs_common_out.export_layer_id) {
-      switch (nir->info.stage) {
-      case MESA_SHADER_VERTEX:
-         info->vs.output_usage_mask[VARYING_SLOT_LAYER] |= 0x1;
-         break;
-      case MESA_SHADER_TESS_EVAL:
-         info->tes.output_usage_mask[VARYING_SLOT_LAYER] |= 0x1;
-         break;
-      case MESA_SHADER_GEOMETRY:
-         info->gs.output_usage_mask[VARYING_SLOT_LAYER] |= 0x1;
-         break;
-      default:
-         break;
+   struct radv_vs_output_info *outinfo = get_vs_output_info(nir, info);
+   if (outinfo) {
+      /* Make sure to export the LayerID if the subpass has multiviews. */
+      if (pipeline_key->has_multiview_view_index) {
+         if (nir->info.stage == MESA_SHADER_MESH)
+            outinfo->writes_layer_per_primitive = true;
+         else
+            outinfo->writes_layer = true;
       }
+
+      /* VS/TES/GS: shading rate is per-vertex, MS: it's per-primitive. */
+      bool force_vrs_per_vertex =
+         device->force_vrs != RADV_FORCE_VRS_NONE && nir->info.stage != MESA_SHADER_MESH;
+      bool writes_primitive_shading_rate =
+         outinfo->writes_primitive_shading_rate || force_vrs_per_vertex;
+      int pos_written = 0x1;
+
+      if (outinfo->writes_pointsize || outinfo->writes_viewport_index || outinfo->writes_layer ||
+          writes_primitive_shading_rate)
+         pos_written |= 1 << 1;
+
+      unsigned num_clip_distances = util_bitcount(outinfo->clip_dist_mask);
+      unsigned num_cull_distances = util_bitcount(outinfo->cull_dist_mask);
+
+      if (num_clip_distances + num_cull_distances > 0)
+         pos_written |= 1 << 2;
+      if (num_clip_distances + num_cull_distances > 4)
+         pos_written |= 1 << 3;
+
+      outinfo->pos_exports = util_bitcount(pos_written);
+
+      memset(outinfo->vs_output_param_offset, AC_EXP_PARAM_UNDEFINED,
+             sizeof(outinfo->vs_output_param_offset));
+
+      uint64_t per_prim_mask =
+         nir->info.outputs_written & nir->info.per_primitive_outputs &
+         ~BITFIELD64_BIT(VARYING_SLOT_PRIMITIVE_INDICES) & ~BITFIELD64_BIT(VARYING_SLOT_PRIMITIVE_COUNT);
+      uint64_t per_vtx_mask =
+         nir->info.outputs_written & ~per_prim_mask;
+
+      unsigned total_param_exports = 0;
+
+      /* Per-vertex outputs */
+      assign_outinfo_params(outinfo, per_vtx_mask, &total_param_exports);
+      if (outinfo->writes_layer)
+         assign_outinfo_param(outinfo, VARYING_SLOT_LAYER, &total_param_exports);
+      if (outinfo->export_prim_id)
+         assign_outinfo_param(outinfo, VARYING_SLOT_PRIMITIVE_ID, &total_param_exports);
+
+      outinfo->param_exports = total_param_exports;
+
+      /* Per-primitive outputs: the HW needs these to be last. */
+      assign_outinfo_params(outinfo, per_prim_mask, &total_param_exports);
+      if (outinfo->writes_layer_per_primitive)
+         assign_outinfo_param(outinfo, VARYING_SLOT_LAYER, &total_param_exports);
+      if (outinfo->writes_viewport_index_per_primitive)
+         assign_outinfo_param(outinfo, VARYING_SLOT_VIEWPORT, &total_param_exports);
+      if (outinfo->export_prim_id_per_primitive)
+         assign_outinfo_param(outinfo, VARYING_SLOT_PRIMITIVE_ID, &total_param_exports);
+
+      outinfo->prim_param_exports = total_param_exports - outinfo->param_exports;
    }
 
-   /* Make sure to export the LayerID if the subpass has multiviews. */
-   if (key->has_multiview_view_index) {
-      switch (nir->info.stage) {
-      case MESA_SHADER_VERTEX:
-         info->vs.outinfo.writes_layer = true;
-         break;
-      case MESA_SHADER_TESS_EVAL:
-         info->tes.outinfo.writes_layer = true;
-         break;
-      case MESA_SHADER_GEOMETRY:
-         info->vs.outinfo.writes_layer = true;
-         break;
-      default:
-         break;
-      }
-   }
+   if (nir->info.stage == MESA_SHADER_FRAGMENT) {
+      uint64_t per_primitive_input_mask = nir->info.inputs_read & nir->info.per_primitive_inputs;
+      unsigned num_per_primitive_inputs = util_bitcount64(per_primitive_input_mask);
+      assert(num_per_primitive_inputs <= nir->num_inputs);
 
-   /* Make sure to export the PrimitiveID if the fragment shader needs it. */
-   if (key->vs_common_out.export_prim_id) {
-      switch (nir->info.stage) {
-      case MESA_SHADER_VERTEX:
-         info->vs.outinfo.export_prim_id = true;
-         break;
-      case MESA_SHADER_TESS_EVAL:
-         info->tes.outinfo.export_prim_id = true;
-         break;
-      case MESA_SHADER_GEOMETRY:
-         info->vs.outinfo.export_prim_id = true;
-         break;
-      default:
-         break;
-      }
+      info->ps.num_interp = nir->num_inputs - num_per_primitive_inputs;
+      info->ps.num_prim_interp = num_per_primitive_inputs;
    }
 
-   /* Make sure to export the ViewportIndex if the fragment shader needs it. */
-   if (key->vs_common_out.export_viewport_index) {
-      switch (nir->info.stage) {
-      case MESA_SHADER_VERTEX:
-         info->vs.output_usage_mask[VARYING_SLOT_VIEWPORT] |= 0x1;
-         break;
-      case MESA_SHADER_TESS_EVAL:
-         info->tes.output_usage_mask[VARYING_SLOT_VIEWPORT] |= 0x1;
-         break;
-      case MESA_SHADER_GEOMETRY:
-         info->gs.output_usage_mask[VARYING_SLOT_VIEWPORT] |= 0x1;
-         break;
-      default:
-         break;
-      }
-   }
-
-   if (nir->info.stage == MESA_SHADER_FRAGMENT)
-      info->ps.num_interp = nir->num_inputs;
-
    switch (nir->info.stage) {
    case MESA_SHADER_COMPUTE:
       for (int i = 0; i < 3; ++i)
@@ -665,24 +759,18 @@ radv_nir_shader_info_pass(struct radv_device *device, const struct nir_shader *n
       info->gs.invocations = nir->info.gs.invocations;
       break;
    case MESA_SHADER_TESS_EVAL:
-      info->tes.primitive_mode = nir->info.tess.primitive_mode;
+      info->tes._primitive_mode = nir->info.tess._primitive_mode;
       info->tes.spacing = nir->info.tess.spacing;
       info->tes.ccw = nir->info.tess.ccw;
       info->tes.point_mode = nir->info.tess.point_mode;
-      info->tes.as_es = key->vs_common_out.as_es;
-      info->tes.export_prim_id = key->vs_common_out.export_prim_id;
-      info->is_ngg = key->vs_common_out.as_ngg;
-      info->is_ngg_passthrough = key->vs_common_out.as_ngg_passthrough;
       break;
    case MESA_SHADER_TESS_CTRL:
       info->tcs.tcs_vertices_out = nir->info.tess.tcs_vertices_out;
       break;
    case MESA_SHADER_VERTEX:
-      info->vs.as_es = key->vs_common_out.as_es;
-      info->vs.as_ls = key->vs_common_out.as_ls;
-      info->vs.export_prim_id = key->vs_common_out.export_prim_id;
-      info->is_ngg = key->vs_common_out.as_ngg;
-      info->is_ngg_passthrough = key->vs_common_out.as_ngg_passthrough;
+      break;
+   case MESA_SHADER_MESH:
+      info->ms.output_prim = nir->info.mesh.primitive_type;
       break;
    default:
       break;
@@ -696,8 +784,8 @@ radv_nir_shader_info_pass(struct radv_device *device, const struct nir_shader *n
    }
 
    /* Compute the ESGS item size for VS or TES as ES. */
-   if ((nir->info.stage == MESA_SHADER_VERTEX || nir->info.stage == MESA_SHADER_TESS_EVAL) &&
-       key->vs_common_out.as_es) {
+   if ((nir->info.stage == MESA_SHADER_VERTEX && info->vs.as_es) ||
+       (nir->info.stage == MESA_SHADER_TESS_EVAL && info->tes.as_es)) {
       struct radv_es_output_info *es_info =
          nir->info.stage == MESA_SHADER_VERTEX ? &info->vs.es_info : &info->tes.es_info;
       uint32_t num_outputs_written = nir->info.stage == MESA_SHADER_VERTEX
@@ -706,11 +794,16 @@ radv_nir_shader_info_pass(struct radv_device *device, const struct nir_shader *n
       es_info->esgs_itemsize = num_outputs_written * 16;
    }
 
-   info->float_controls_mode = nir->info.float_controls_execution_mode;
-
    if (nir->info.stage == MESA_SHADER_FRAGMENT) {
+      bool uses_persp_or_linear_interp = info->ps.reads_persp_center ||
+                                         info->ps.reads_persp_centroid ||
+                                         info->ps.reads_persp_sample ||
+                                         info->ps.reads_linear_center ||
+                                         info->ps.reads_linear_centroid ||
+                                         info->ps.reads_linear_sample;
+
       info->ps.allow_flat_shading =
-         !(info->ps.uses_persp_or_linear_interp || info->ps.needs_sample_positions ||
+         !(uses_persp_or_linear_interp || info->ps.needs_sample_positions ||
            info->ps.writes_memory || nir->info.fs.needs_quad_helper_invocations ||
            BITSET_TEST(nir->info.system_values_read, SYSTEM_VALUE_FRAG_COORD) ||
            BITSET_TEST(nir->info.system_values_read, SYSTEM_VALUE_POINT_COORD) ||
@@ -718,5 +811,7 @@ radv_nir_shader_info_pass(struct radv_device *device, const struct nir_shader *n
            BITSET_TEST(nir->info.system_values_read, SYSTEM_VALUE_SAMPLE_POS) ||
            BITSET_TEST(nir->info.system_values_read, SYSTEM_VALUE_SAMPLE_MASK_IN) ||
            BITSET_TEST(nir->info.system_values_read, SYSTEM_VALUE_HELPER_INVOCATION));
+
+      info->ps.spi_ps_input = radv_compute_spi_ps_input(device, info);
    }
 }
