@@ -235,6 +235,25 @@ agx_pack_cmpsel_src(agx_index src, enum agx_size dest_size)
    }
 }
 
+static unsigned
+agx_pack_sample_mask_src(agx_index src)
+{
+   unsigned value = src.value;
+   unsigned packed_value =
+         (value & BITFIELD_MASK(6)) |
+         (((value >> 6) & BITFIELD_MASK(2)) << 10);
+
+   if (src.type == AGX_INDEX_IMMEDIATE) {
+      assert(value < 0x100);
+      return packed_value | (1 << 7);
+   } else {
+      assert(src.type == AGX_INDEX_REGISTER);
+      assert(!(src.cache && src.discard));
+
+      return packed_value;
+   }
+}
+
 static unsigned
 agx_pack_float_mod(agx_index src)
 {
@@ -412,6 +431,21 @@ agx_pack_instr(struct util_dynarray *emission, struct util_dynarray *fixups, agx
       break;
    }
 
+   case AGX_OPCODE_SAMPLE_MASK:
+   {
+      unsigned S = agx_pack_sample_mask_src(I->src[0]);
+      uint64_t raw =
+         0x7fc1 |
+         ((S & 0xff) << 16) |
+         (0x3 << 24) |
+         ((S >> 8) << 26) |
+         (0x158ull << 32);
+
+      unsigned size = 8;
+      memcpy(util_dynarray_grow_bytes(emission, 1, size), &raw, size);
+      break;
+   }
+
    case AGX_OPCODE_LD_VARY:
    case AGX_OPCODE_LD_VARY_FLAT:
    {
@@ -526,13 +560,13 @@ agx_pack_instr(struct util_dynarray *emission, struct util_dynarray *fixups, agx
       unsigned q1 = 0; // XXX
       unsigned q2 = 0; // XXX
       unsigned q3 = 12; // XXX
-      unsigned q4 = 1; // XXX
+      unsigned kill = 0; // helper invocation kill bit
       unsigned q5 = 0; // XXX
       unsigned q6 = 0; // XXX
 
       uint32_t extend =
             ((U & BITFIELD_MASK(5)) << 0) |
-            (q4 << 5) |
+            (kill << 5) |
             ((R >> 6) << 8) |
             ((C >> 6) << 10) |
             ((D >> 6) << 12) |
