@@ -26,7 +26,52 @@
  */
 
 #include "v3dv_private.h"
-#include "vk_format_info.h"
+
+/* The only version specific structure that we need is
+ * TMU_CONFIG_PARAMETER_1. This didn't seem to change significantly from
+ * previous V3D versions and we don't expect that to change, so for now let's
+ * just hardcode the V3D version here.
+ */
+#define V3D_VERSION 41
+#include "broadcom/common/v3d_macros.h"
+#include "broadcom/cle/v3dx_pack.h"
+
+/* Our Vulkan resource indices represent indices in descriptor maps which
+ * include all shader stages, so we need to size the arrays below
+ * accordingly. For now we only support a maximum of 3 stages: VS, GS, FS.
+ */
+#define MAX_STAGES 3
+
+#define MAX_TOTAL_TEXTURE_SAMPLERS (V3D_MAX_TEXTURE_SAMPLERS * MAX_STAGES)
+struct texture_bo_list {
+   struct v3dv_bo *tex[MAX_TOTAL_TEXTURE_SAMPLERS];
+};
+
+/* This tracks state BOs for both textures and samplers, so we
+ * multiply by 2.
+ */
+#define MAX_TOTAL_STATES (2 * V3D_MAX_TEXTURE_SAMPLERS * MAX_STAGES)
+struct state_bo_list {
+   uint32_t count;
+   struct v3dv_bo *states[MAX_TOTAL_STATES];
+};
+
+#define MAX_TOTAL_UNIFORM_BUFFERS (1 + MAX_UNIFORM_BUFFERS * MAX_STAGES)
+#define MAX_TOTAL_STORAGE_BUFFERS (MAX_STORAGE_BUFFERS * MAX_STAGES)
+struct buffer_bo_list {
+   struct v3dv_bo *ubo[MAX_TOTAL_UNIFORM_BUFFERS];
+   struct v3dv_bo *ssbo[MAX_TOTAL_STORAGE_BUFFERS];
+};
+
+static bool
+state_bo_in_list(struct state_bo_list *list, struct v3dv_bo *bo)
+{
+   for (int i = 0; i < list->count; i++) {
+      if (list->states[i] == bo)
+         return true;
+   }
+   return false;
+}
 
 /*
  * This method checks if the ubo used for push constants is needed to be
@@ -40,10 +85,11 @@
  * we need to rely on a UBO.
  */
 static void
-check_push_constants_ubo(struct v3dv_cmd_buffer *cmd_buffer)
+check_push_constants_ubo(struct v3dv_cmd_buffer *cmd_buffer,
+                         struct v3dv_pipeline *pipeline)
 {
    if (!(cmd_buffer->state.dirty & V3DV_CMD_DIRTY_PUSH_CONSTANTS) ||
-       cmd_buffer->state.pipeline->layout->push_constant_size == 0)
+       pipeline->layout->push_constant_size == 0)
       return;
 
    if (cmd_buffer->push_constants_resource.bo == NULL) {
@@ -86,62 +132,71 @@ check_push_constants_ubo(struct v3dv_cmd_buffer *cmd_buffer)
 static void
 write_tmu_p0(struct v3dv_cmd_buffer *cmd_buffer,
              struct v3dv_pipeline *pipeline,
+             enum broadcom_shader_stage stage,
              struct v3dv_cl_out **uniforms,
-             uint32_t data)
+             uint32_t data,
+             struct texture_bo_list *tex_bos,
+             struct state_bo_list *state_bos)
 {
-   int unit = v3d_unit_data_get_unit(data);
-   uint32_t texture_idx;
-   struct v3dv_job *job = cmd_buffer->state.job;
-   struct v3dv_descriptor_state *descriptor_state =
-      &cmd_buffer->state.descriptor_state[v3dv_pipeline_get_binding_point(pipeline)];
+   uint32_t texture_idx = v3d_unit_data_get_unit(data);
 
-   v3dv_pipeline_combined_index_key_unpack(pipeline->combined_index_to_key_map[unit],
-                                           &texture_idx,
-                                           NULL);
+   struct v3dv_descriptor_state *descriptor_state =
+      v3dv_cmd_buffer_get_descriptor_state(cmd_buffer, pipeline);
 
    /* We need to ensure that the texture bo is added to the job */
    struct v3dv_bo *texture_bo =
-      v3dv_descriptor_map_get_texture_bo(descriptor_state, &pipeline->texture_map,
+      v3dv_descriptor_map_get_texture_bo(descriptor_state,
+                                         &pipeline->shared_data->maps[stage]->texture_map,
                                          pipeline->layout, texture_idx);
    assert(texture_bo);
-   v3dv_job_add_bo(job, texture_bo);
+   assert(texture_idx < V3D_MAX_TEXTURE_SAMPLERS);
+   tex_bos->tex[texture_idx] = texture_bo;
 
    struct v3dv_cl_reloc state_reloc =
-      v3dv_descriptor_map_get_texture_shader_state(descriptor_state,
-                                                   &pipeline->texture_map,
+      v3dv_descriptor_map_get_texture_shader_state(cmd_buffer->device, descriptor_state,
+                                                   &pipeline->shared_data->maps[stage]->texture_map,
                                                    pipeline->layout,
                                                    texture_idx);
 
-   cl_aligned_reloc(&job->indirect, uniforms,
-                    state_reloc.bo,
-                    state_reloc.offset +
-                    v3d_unit_data_get_offset(data));
+   cl_aligned_u32(uniforms, state_reloc.bo->offset +
+                            state_reloc.offset +
+                            v3d_unit_data_get_offset(data));
+
+   /* Texture and Sampler states are typically suballocated, so they are
+    * usually the same BO: only flag them once to avoid trying to add them
+    * multiple times to the job later.
+    */
+   if (!state_bo_in_list(state_bos, state_reloc.bo)) {
+      assert(state_bos->count < 2 * V3D_MAX_TEXTURE_SAMPLERS);
+      state_bos->states[state_bos->count++] = state_reloc.bo;
+   }
 }
 
 /** V3D 4.x TMU configuration parameter 1 (sampler) */
 static void
 write_tmu_p1(struct v3dv_cmd_buffer *cmd_buffer,
              struct v3dv_pipeline *pipeline,
+             enum broadcom_shader_stage stage,
              struct v3dv_cl_out **uniforms,
-             uint32_t data)
+             uint32_t data,
+             struct state_bo_list *state_bos)
 {
-   uint32_t unit = v3d_unit_data_get_unit(data);
-   uint32_t sampler_idx;
-   struct v3dv_job *job = cmd_buffer->state.job;
+   uint32_t sampler_idx = v3d_unit_data_get_unit(data);
    struct v3dv_descriptor_state *descriptor_state =
-      &cmd_buffer->state.descriptor_state[v3dv_pipeline_get_binding_point(pipeline)];
+      v3dv_cmd_buffer_get_descriptor_state(cmd_buffer, pipeline);
 
-   v3dv_pipeline_combined_index_key_unpack(pipeline->combined_index_to_key_map[unit],
-                                           NULL, &sampler_idx);
-   assert(sampler_idx != V3DV_NO_SAMPLER_IDX);
+   assert(sampler_idx != V3DV_NO_SAMPLER_16BIT_IDX &&
+          sampler_idx != V3DV_NO_SAMPLER_32BIT_IDX);
 
    struct v3dv_cl_reloc sampler_state_reloc =
-      v3dv_descriptor_map_get_sampler_state(descriptor_state, &pipeline->sampler_map,
+      v3dv_descriptor_map_get_sampler_state(cmd_buffer->device, descriptor_state,
+                                            &pipeline->shared_data->maps[stage]->sampler_map,
                                             pipeline->layout, sampler_idx);
 
    const struct v3dv_sampler *sampler =
-      v3dv_descriptor_map_get_sampler(descriptor_state, &pipeline->sampler_map,
-                                         pipeline->layout, sampler_idx);
+      v3dv_descriptor_map_get_sampler(descriptor_state,
+                                      &pipeline->shared_data->maps[stage]->sampler_map,
+                                      pipeline->layout, sampler_idx);
    assert(sampler);
 
    /* Set unnormalized coordinates flag from sampler object */
@@ -154,26 +209,36 @@ write_tmu_p1(struct v3dv_cmd_buffer *cmd_buffer,
                                         &p1_unpacked);
    }
 
-   cl_aligned_reloc(&job->indirect, uniforms,
-                    sampler_state_reloc.bo,
-                    sampler_state_reloc.offset +
-                    p1_packed);
+   cl_aligned_u32(uniforms, sampler_state_reloc.bo->offset +
+                            sampler_state_reloc.offset +
+                            p1_packed);
+
+   /* Texture and Sampler states are typically suballocated, so they are
+    * usually the same BO: only flag them once to avoid trying to add them
+    * multiple times to the job later.
+    */
+   if (!state_bo_in_list(state_bos, sampler_state_reloc.bo)) {
+      assert(state_bos->count < 2 * V3D_MAX_TEXTURE_SAMPLERS);
+      state_bos->states[state_bos->count++] = sampler_state_reloc.bo;
+   }
 }
 
 static void
 write_ubo_ssbo_uniforms(struct v3dv_cmd_buffer *cmd_buffer,
                         struct v3dv_pipeline *pipeline,
+                        enum broadcom_shader_stage stage,
                         struct v3dv_cl_out **uniforms,
                         enum quniform_contents content,
-                        uint32_t data)
+                        uint32_t data,
+                        struct buffer_bo_list *buffer_bos)
 {
-   struct v3dv_job *job = cmd_buffer->state.job;
    struct v3dv_descriptor_state *descriptor_state =
-      &cmd_buffer->state.descriptor_state[v3dv_pipeline_get_binding_point(pipeline)];
+      v3dv_cmd_buffer_get_descriptor_state(cmd_buffer, pipeline);
 
    struct v3dv_descriptor_map *map =
       content == QUNIFORM_UBO_ADDR || content == QUNIFORM_GET_UBO_SIZE ?
-      &pipeline->ubo_map : &pipeline->ssbo_map;
+      &pipeline->shared_data->maps[stage]->ubo_map :
+      &pipeline->shared_data->maps[stage]->ssbo_map;
 
    uint32_t offset =
       content == QUNIFORM_UBO_ADDR ?
@@ -190,16 +255,16 @@ write_ubo_ssbo_uniforms(struct v3dv_cmd_buffer *cmd_buffer,
        * updated. It already take into account it is should do the
        * update or not
        */
-      check_push_constants_ubo(cmd_buffer);
+      check_push_constants_ubo(cmd_buffer, pipeline);
 
-      struct v3dv_resource *resource =
+      struct v3dv_cl_reloc *resource =
          &cmd_buffer->push_constants_resource;
       assert(resource->bo);
 
-      cl_aligned_reloc(&job->indirect, uniforms,
-                       resource->bo,
-                       resource->offset + offset + dynamic_offset);
-
+      cl_aligned_u32(uniforms, resource->bo->offset +
+                               resource->offset +
+                               offset + dynamic_offset);
+      buffer_bos->ubo[0] = resource->bo;
    } else {
       uint32_t index =
          content == QUNIFORM_UBO_ADDR ?
@@ -219,10 +284,18 @@ write_ubo_ssbo_uniforms(struct v3dv_cmd_buffer *cmd_buffer,
           content == QUNIFORM_GET_UBO_SIZE) {
          cl_aligned_u32(uniforms, descriptor->range);
       } else {
-         cl_aligned_reloc(&job->indirect, uniforms,
-                          descriptor->buffer->mem->bo,
-                          descriptor->buffer->mem_offset +
-                          descriptor->offset + offset + dynamic_offset);
+         cl_aligned_u32(uniforms, descriptor->buffer->mem->bo->offset +
+                                  descriptor->buffer->mem_offset +
+                                  descriptor->offset +
+                                  offset + dynamic_offset);
+
+         if (content == QUNIFORM_UBO_ADDR) {
+            assert(index + 1 < MAX_TOTAL_UNIFORM_BUFFERS);
+            buffer_bos->ubo[index + 1] = descriptor->buffer->mem->bo;
+         } else {
+            assert(index < MAX_TOTAL_STORAGE_BUFFERS);
+            buffer_bos->ssbo[index] = descriptor->buffer->mem->bo;
+         }
       }
    }
 }
@@ -238,26 +311,26 @@ get_texture_size_from_image_view(struct v3dv_image_view *image_view,
       /* We don't u_minify the values, as we are using the image_view
        * extents
        */
-      return image_view->extent.width;
+      return image_view->vk.extent.width;
    case QUNIFORM_IMAGE_HEIGHT:
    case QUNIFORM_TEXTURE_HEIGHT:
-      return image_view->extent.height;
+      return image_view->vk.extent.height;
    case QUNIFORM_IMAGE_DEPTH:
    case QUNIFORM_TEXTURE_DEPTH:
-      return image_view->extent.depth;
+      return image_view->vk.extent.depth;
    case QUNIFORM_IMAGE_ARRAY_SIZE:
    case QUNIFORM_TEXTURE_ARRAY_SIZE:
-      if (image_view->type != VK_IMAGE_VIEW_TYPE_CUBE_ARRAY) {
-         return image_view->last_layer - image_view->first_layer + 1;
+      if (image_view->vk.view_type != VK_IMAGE_VIEW_TYPE_CUBE_ARRAY) {
+         return image_view->vk.layer_count;
       } else {
-         assert((image_view->last_layer - image_view->first_layer + 1) % 6 == 0);
-         return (image_view->last_layer - image_view->first_layer + 1) / 6;
+         assert(image_view->vk.layer_count % 6 == 0);
+         return image_view->vk.layer_count / 6;
       }
    case QUNIFORM_TEXTURE_LEVELS:
-      return image_view->max_level - image_view->base_level + 1;
+      return image_view->vk.level_count;
    case QUNIFORM_TEXTURE_SAMPLES:
-      assert(image_view->image);
-      return image_view->image->samples;
+      assert(image_view->vk.image);
+      return image_view->vk.image->samples;
    default:
       unreachable("Bad texture size field");
    }
@@ -282,21 +355,18 @@ get_texture_size_from_buffer_view(struct v3dv_buffer_view *buffer_view,
 static uint32_t
 get_texture_size(struct v3dv_cmd_buffer *cmd_buffer,
                  struct v3dv_pipeline *pipeline,
+                 enum broadcom_shader_stage stage,
                  enum quniform_contents contents,
                  uint32_t data)
 {
-   int unit = v3d_unit_data_get_unit(data);
-   uint32_t texture_idx;
-   struct v3dv_descriptor_state *descriptor_state =
-      &cmd_buffer->state.descriptor_state[v3dv_pipeline_get_binding_point(pipeline)];
+   uint32_t texture_idx = data;
 
-   v3dv_pipeline_combined_index_key_unpack(pipeline->combined_index_to_key_map[unit],
-                                           &texture_idx,
-                                           NULL);
+   struct v3dv_descriptor_state *descriptor_state =
+      v3dv_cmd_buffer_get_descriptor_state(cmd_buffer, pipeline);
 
    struct v3dv_descriptor *descriptor =
       v3dv_descriptor_map_get_descriptor(descriptor_state,
-                                         &pipeline->texture_map,
+                                         &pipeline->shared_data->maps[stage]->texture_map,
                                          pipeline->layout,
                                          texture_idx, NULL);
 
@@ -320,16 +390,21 @@ get_texture_size(struct v3dv_cmd_buffer *cmd_buffer,
 
 struct v3dv_cl_reloc
 v3dv_write_uniforms_wg_offsets(struct v3dv_cmd_buffer *cmd_buffer,
-                               struct v3dv_pipeline_stage *p_stage,
+                               struct v3dv_pipeline *pipeline,
+                               struct v3dv_shader_variant *variant,
                                uint32_t **wg_count_offsets)
 {
    struct v3d_uniform_list *uinfo =
-      &p_stage->current_variant->prog_data.base->uniforms;
+      &variant->prog_data.base->uniforms;
    struct v3dv_dynamic_state *dynamic = &cmd_buffer->state.dynamic;
-   struct v3dv_pipeline *pipeline = p_stage->pipeline;
 
    struct v3dv_job *job = cmd_buffer->state.job;
    assert(job);
+   assert(job->cmd_buffer == cmd_buffer);
+
+   struct texture_bo_list tex_bos = { 0 };
+   struct state_bo_list state_bos = { 0 };
+   struct buffer_bo_list buffer_bos = { 0 };
 
    /* The hardware always pre-fetches the next uniform (also when there
     * aren't any), so we always allocate space for an extra slot. This
@@ -354,7 +429,6 @@ v3dv_write_uniforms_wg_offsets(struct v3dv_cmd_buffer *cmd_buffer,
          break;
 
       case QUNIFORM_UNIFORM:
-         assert(pipeline->use_push_constants);
          cl_aligned_u32(&uniforms, cmd_buffer->push_constants_data[data]);
          break;
 
@@ -378,17 +452,20 @@ v3dv_write_uniforms_wg_offsets(struct v3dv_cmd_buffer *cmd_buffer,
       case QUNIFORM_UBO_ADDR:
       case QUNIFORM_GET_SSBO_SIZE:
       case QUNIFORM_GET_UBO_SIZE:
-         write_ubo_ssbo_uniforms(cmd_buffer, pipeline, &uniforms,
-                                 uinfo->contents[i], data);
+         write_ubo_ssbo_uniforms(cmd_buffer, pipeline, variant->stage, &uniforms,
+                                 uinfo->contents[i], data, &buffer_bos);
+
         break;
 
       case QUNIFORM_IMAGE_TMU_CONFIG_P0:
       case QUNIFORM_TMU_CONFIG_P0:
-         write_tmu_p0(cmd_buffer, pipeline, &uniforms, data);
+         write_tmu_p0(cmd_buffer, pipeline, variant->stage,
+                      &uniforms, data, &tex_bos, &state_bos);
          break;
 
       case QUNIFORM_TMU_CONFIG_P1:
-         write_tmu_p1(cmd_buffer, pipeline, &uniforms, data);
+         write_tmu_p1(cmd_buffer, pipeline, variant->stage,
+                      &uniforms, data, &state_bos);
          break;
 
       case QUNIFORM_IMAGE_WIDTH:
@@ -404,10 +481,66 @@ v3dv_write_uniforms_wg_offsets(struct v3dv_cmd_buffer *cmd_buffer,
          cl_aligned_u32(&uniforms,
                         get_texture_size(cmd_buffer,
                                          pipeline,
+                                         variant->stage,
                                          uinfo->contents[i],
                                          data));
          break;
 
+      /* We generate this from geometry shaders to cap the generated gl_Layer
+       * to be within the number of layers of the framebuffer so we prevent the
+       * binner from trying to access tile state memory out of bounds (for
+       * layers that don't exist).
+       *
+       * Unfortunately, for secondary command buffers we may not know the
+       * number of layers in the framebuffer at this stage. Since we are
+       * only using this to sanitize the shader and it should not have any
+       * impact on correct shaders that emit valid values for gl_Layer,
+       * we just work around it by using the largest number of layers we
+       * support.
+       *
+       * FIXME: we could do better than this by recording in the job that
+       * the value at this uniform offset is not correct, and patch it when
+       * we execute the secondary command buffer into a primary, since we do
+       * have the correct number of layers at that point, but again, since this
+       * is only for sanityzing the shader and it only affects the specific case
+       * of secondary command buffers without framebuffer info available it
+       * might not be worth the trouble.
+       *
+       * With multiview the number of layers is dictated by the view mask
+       * and not by the framebuffer layers. We do set the job's frame tiling
+       * information correctly from the view mask in that case, however,
+       * secondary command buffers may not have valid frame tiling data,
+       * so when multiview is enabled, we always set the number of layers
+       * from the subpass view mask.
+       */
+      case QUNIFORM_FB_LAYERS: {
+         const struct v3dv_cmd_buffer_state *state = &job->cmd_buffer->state;
+         const uint32_t view_mask =
+            state->pass->subpasses[state->subpass_idx].view_mask;
+
+         uint32_t num_layers;
+         if (view_mask != 0) {
+            num_layers = util_last_bit(view_mask);
+         } else if (job->frame_tiling.layers != 0) {
+            num_layers = job->frame_tiling.layers;
+         } else if (cmd_buffer->state.framebuffer) {
+            num_layers = cmd_buffer->state.framebuffer->layers;
+         } else {
+            assert(cmd_buffer->level == VK_COMMAND_BUFFER_LEVEL_SECONDARY);
+            num_layers = 2048;
+#if DEBUG
+            fprintf(stderr, "Skipping gl_LayerID shader sanity check for "
+                            "secondary command buffer\n");
+#endif
+         }
+         cl_aligned_u32(&uniforms, num_layers);
+         break;
+      }
+
+      case QUNIFORM_VIEW_INDEX:
+         cl_aligned_u32(&uniforms, job->cmd_buffer->state.view_index);
+         break;
+
       case QUNIFORM_NUM_WORK_GROUPS:
          assert(job->type == V3DV_JOB_TYPE_GPU_CSD);
          assert(job->csd.wg_count[data] > 0);
@@ -416,15 +549,20 @@ v3dv_write_uniforms_wg_offsets(struct v3dv_cmd_buffer *cmd_buffer,
          cl_aligned_u32(&uniforms, job->csd.wg_count[data]);
          break;
 
+      case QUNIFORM_WORK_GROUP_BASE:
+         assert(job->type == V3DV_JOB_TYPE_GPU_CSD);
+         cl_aligned_u32(&uniforms, job->csd.wg_base[data]);
+         break;
+
       case QUNIFORM_SHARED_OFFSET:
          assert(job->type == V3DV_JOB_TYPE_GPU_CSD);
          assert(job->csd.shared_memory);
-         cl_aligned_reloc(&job->indirect, &uniforms, job->csd.shared_memory, 0);
+         cl_aligned_u32(&uniforms, job->csd.shared_memory->offset);
          break;
 
       case QUNIFORM_SPILL_OFFSET:
          assert(pipeline->spill.bo);
-         cl_aligned_reloc(&job->indirect, &uniforms, pipeline->spill.bo, 0);
+         cl_aligned_u32(&uniforms, pipeline->spill.bo->offset);
          break;
 
       case QUNIFORM_SPILL_SIZE_PER_THREAD:
@@ -439,12 +577,37 @@ v3dv_write_uniforms_wg_offsets(struct v3dv_cmd_buffer *cmd_buffer,
 
    cl_end(&job->indirect, uniforms);
 
+   for (int i = 0; i < MAX_TOTAL_TEXTURE_SAMPLERS; i++) {
+      if (tex_bos.tex[i])
+         v3dv_job_add_bo(job, tex_bos.tex[i]);
+   }
+
+   for (int i = 0; i < state_bos.count; i++)
+      v3dv_job_add_bo(job, state_bos.states[i]);
+
+   for (int i = 0; i < MAX_TOTAL_UNIFORM_BUFFERS; i++) {
+      if (buffer_bos.ubo[i])
+         v3dv_job_add_bo(job, buffer_bos.ubo[i]);
+   }
+
+   for (int i = 0; i < MAX_TOTAL_STORAGE_BUFFERS; i++) {
+      if (buffer_bos.ssbo[i])
+         v3dv_job_add_bo(job, buffer_bos.ssbo[i]);
+   }
+
+   if (job->csd.shared_memory)
+      v3dv_job_add_bo(job, job->csd.shared_memory);
+
+   if (pipeline->spill.bo)
+      v3dv_job_add_bo(job, pipeline->spill.bo);
+
    return uniform_stream;
 }
 
 struct v3dv_cl_reloc
 v3dv_write_uniforms(struct v3dv_cmd_buffer *cmd_buffer,
-                    struct v3dv_pipeline_stage *p_stage)
+                    struct v3dv_pipeline *pipeline,
+                    struct v3dv_shader_variant *variant)
 {
-   return v3dv_write_uniforms_wg_offsets(cmd_buffer, p_stage, NULL);
+   return v3dv_write_uniforms_wg_offsets(cmd_buffer, pipeline, variant, NULL);
 }
