@@ -438,6 +438,7 @@ ir_expression::ir_expression(int op, ir_rvalue *op0)
 
    case ir_unop_get_buffer_size:
    case ir_unop_ssbo_unsized_array_length:
+   case ir_unop_implicitly_sized_array_length:
       this->type = glsl_type::int_type;
       break;
 
@@ -1748,7 +1749,7 @@ ir_dereference::is_lvalue(const struct _mesa_glsl_parse_state *state) const
 {
    ir_variable *var = this->variable_referenced();
 
-   /* Every l-value derference chain eventually ends in a variable.
+   /* Every l-value dereference chain eventually ends in a variable.
     */
    if ((var == NULL) || var->data.read_only)
       return false;
@@ -1806,7 +1807,16 @@ ir_texture::set_sampler(ir_dereference *sampler, const glsl_type *type)
    assert(sampler != NULL);
    assert(type != NULL);
    this->sampler = sampler;
-   this->type = type;
+
+   if (this->is_sparse) {
+      /* code holds residency info */
+      glsl_struct_field fields[2] = {
+         glsl_struct_field(glsl_type::int_type, "code"),
+         glsl_struct_field(type, "texel"),
+      };
+      this->type = glsl_type::get_struct_instance(fields, 2, "struct");
+   } else
+      this->type = type;
 
    if (this->op == ir_txs || this->op == ir_query_levels ||
        this->op == ir_texture_samples) {
@@ -2025,6 +2035,7 @@ ir_variable::ir_variable(const struct glsl_type *type, const char *name,
    this->data.has_initializer = false;
    this->data.is_implicit_initializer = false;
    this->data.is_unmatched_generic_inout = false;
+   this->data.is_xfb = false;
    this->data.is_xfb_only = false;
    this->data.explicit_xfb_buffer = false;
    this->data.explicit_xfb_offset = false;
@@ -2072,6 +2083,7 @@ ir_variable::ir_variable(const struct glsl_type *type, const char *name,
    this->data.stream = 0;
    this->data.xfb_buffer = -1;
    this->data.xfb_stride = -1;
+   this->data.implicit_conversion_prohibited = false;
 
    this->interface_type = NULL;
 
