@@ -29,7 +29,7 @@
 #include "program/prog_statevars.h"
 #include "util/bitscan.h"
 #include "builtin_functions.h"
-#include "main/mtypes.h"
+#include "main/shader_types.h"
 
 using namespace ir_builder;
 
@@ -463,7 +463,9 @@ get_main(gl_linked_shader *sh)
 bool
 lower_blend_equation_advanced(struct gl_linked_shader *sh, bool coherent)
 {
-   if (sh->Program->sh.fs.BlendSupport == 0)
+   assert(sh->Stage == MESA_SHADER_FRAGMENT);
+
+   if (sh->Program->info.fs.advanced_blend_modes == 0)
       return false;
 
    /* Lower early returns in main() so there's a single exit point
@@ -489,9 +491,8 @@ lower_blend_equation_advanced(struct gl_linked_shader *sh, bool coherent)
    mode->allocate_state_slots(1);
    ir_state_slot *slot0 = &mode->get_state_slots()[0];
    slot0->swizzle = SWIZZLE_XXXX;
-   slot0->tokens[0] = STATE_INTERNAL;
-   slot0->tokens[1] = STATE_ADVANCED_BLENDING_MODE;
-   for (int i = 2; i < STATE_LENGTH; i++)
+   slot0->tokens[0] = STATE_ADVANCED_BLENDING_MODE;
+   for (int i = 1; i < STATE_LENGTH; i++)
       slot0->tokens[i] = 0;
 
    sh->ir->push_head(fb);
@@ -549,7 +550,7 @@ lower_blend_equation_advanced(struct gl_linked_shader *sh, bool coherent)
 
    ir_variable *result_dest =
       calc_blend_result(f, mode, fb, blend_source,
-                        sh->Program->sh.fs.BlendSupport);
+                        sh->Program->info.fs.advanced_blend_modes);
 
    /* Copy the result back to the original values.  It would be simpler
     * to demote the program's output variables, and create a new vec4
