@@ -52,7 +52,8 @@
 #include "ir_print_visitor.h"
 #include "compiler/glsl_types.h"
 #include "link_varyings.h"
-#include "main/mtypes.h"
+#include "main/consts_exts.h"
+#include "main/shader_types.h"
 #include "util/u_string.h"
 
 namespace {
@@ -527,27 +528,29 @@ lower_fragdata_array(struct gl_linked_shader *shader)
    varying_info_visitor info(ir_var_shader_out, true);
    info.get(shader->ir, 0, NULL);
 
-   replace_varyings_visitor(shader, &info, 0, 0, 0);
+   replace_varyings_visitor(shader, &info, 0, 0, false);
 }
 
 
 void
-do_dead_builtin_varyings(struct gl_context *ctx,
+do_dead_builtin_varyings(const struct gl_constants *consts,
+                         gl_api api,
                          gl_linked_shader *producer,
                          gl_linked_shader *consumer,
                          unsigned num_tfeedback_decls,
                          tfeedback_decl *tfeedback_decls)
 {
    /* Lower the gl_FragData array to separate variables. */
-   if (consumer && consumer->Stage == MESA_SHADER_FRAGMENT) {
+   if (consumer && consumer->Stage == MESA_SHADER_FRAGMENT &&
+       !consts->ShaderCompilerOptions[MESA_SHADER_FRAGMENT].NirOptions) {
       lower_fragdata_array(consumer);
    }
 
    /* Lowering of built-in varyings has no effect with the core context and
     * GLES2, because they are not available there.
     */
-   if (ctx->API == API_OPENGL_CORE ||
-       ctx->API == API_OPENGLES2) {
+   if (api == API_OPENGL_CORE ||
+       api == API_OPENGLES2) {
       return;
    }
 
