@@ -21,7 +21,6 @@
  * DEALINGS IN THE SOFTWARE.
  */
 #include <gtest/gtest.h>
-#include "main/mtypes.h"
 #include "main/macros.h"
 #include "util/ralloc.h"
 #include "uniform_initializer_utils.h"
@@ -84,6 +83,7 @@ generate_data_element(void *mem_ctx, const glsl_type *type,
       case GLSL_TYPE_UINT:
       case GLSL_TYPE_INT:
       case GLSL_TYPE_SAMPLER:
+      case GLSL_TYPE_TEXTURE:
       case GLSL_TYPE_IMAGE:
 	 data.i[i] = values[idx];
 	 break;
@@ -110,6 +110,11 @@ generate_data_element(void *mem_ctx, const glsl_type *type,
       case GLSL_TYPE_INTERFACE:
       case GLSL_TYPE_SUBROUTINE:
       case GLSL_TYPE_FUNCTION:
+      case GLSL_TYPE_FLOAT16:
+      case GLSL_TYPE_UINT16:
+      case GLSL_TYPE_INT16:
+      case GLSL_TYPE_UINT8:
+      case GLSL_TYPE_INT8:
 	 ASSERT_TRUE(false);
 	 break;
       }
@@ -124,6 +129,7 @@ generate_data_element(void *mem_ctx, const glsl_type *type,
       case GLSL_TYPE_UINT:
       case GLSL_TYPE_INT:
       case GLSL_TYPE_SAMPLER:
+      case GLSL_TYPE_TEXTURE:
       case GLSL_TYPE_IMAGE:
 	 ASSERT_EQ(data.i[i], val->value.i[i]);
 	 break;
@@ -150,6 +156,11 @@ generate_data_element(void *mem_ctx, const glsl_type *type,
       case GLSL_TYPE_INTERFACE:
       case GLSL_TYPE_SUBROUTINE:
       case GLSL_TYPE_FUNCTION:
+      case GLSL_TYPE_FLOAT16:
+      case GLSL_TYPE_UINT16:
+      case GLSL_TYPE_INT16:
+      case GLSL_TYPE_UINT8:
+      case GLSL_TYPE_INT8:
 	 ASSERT_TRUE(false);
 	 break;
       }
@@ -252,6 +263,7 @@ verify_data(gl_constant_value *storage, unsigned storage_array_size,
 	 case GLSL_TYPE_UINT:
 	 case GLSL_TYPE_INT:
 	 case GLSL_TYPE_SAMPLER:
+	 case GLSL_TYPE_TEXTURE:
 	 case GLSL_TYPE_IMAGE:
 	    EXPECT_EQ(val->value.i[i], storage[i].i);
 	    break;
@@ -278,6 +290,11 @@ verify_data(gl_constant_value *storage, unsigned storage_array_size,
 	 case GLSL_TYPE_INTERFACE:
 	 case GLSL_TYPE_SUBROUTINE:
          case GLSL_TYPE_FUNCTION:
+         case GLSL_TYPE_FLOAT16:
+         case GLSL_TYPE_UINT16:
+         case GLSL_TYPE_INT16:
+         case GLSL_TYPE_UINT8:
+         case GLSL_TYPE_INT8:
 	    ASSERT_TRUE(false);
 	    break;
 	 }
