@@ -28,6 +28,10 @@
 #include "util/bitscan.h"
 #include "util/half_float.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct exec_list;
 
 typedef struct nir_builder {
@@ -44,28 +48,12 @@ typedef struct nir_builder {
    nir_function_impl *impl;
 } nir_builder;
 
-static inline void
-nir_builder_init(nir_builder *build, nir_function_impl *impl)
-{
-   memset(build, 0, sizeof(*build));
-   build->exact = false;
-   build->impl = impl;
-   build->shader = impl->function->shader;
-}
+void nir_builder_init(nir_builder *build, nir_function_impl *impl);
 
-static inline void
-nir_builder_init_simple_shader(nir_builder *build, void *mem_ctx,
-                               gl_shader_stage stage,
-                               const nir_shader_compiler_options *options)
-{
-   memset(build, 0, sizeof(*build));
-   build->shader = nir_shader_create(mem_ctx, stage, options, NULL);
-   nir_function *func = nir_function_create(build->shader, "main");
-   func->is_entrypoint = true;
-   build->exact = false;
-   build->impl = nir_function_impl_create(func);
-   build->cursor = nir_after_cf_list(&build->impl->body);
-}
+nir_builder MUST_CHECK PRINTFLIKE(3, 4)
+nir_builder_init_simple_shader(gl_shader_stage stage,
+                               const nir_shader_compiler_options *options,
+                               const char *name, ...);
 
 typedef bool (*nir_instr_pass_cb)(struct nir_builder *, nir_instr *, void *);
 
@@ -91,17 +79,19 @@ nir_shader_instructions_pass(nir_shader *shader,
       if (!function->impl)
          continue;
 
+      bool func_progress = false;
       nir_builder b;
       nir_builder_init(&b, function->impl);
 
       nir_foreach_block_safe(block, function->impl) {
          nir_foreach_instr_safe(instr, block) {
-            progress |= pass(&b, instr, cb_data);
+            func_progress |= pass(&b, instr, cb_data);
          }
       }
 
-      if (progress) {
+      if (func_progress) {
          nir_metadata_preserve(function->impl, preserved);
+         progress = true;
       } else {
          nir_metadata_preserve(function->impl, nir_metadata_all);
       }
@@ -110,17 +100,7 @@ nir_shader_instructions_pass(nir_shader *shader,
    return progress;
 }
 
-static inline void
-nir_builder_instr_insert(nir_builder *build, nir_instr *instr)
-{
-   nir_instr_insert(build->cursor, instr);
-
-   if (build->update_divergence)
-      nir_update_instr_divergence(build->shader, instr);
-
-   /* Move the cursor forward. */
-   build->cursor = nir_after_instr(instr);
-}
+void nir_builder_instr_insert(nir_builder *build, nir_instr *instr);
 
 static inline nir_instr *
 nir_builder_last_instr(nir_builder *build)
@@ -129,112 +109,50 @@ nir_builder_last_instr(nir_builder *build)
    return build->cursor.instr;
 }
 
-static inline void
-nir_builder_cf_insert(nir_builder *build, nir_cf_node *cf)
-{
-   nir_cf_node_insert(build->cursor, cf);
-}
+/* General nir_build_alu() taking a variable arg count with NULLs for the rest. */
+nir_ssa_def *
+nir_build_alu(nir_builder *build, nir_op op, nir_ssa_def *src0,
+              nir_ssa_def *src1, nir_ssa_def *src2, nir_ssa_def *src3);
 
-static inline bool
-nir_builder_is_inside_cf(nir_builder *build, nir_cf_node *cf_node)
-{
-   nir_block *block = nir_cursor_current_block(build->cursor);
-   for (nir_cf_node *n = &block->cf_node; n; n = n->parent) {
-      if (n == cf_node)
-         return true;
-   }
-   return false;
-}
+/* Fixed-arg-count variants to reduce size of codegen. */
+nir_ssa_def *
+nir_build_alu1(nir_builder *build, nir_op op, nir_ssa_def *src0);
+nir_ssa_def *
+nir_build_alu2(nir_builder *build, nir_op op, nir_ssa_def *src0,
+               nir_ssa_def *src1);
+nir_ssa_def *
+nir_build_alu3(nir_builder *build, nir_op op, nir_ssa_def *src0,
+               nir_ssa_def *src1, nir_ssa_def *src2);
+nir_ssa_def *
+nir_build_alu4(nir_builder *build, nir_op op, nir_ssa_def *src0,
+               nir_ssa_def *src1, nir_ssa_def *src2, nir_ssa_def *src3);
 
-static inline nir_if *
-nir_push_if_src(nir_builder *build, nir_src condition)
-{
-   nir_if *nif = nir_if_create(build->shader);
-   nif->condition = condition;
-   nir_builder_cf_insert(build, &nif->cf_node);
-   build->cursor = nir_before_cf_list(&nif->then_list);
-   return nif;
-}
+nir_ssa_def *nir_build_alu_src_arr(nir_builder *build, nir_op op, nir_ssa_def **srcs);
 
-static inline nir_if *
-nir_push_if(nir_builder *build, nir_ssa_def *condition)
-{
-   return nir_push_if_src(build, nir_src_for_ssa(condition));
-}
+nir_instr *nir_builder_last_instr(nir_builder *build);
 
-static inline nir_if *
-nir_push_else(nir_builder *build, nir_if *nif)
-{
-   if (nif) {
-      assert(nir_builder_is_inside_cf(build, &nif->cf_node));
-   } else {
-      nir_block *block = nir_cursor_current_block(build->cursor);
-      nif = nir_cf_node_as_if(block->cf_node.parent);
-   }
-   build->cursor = nir_before_cf_list(&nif->else_list);
-   return nif;
-}
+void nir_builder_cf_insert(nir_builder *build, nir_cf_node *cf);
 
-static inline void
-nir_pop_if(nir_builder *build, nir_if *nif)
-{
-   if (nif) {
-      assert(nir_builder_is_inside_cf(build, &nif->cf_node));
-   } else {
-      nir_block *block = nir_cursor_current_block(build->cursor);
-      nif = nir_cf_node_as_if(block->cf_node.parent);
-   }
-   build->cursor = nir_after_cf_node(&nif->cf_node);
-}
+bool nir_builder_is_inside_cf(nir_builder *build, nir_cf_node *cf_node);
 
-static inline nir_ssa_def *
-nir_if_phi(nir_builder *build, nir_ssa_def *then_def, nir_ssa_def *else_def)
-{
-   nir_block *block = nir_cursor_current_block(build->cursor);
-   nir_if *nif = nir_cf_node_as_if(nir_cf_node_prev(&block->cf_node));
+nir_if *
+nir_push_if_src(nir_builder *build, nir_src condition);
 
-   nir_phi_instr *phi = nir_phi_instr_create(build->shader);
+nir_if *
+nir_push_if(nir_builder *build, nir_ssa_def *condition);
 
-   nir_phi_src *src = ralloc(phi, nir_phi_src);
-   src->pred = nir_if_last_then_block(nif);
-   src->src = nir_src_for_ssa(then_def);
-   exec_list_push_tail(&phi->srcs, &src->node);
+nir_if *
+nir_push_else(nir_builder *build, nir_if *nif);
 
-   src = ralloc(phi, nir_phi_src);
-   src->pred = nir_if_last_else_block(nif);
-   src->src = nir_src_for_ssa(else_def);
-   exec_list_push_tail(&phi->srcs, &src->node);
+void nir_pop_if(nir_builder *build, nir_if *nif);
 
-   assert(then_def->num_components == else_def->num_components);
-   assert(then_def->bit_size == else_def->bit_size);
-   nir_ssa_dest_init(&phi->instr, &phi->dest,
-                     then_def->num_components, then_def->bit_size, NULL);
+nir_ssa_def *
+nir_if_phi(nir_builder *build, nir_ssa_def *then_def, nir_ssa_def *else_def);
 
-   nir_builder_instr_insert(build, &phi->instr);
+nir_loop *
+nir_push_loop(nir_builder *build);
 
-   return &phi->dest.ssa;
-}
-
-static inline nir_loop *
-nir_push_loop(nir_builder *build)
-{
-   nir_loop *loop = nir_loop_create(build->shader);
-   nir_builder_cf_insert(build, &loop->cf_node);
-   build->cursor = nir_before_cf_list(&loop->body);
-   return loop;
-}
-
-static inline void
-nir_pop_loop(nir_builder *build, nir_loop *loop)
-{
-   if (loop) {
-      assert(nir_builder_is_inside_cf(build, &loop->cf_node));
-   } else {
-      nir_block *block = nir_cursor_current_block(build->cursor);
-      loop = nir_cf_node_as_loop(block->cf_node.parent);
-   }
-   build->cursor = nir_after_cf_node(&loop->cf_node);
-}
+void nir_pop_loop(nir_builder *build, nir_loop *loop);
 
 static inline nir_ssa_def *
 nir_ssa_undef(nir_builder *build, unsigned num_components, unsigned bit_size)
@@ -340,6 +258,17 @@ nir_imm_vec2(nir_builder *build, float x, float y)
    return nir_build_imm(build, 2, 32, v);
 }
 
+static inline nir_ssa_def *
+nir_imm_vec3(nir_builder *build, float x, float y, float z)
+{
+   nir_const_value v[3] = {
+      nir_const_value_for_float(x, 32),
+      nir_const_value_for_float(y, 32),
+      nir_const_value_for_float(z, 32),
+   };
+   return nir_build_imm(build, 3, 32, v);
+}
+
 static inline nir_ssa_def *
 nir_imm_vec4(nir_builder *build, float x, float y, float z, float w)
 {
@@ -396,6 +325,18 @@ nir_imm_ivec2(nir_builder *build, int x, int y)
    return nir_build_imm(build, 2, 32, v);
 }
 
+static inline nir_ssa_def *
+nir_imm_ivec3(nir_builder *build, int x, int y, int z)
+{
+   nir_const_value v[3] = {
+      nir_const_value_for_int(x, 32),
+      nir_const_value_for_int(y, 32),
+      nir_const_value_for_int(z, 32),
+   };
+
+   return nir_build_imm(build, 3, 32, v);
+}
+
 static inline nir_ssa_def *
 nir_imm_ivec4(nir_builder *build, int x, int y, int z, int w)
 {
@@ -409,103 +350,20 @@ nir_imm_ivec4(nir_builder *build, int x, int y, int z, int w)
    return nir_build_imm(build, 4, 32, v);
 }
 
-static inline nir_ssa_def *
-nir_builder_alu_instr_finish_and_insert(nir_builder *build, nir_alu_instr *instr)
-{
-   const nir_op_info *op_info = &nir_op_infos[instr->op];
-
-   instr->exact = build->exact;
-
-   /* Guess the number of components the destination temporary should have
-    * based on our input sizes, if it's not fixed for the op.
-    */
-   unsigned num_components = op_info->output_size;
-   if (num_components == 0) {
-      for (unsigned i = 0; i < op_info->num_inputs; i++) {
-         if (op_info->input_sizes[i] == 0)
-            num_components = MAX2(num_components,
-                                  instr->src[i].src.ssa->num_components);
-      }
-   }
-   assert(num_components != 0);
-
-   /* Figure out the bitwidth based on the source bitwidth if the instruction
-    * is variable-width.
-    */
-   unsigned bit_size = nir_alu_type_get_type_size(op_info->output_type);
-   if (bit_size == 0) {
-      for (unsigned i = 0; i < op_info->num_inputs; i++) {
-         unsigned src_bit_size = instr->src[i].src.ssa->bit_size;
-         if (nir_alu_type_get_type_size(op_info->input_types[i]) == 0) {
-            if (bit_size)
-               assert(src_bit_size == bit_size);
-            else
-               bit_size = src_bit_size;
-         } else {
-            assert(src_bit_size ==
-               nir_alu_type_get_type_size(op_info->input_types[i]));
-         }
-      }
-   }
-
-   /* When in doubt, assume 32. */
-   if (bit_size == 0)
-      bit_size = 32;
-
-   /* Make sure we don't swizzle from outside of our source vector (like if a
-    * scalar value was passed into a multiply with a vector).
-    */
-   for (unsigned i = 0; i < op_info->num_inputs; i++) {
-      for (unsigned j = instr->src[i].src.ssa->num_components;
-           j < NIR_MAX_VEC_COMPONENTS; j++) {
-         instr->src[i].swizzle[j] = instr->src[i].src.ssa->num_components - 1;
-      }
-   }
-
-   nir_ssa_dest_init(&instr->instr, &instr->dest.dest, num_components,
-                     bit_size, NULL);
-   instr->dest.write_mask = (1 << num_components) - 1;
-
-   nir_builder_instr_insert(build, &instr->instr);
-
-   return &instr->dest.dest.ssa;
-}
-
-static inline nir_ssa_def *
-nir_build_alu(nir_builder *build, nir_op op, nir_ssa_def *src0,
-              nir_ssa_def *src1, nir_ssa_def *src2, nir_ssa_def *src3)
-{
-   nir_alu_instr *instr = nir_alu_instr_create(build->shader, op);
-   if (!instr)
-      return NULL;
-
-   instr->src[0].src = nir_src_for_ssa(src0);
-   if (src1)
-      instr->src[1].src = nir_src_for_ssa(src1);
-   if (src2)
-      instr->src[2].src = nir_src_for_ssa(src2);
-   if (src3)
-      instr->src[3].src = nir_src_for_ssa(src3);
-
-   return nir_builder_alu_instr_finish_and_insert(build, instr);
-}
+nir_ssa_def *
+nir_builder_alu_instr_finish_and_insert(nir_builder *build, nir_alu_instr *instr);
 
 /* for the couple special cases with more than 4 src args: */
-static inline nir_ssa_def *
-nir_build_alu_src_arr(nir_builder *build, nir_op op, nir_ssa_def **srcs)
-{
-   const nir_op_info *op_info = &nir_op_infos[op];
-   nir_alu_instr *instr = nir_alu_instr_create(build->shader, op);
-   if (!instr)
-      return NULL;
+nir_ssa_def *
+nir_build_alu_src_arr(nir_builder *build, nir_op op, nir_ssa_def **srcs);
 
-   for (unsigned i = 0; i < op_info->num_inputs; i++)
-      instr->src[i].src = nir_src_for_ssa(srcs[i]);
-
-   return nir_builder_alu_instr_finish_and_insert(build, instr);
-}
+/* Generic builder for system values. */
+nir_ssa_def *
+nir_load_system_value(nir_builder *build, nir_intrinsic_op op, int index,
+                      unsigned num_components, unsigned bit_size);
 
 #include "nir_builder_opcodes.h"
+#undef nir_deref_mode_is
 
 static inline nir_ssa_def *
 nir_vec(nir_builder *build, nir_ssa_def **comp, unsigned num_components)
@@ -539,7 +397,7 @@ nir_mov_alu(nir_builder *build, nir_alu_src src, unsigned num_components)
 }
 
 /**
- * Construct an fmov or imov that reswizzles the source's components.
+ * Construct a mov that reswizzles the source's components.
  */
 static inline nir_ssa_def *
 nir_swizzle(nir_builder *build, nir_ssa_def *src, const unsigned *swiz,
@@ -572,6 +430,7 @@ nir_fdot(nir_builder *build, nir_ssa_def *src0, nir_ssa_def *src1)
    case 2: return nir_fdot2(build, src0, src1);
    case 3: return nir_fdot3(build, src0, src1);
    case 4: return nir_fdot4(build, src0, src1);
+   case 5: return nir_fdot5(build, src0, src1);
    case 8: return nir_fdot8(build, src0, src1);
    case 16: return nir_fdot16(build, src0, src1);
    default:
@@ -589,6 +448,7 @@ nir_ball_iequal(nir_builder *b, nir_ssa_def *src0, nir_ssa_def *src1)
    case 2: return nir_ball_iequal2(b, src0, src1);
    case 3: return nir_ball_iequal3(b, src0, src1);
    case 4: return nir_ball_iequal4(b, src0, src1);
+   case 5: return nir_ball_iequal5(b, src0, src1);
    case 8: return nir_ball_iequal8(b, src0, src1);
    case 16: return nir_ball_iequal16(b, src0, src1);
    default:
@@ -610,6 +470,7 @@ nir_bany_inequal(nir_builder *b, nir_ssa_def *src0, nir_ssa_def *src1)
    case 2: return nir_bany_inequal2(b, src0, src1);
    case 3: return nir_bany_inequal3(b, src0, src1);
    case 4: return nir_bany_inequal4(b, src0, src1);
+   case 5: return nir_bany_inequal5(b, src0, src1);
    case 8: return nir_bany_inequal8(b, src0, src1);
    case 16: return nir_bany_inequal16(b, src0, src1);
    default:
@@ -781,6 +642,22 @@ nir_iadd_imm(nir_builder *build, nir_ssa_def *x, uint64_t y)
    }
 }
 
+static inline nir_ssa_def *
+nir_iadd_imm_nuw(nir_builder *b, nir_ssa_def *x, uint64_t y)
+{
+   nir_ssa_def *d = nir_iadd_imm(b, x, y);
+   if (d != x && d->parent_instr->type == nir_instr_type_alu)
+      nir_instr_as_alu(d->parent_instr)->no_unsigned_wrap = true;
+   return d;
+}
+
+static inline nir_ssa_def *
+nir_iadd_nuw(nir_builder *b, nir_ssa_def *x, nir_ssa_def *y)
+{
+   nir_ssa_def *d = nir_iadd(b, x, y);
+   nir_instr_as_alu(d->parent_instr)->no_unsigned_wrap = true;
+   return d;
+}
 
 static inline nir_ssa_def *
 nir_ieq_imm(nir_builder *build, nir_ssa_def *x, uint64_t y)
@@ -788,6 +665,13 @@ nir_ieq_imm(nir_builder *build, nir_ssa_def *x, uint64_t y)
    return nir_ieq(build, x, nir_imm_intN_t(build, y, x->bit_size));
 }
 
+/* Use nir_iadd(x, -y) for reversing parameter ordering */
+static inline nir_ssa_def *
+nir_isub_imm(nir_builder *build, uint64_t y, nir_ssa_def *x)
+{
+   return nir_isub(build, nir_imm_intN_t(build, y, x->bit_size), x);
+}
+
 static inline nir_ssa_def *
 _nir_mul_imm(nir_builder *build, nir_ssa_def *x, uint64_t y, bool amul)
 {
@@ -847,6 +731,18 @@ nir_iand_imm(nir_builder *build, nir_ssa_def *x, uint64_t y)
    }
 }
 
+static inline nir_ssa_def *
+nir_ishl_imm(nir_builder *build, nir_ssa_def *x, uint32_t y)
+{
+   if (y == 0) {
+      return x;
+   } else if (y >= x->bit_size) {
+      return nir_imm_intN_t(build, 0, x->bit_size);
+   } else {
+      return nir_ishl(build, x, nir_imm_int(build, y));
+   }
+}
+
 static inline nir_ssa_def *
 nir_ishr_imm(nir_builder *build, nir_ssa_def *x, uint32_t y)
 {
@@ -882,6 +778,56 @@ nir_udiv_imm(nir_builder *build, nir_ssa_def *x, uint64_t y)
    }
 }
 
+static inline nir_ssa_def *
+nir_fclamp(nir_builder *b,
+           nir_ssa_def *x, nir_ssa_def *min_val, nir_ssa_def *max_val)
+{
+   return nir_fmin(b, nir_fmax(b, x, min_val), max_val);
+}
+
+static inline nir_ssa_def *
+nir_iclamp(nir_builder *b,
+           nir_ssa_def *x, nir_ssa_def *min_val, nir_ssa_def *max_val)
+{
+   return nir_imin(b, nir_imax(b, x, min_val), max_val);
+}
+
+static inline nir_ssa_def *
+nir_uclamp(nir_builder *b,
+           nir_ssa_def *x, nir_ssa_def *min_val, nir_ssa_def *max_val)
+{
+   return nir_umin(b, nir_umax(b, x, min_val), max_val);
+}
+
+static inline nir_ssa_def *
+nir_ffma_imm12(nir_builder *build, nir_ssa_def *src0, double src1, double src2)
+{
+   if (build->shader->options->avoid_ternary_with_two_constants)
+      return nir_fadd_imm(build, nir_fmul_imm(build, src0, src1), src2);
+   else
+      return nir_ffma(build, src0, nir_imm_floatN_t(build, src1, src0->bit_size),
+                             nir_imm_floatN_t(build, src2, src0->bit_size));
+}
+
+static inline nir_ssa_def *
+nir_ffma_imm1(nir_builder *build, nir_ssa_def *src0, double src1, nir_ssa_def *src2)
+{
+   return nir_ffma(build, src0, nir_imm_floatN_t(build, src1, src0->bit_size), src2);
+}
+
+static inline nir_ssa_def *
+nir_ffma_imm2(nir_builder *build, nir_ssa_def *src0, nir_ssa_def *src1, double src2)
+{
+   return nir_ffma(build, src0, src1, nir_imm_floatN_t(build, src2, src0->bit_size));
+}
+
+static inline nir_ssa_def *
+nir_a_minus_bc(nir_builder *build, nir_ssa_def *src0, nir_ssa_def *src1,
+               nir_ssa_def *src2)
+{
+   return nir_ffma(build, nir_fneg(build, src1), src2, src0);
+}
+
 static inline nir_ssa_def *
 nir_pack_bits(nir_builder *b, nir_ssa_def *src, unsigned dest_bit_size)
 {
@@ -1041,40 +987,67 @@ nir_bitcast_vector(nir_builder *b, nir_ssa_def *src, unsigned dest_bit_size)
 }
 
 /**
- * Turns a nir_src into a nir_ssa_def * so it can be passed to
- * nir_build_alu()-based builder calls.
- *
- * See nir_ssa_for_alu_src() for alu instructions.
+ * Pad a value to N components with undefs of matching bit size.
+ * If the value already contains >= num_components, it is returned without change.
  */
 static inline nir_ssa_def *
-nir_ssa_for_src(nir_builder *build, nir_src src, int num_components)
+nir_pad_vector(nir_builder *b, nir_ssa_def *src, unsigned num_components)
 {
-   if (src.is_ssa && src.ssa->num_components == num_components)
-      return src.ssa;
+   assert(src->num_components <= num_components);
+   if (src->num_components == num_components)
+      return src;
 
-   nir_alu_src alu = { NIR_SRC_INIT };
-   alu.src = src;
-   for (int j = 0; j < NIR_MAX_VEC_COMPONENTS; j++)
-      alu.swizzle[j] = j;
+   nir_ssa_def *components[NIR_MAX_VEC_COMPONENTS];
+   nir_ssa_def *undef = nir_ssa_undef(b, 1, src->bit_size);
+   unsigned i = 0;
+   for (; i < src->num_components; i++)
+      components[i] = nir_channel(b, src, i);
+   for (; i < num_components; i++)
+      components[i] = undef;
 
-   return nir_mov_alu(build, alu, num_components);
+   return nir_vec(b, components, num_components);
 }
 
 /**
- * Similar to nir_ssa_for_src(), but for alu srcs, respecting the
- * nir_alu_src's swizzle.
+ * Pad a value to N components with copies of the given immediate of matching
+ * bit size. If the value already contains >= num_components, it is returned
+ * without change.
  */
 static inline nir_ssa_def *
-nir_ssa_for_alu_src(nir_builder *build, nir_alu_instr *instr, unsigned srcn)
+nir_pad_vector_imm_int(nir_builder *b, nir_ssa_def *src, uint64_t imm_val,
+                       unsigned num_components)
 {
-   if (nir_alu_src_is_trivial_ssa(instr, srcn))
-      return instr->src[srcn].src.ssa;
+   assert(src->num_components <= num_components);
+   if (src->num_components == num_components)
+      return src;
 
-   nir_alu_src *src = &instr->src[srcn];
-   unsigned num_components = nir_ssa_alu_instr_src_components(instr, srcn);
-   return nir_mov_alu(build, *src, num_components);
+   nir_ssa_def *components[NIR_MAX_VEC_COMPONENTS];
+   nir_ssa_def *imm = nir_imm_intN_t(b, imm_val, src->bit_size);
+   unsigned i = 0;
+   for (; i < src->num_components; i++)
+      components[i] = nir_channel(b, src, i);
+   for (; i < num_components; i++)
+      components[i] = imm;
+
+   return nir_vec(b, components, num_components);
 }
 
+/**
+ * Pad a value to 4 components with undefs of matching bit size.
+ * If the value already contains >= 4 components, it is returned without change.
+ */
+static inline nir_ssa_def *
+nir_pad_vec4(nir_builder *b, nir_ssa_def *src)
+{
+   return nir_pad_vector(b, src, 4);
+}
+
+nir_ssa_def *
+nir_ssa_for_src(nir_builder *build, nir_src src, int num_components);
+
+nir_ssa_def *
+nir_ssa_for_alu_src(nir_builder *build, nir_alu_instr *instr, unsigned srcn);
+
 static inline unsigned
 nir_get_ptr_bitsize(nir_shader *shader)
 {
@@ -1331,17 +1304,12 @@ static inline nir_ssa_def *
 nir_load_deref_with_access(nir_builder *build, nir_deref_instr *deref,
                            enum gl_access_qualifier access)
 {
-   nir_intrinsic_instr *load =
-      nir_intrinsic_instr_create(build->shader, nir_intrinsic_load_deref);
-   load->num_components = glsl_get_vector_elements(deref->type);
-   load->src[0] = nir_src_for_ssa(&deref->dest.ssa);
-   nir_ssa_dest_init(&load->instr, &load->dest, load->num_components,
-                     glsl_get_bit_size(deref->type), NULL);
-   nir_intrinsic_set_access(load, access);
-   nir_builder_instr_insert(build, &load->instr);
-   return &load->dest.ssa;
+   return nir_build_load_deref(build, glsl_get_vector_elements(deref->type),
+                               glsl_get_bit_size(deref->type), &deref->dest.ssa,
+                               access);
 }
 
+#undef nir_load_deref
 static inline nir_ssa_def *
 nir_load_deref(nir_builder *build, nir_deref_instr *deref)
 {
@@ -1353,17 +1321,11 @@ nir_store_deref_with_access(nir_builder *build, nir_deref_instr *deref,
                             nir_ssa_def *value, unsigned writemask,
                             enum gl_access_qualifier access)
 {
-   nir_intrinsic_instr *store =
-      nir_intrinsic_instr_create(build->shader, nir_intrinsic_store_deref);
-   store->num_components = glsl_get_vector_elements(deref->type);
-   store->src[0] = nir_src_for_ssa(&deref->dest.ssa);
-   store->src[1] = nir_src_for_ssa(value);
-   nir_intrinsic_set_write_mask(store,
-                                writemask & ((1 << store->num_components) - 1));
-   nir_intrinsic_set_access(store, access);
-   nir_builder_instr_insert(build, &store->instr);
+   writemask &= (1u << value->num_components) - 1u;
+   nir_build_store_deref(build, &deref->dest.ssa, value, writemask, access);
 }
 
+#undef nir_store_deref
 static inline void
 nir_store_deref(nir_builder *build, nir_deref_instr *deref,
                 nir_ssa_def *value, unsigned writemask)
@@ -1378,15 +1340,10 @@ nir_copy_deref_with_access(nir_builder *build, nir_deref_instr *dest,
                            enum gl_access_qualifier dest_access,
                            enum gl_access_qualifier src_access)
 {
-   nir_intrinsic_instr *copy =
-      nir_intrinsic_instr_create(build->shader, nir_intrinsic_copy_deref);
-   copy->src[0] = nir_src_for_ssa(&dest->dest.ssa);
-   copy->src[1] = nir_src_for_ssa(&src->dest.ssa);
-   nir_intrinsic_set_dst_access(copy, dest_access);
-   nir_intrinsic_set_src_access(copy, src_access);
-   nir_builder_instr_insert(build, &copy->instr);
+   nir_build_copy_deref(build, &dest->dest.ssa, &src->dest.ssa, dest_access, src_access);
 }
 
+#undef nir_copy_deref
 static inline void
 nir_copy_deref(nir_builder *build, nir_deref_instr *dest, nir_deref_instr *src)
 {
@@ -1401,16 +1358,11 @@ nir_memcpy_deref_with_access(nir_builder *build, nir_deref_instr *dest,
                              enum gl_access_qualifier dest_access,
                              enum gl_access_qualifier src_access)
 {
-   nir_intrinsic_instr *copy =
-      nir_intrinsic_instr_create(build->shader, nir_intrinsic_memcpy_deref);
-   copy->src[0] = nir_src_for_ssa(&dest->dest.ssa);
-   copy->src[1] = nir_src_for_ssa(&src->dest.ssa);
-   copy->src[2] = nir_src_for_ssa(size);
-   nir_intrinsic_set_dst_access(copy, dest_access);
-   nir_intrinsic_set_src_access(copy, src_access);
-   nir_builder_instr_insert(build, &copy->instr);
+   nir_build_memcpy_deref(build, &dest->dest.ssa, &src->dest.ssa,
+                          size, dest_access, src_access);
 }
 
+#undef nir_memcpy_deref
 static inline void
 nir_memcpy_deref(nir_builder *build, nir_deref_instr *dest,
                  nir_deref_instr *src, nir_ssa_def *size)
@@ -1420,19 +1372,6 @@ nir_memcpy_deref(nir_builder *build, nir_deref_instr *dest,
                                 (enum gl_access_qualifier)0);
 }
 
-static inline nir_ssa_def *
-nir_build_deref_mode_is(nir_builder *build, nir_deref_instr *deref,
-                        nir_variable_mode mode)
-{
-   nir_intrinsic_instr *intrin =
-      nir_intrinsic_instr_create(build->shader, nir_intrinsic_deref_mode_is);
-   intrin->src[0] = nir_src_for_ssa(&deref->dest.ssa);
-   nir_intrinsic_set_memory_modes(intrin, mode);
-   nir_ssa_dest_init(&intrin->instr, &intrin->dest, 1, 1, NULL);
-   nir_builder_instr_insert(build, &intrin->instr);
-   return &intrin->dest.ssa;
-}
-
 static inline nir_ssa_def *
 nir_load_var(nir_builder *build, nir_variable *var)
 {
@@ -1453,6 +1392,7 @@ nir_copy_var(nir_builder *build, nir_variable *dest, nir_variable *src)
                          nir_build_deref_var(build, src));
 }
 
+#undef nir_load_global
 static inline nir_ssa_def *
 nir_load_global(nir_builder *build, nir_ssa_def *addr, unsigned align,
                 unsigned num_components, unsigned bit_size)
@@ -1468,6 +1408,7 @@ nir_load_global(nir_builder *build, nir_ssa_def *addr, unsigned align,
    return &load->dest.ssa;
 }
 
+#undef nir_store_global
 static inline void
 nir_store_global(nir_builder *build, nir_ssa_def *addr, unsigned align,
                  nir_ssa_def *value, nir_component_mask_t write_mask)
@@ -1483,60 +1424,65 @@ nir_store_global(nir_builder *build, nir_ssa_def *addr, unsigned align,
    nir_builder_instr_insert(build, &store->instr);
 }
 
+#undef nir_load_global_constant
+static inline nir_ssa_def *
+nir_load_global_constant(nir_builder *build, nir_ssa_def *addr, unsigned align,
+                         unsigned num_components, unsigned bit_size)
+{
+   nir_intrinsic_instr *load =
+      nir_intrinsic_instr_create(build->shader, nir_intrinsic_load_global_constant);
+   load->num_components = num_components;
+   load->src[0] = nir_src_for_ssa(addr);
+   nir_intrinsic_set_align(load, align, 0);
+   nir_ssa_dest_init(&load->instr, &load->dest,
+                     num_components, bit_size, NULL);
+   nir_builder_instr_insert(build, &load->instr);
+   return &load->dest.ssa;
+}
+
+#undef nir_load_param
 static inline nir_ssa_def *
 nir_load_param(nir_builder *build, uint32_t param_idx)
 {
    assert(param_idx < build->impl->function->num_params);
    nir_parameter *param = &build->impl->function->params[param_idx];
-
-   nir_intrinsic_instr *load =
-      nir_intrinsic_instr_create(build->shader, nir_intrinsic_load_param);
-   nir_intrinsic_set_param_idx(load, param_idx);
-   load->num_components = param->num_components;
-   nir_ssa_dest_init(&load->instr, &load->dest,
-                     param->num_components, param->bit_size, NULL);
-   nir_builder_instr_insert(build, &load->instr);
-   return &load->dest.ssa;
+   return nir_build_load_param(build, param->num_components, param->bit_size, param_idx);
 }
 
+/**
+ * This function takes an I/O intrinsic like load/store_input,
+ * and emits a sequence that calculates the full offset of that instruction,
+ * including a stride to the base and component offsets.
+ */
 static inline nir_ssa_def *
-nir_load_reloc_const_intel(nir_builder *b, uint32_t id)
+nir_build_calc_io_offset(nir_builder *b,
+                         nir_intrinsic_instr *intrin,
+                         nir_ssa_def *base_stride,
+                         unsigned component_stride)
 {
-   nir_intrinsic_instr *load =
-      nir_intrinsic_instr_create(b->shader,
-                                 nir_intrinsic_load_reloc_const_intel);
-   nir_intrinsic_set_param_idx(load, id);
-   nir_ssa_dest_init(&load->instr, &load->dest, 1, 32, NULL);
-   nir_builder_instr_insert(b, &load->instr);
-   return &load->dest.ssa;
+   /* base is the driver_location, which is in slots (1 slot = 4x4 bytes) */
+   nir_ssa_def *base_op = nir_imul_imm(b, base_stride, nir_intrinsic_base(intrin));
+
+   /* offset should be interpreted in relation to the base,
+    * so the instruction effectively reads/writes another input/output
+    * when it has an offset
+    */
+   nir_ssa_def *offset_op = nir_imul(b, base_stride, nir_ssa_for_src(b, *nir_get_io_offset_src(intrin), 1));
+
+   /* component is in bytes */
+   unsigned const_op = nir_intrinsic_component(intrin) * component_stride;
+
+   return nir_iadd_imm_nuw(b, nir_iadd_nuw(b, base_op, offset_op), const_op);
 }
 
+/* calculate a `(1 << value) - 1` in ssa without overflows */
 static inline nir_ssa_def *
-nir_convert_alu_types(nir_builder *b, nir_ssa_def *src,
-                      nir_alu_type src_type, nir_alu_type dest_type,
-                      nir_rounding_mode round, bool saturate)
+nir_mask(nir_builder *b, nir_ssa_def *bits, unsigned dst_bit_size)
 {
-   assert(nir_alu_type_get_type_size(dest_type) != 0);
-   assert(nir_alu_type_get_type_size(src_type) == 0 ||
-          nir_alu_type_get_type_size(src_type) == src->bit_size);
-   src_type = (nir_alu_type)(src_type | src->bit_size);
-
-   nir_intrinsic_instr *conv =
-      nir_intrinsic_instr_create(b->shader, nir_intrinsic_convert_alu_types);
-   conv->src[0] = nir_src_for_ssa(src);
-   conv->num_components = src->num_components;
-   nir_intrinsic_set_src_type(conv, src_type);
-   nir_intrinsic_set_dest_type(conv, dest_type);
-   nir_intrinsic_set_rounding_mode(conv, round);
-   nir_intrinsic_set_saturate(conv, saturate);
-   nir_ssa_dest_init(&conv->instr, &conv->dest, src->num_components,
-                     nir_alu_type_get_type_size(dest_type), NULL);
-   nir_builder_instr_insert(b, &conv->instr);
-   return &conv->dest.ssa;
+   return nir_ushr(b, nir_imm_intN_t(b, -1, dst_bit_size),
+                      nir_isub_imm(b, dst_bit_size, nir_u2u32(b, bits)));
 }
 
-#include "nir_builder_opcodes.h"
-
 static inline nir_ssa_def *
 nir_f2b(nir_builder *build, nir_ssa_def *f)
 {
@@ -1614,46 +1560,9 @@ nir_goto_if(nir_builder *build, struct nir_block *target, nir_src cond,
    nir_builder_instr_insert(build, &jump->instr);
 }
 
-static inline nir_ssa_def *
+nir_ssa_def *
 nir_compare_func(nir_builder *b, enum compare_func func,
-                 nir_ssa_def *src0, nir_ssa_def *src1)
-{
-   switch (func) {
-   case COMPARE_FUNC_NEVER:
-      return nir_imm_int(b, 0);
-   case COMPARE_FUNC_ALWAYS:
-      return nir_imm_int(b, ~0);
-   case COMPARE_FUNC_EQUAL:
-      return nir_feq(b, src0, src1);
-   case COMPARE_FUNC_NOTEQUAL:
-      return nir_fneu(b, src0, src1);
-   case COMPARE_FUNC_GREATER:
-      return nir_flt(b, src1, src0);
-   case COMPARE_FUNC_GEQUAL:
-      return nir_fge(b, src0, src1);
-   case COMPARE_FUNC_LESS:
-      return nir_flt(b, src0, src1);
-   case COMPARE_FUNC_LEQUAL:
-      return nir_fge(b, src1, src0);
-   }
-   unreachable("bad compare func");
-}
-
-static inline void
-nir_scoped_barrier(nir_builder *b,
-                   nir_scope exec_scope,
-                   nir_scope mem_scope,
-                   nir_memory_semantics mem_semantics,
-                   nir_variable_mode mem_modes)
-{
-   nir_intrinsic_instr *intrin =
-      nir_intrinsic_instr_create(b->shader, nir_intrinsic_scoped_barrier);
-   nir_intrinsic_set_execution_scope(intrin, exec_scope);
-   nir_intrinsic_set_memory_scope(intrin, mem_scope);
-   nir_intrinsic_set_memory_semantics(intrin, mem_semantics);
-   nir_intrinsic_set_memory_modes(intrin, mem_modes);
-   nir_builder_instr_insert(b, &intrin->instr);
-}
+                 nir_ssa_def *src0, nir_ssa_def *src1);
 
 static inline void
 nir_scoped_memory_barrier(nir_builder *b,
@@ -1664,19 +1573,20 @@ nir_scoped_memory_barrier(nir_builder *b,
    nir_scoped_barrier(b, NIR_SCOPE_NONE, scope, semantics, modes);
 }
 
+nir_ssa_def *
+nir_type_convert(nir_builder *b,
+                 nir_ssa_def *src,
+                 nir_alu_type src_type,
+                 nir_alu_type dest_type);
+
+
 static inline nir_ssa_def *
 nir_convert_to_bit_size(nir_builder *b,
                     nir_ssa_def *src,
                     nir_alu_type type,
                     unsigned bit_size)
 {
-   nir_alu_type base_type = nir_alu_type_get_base_type(type);
-   nir_alu_type dst_type = (nir_alu_type)(bit_size | base_type);
-
-   nir_op opcode =
-      nir_type_conversion_op(type, dst_type, nir_rounding_mode_undef);
-
-   return nir_build_alu(b, opcode, src, NULL, NULL, NULL);
+   return nir_type_convert(b, src, type, (nir_alu_type) (type | bit_size));
 }
 
 static inline nir_ssa_def *
@@ -1703,4 +1613,36 @@ nir_f2fN(nir_builder *b, nir_ssa_def *src, unsigned bit_size)
    return nir_convert_to_bit_size(b, src, nir_type_float, bit_size);
 }
 
+static inline nir_ssa_def *
+nir_i2fN(nir_builder *b, nir_ssa_def *src, unsigned bit_size)
+{
+   return nir_type_convert(b, src, nir_type_int,
+         (nir_alu_type) (nir_type_float | bit_size));
+}
+
+static inline nir_ssa_def *
+nir_u2fN(nir_builder *b, nir_ssa_def *src, unsigned bit_size)
+{
+   return nir_type_convert(b, src, nir_type_uint,
+         (nir_alu_type) (nir_type_float | bit_size));
+}
+
+static inline nir_ssa_def *
+nir_f2uN(nir_builder *b, nir_ssa_def *src, unsigned bit_size)
+{
+   return nir_type_convert(b, src, nir_type_float,
+         (nir_alu_type) (nir_type_uint | bit_size));
+}
+
+static inline nir_ssa_def *
+nir_f2iN(nir_builder *b, nir_ssa_def *src, unsigned bit_size)
+{
+   return nir_type_convert(b, src, nir_type_float,
+         (nir_alu_type) (nir_type_int | bit_size));
+}
+
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif /* NIR_BUILDER_H */
