@@ -25,9 +25,11 @@
  *
  */
 
+#ifndef NIR_CONTROL_FLOW_PRIVATE_H
+#define NIR_CONTROL_FLOW_PRIVATE_H
+
 #include "nir_control_flow.h"
 
-#pragma once
 
 /* Internal control-flow modification functions used when inserting/removing
  * instructions.
@@ -35,3 +37,5 @@
 
 void nir_handle_add_jump(nir_block *block);
 void nir_handle_remove_jump(nir_block *block, nir_jump_type type);
+
+#endif /* NIR_CONTROL_FLOW_PRIVATE_H */
