@@ -105,6 +105,7 @@ visit_intrinsic(nir_shader *shader, nir_intrinsic_instr *instr)
    case nir_intrinsic_load_workgroup_size:
    case nir_intrinsic_load_subgroup_id:
    case nir_intrinsic_load_num_subgroups:
+   case nir_intrinsic_load_ray_launch_size:
    case nir_intrinsic_load_subgroup_size:
    case nir_intrinsic_load_subgroup_eq_mask:
    case nir_intrinsic_load_subgroup_ge_mask:
@@ -154,6 +155,9 @@ visit_intrinsic(nir_shader *shader, nir_intrinsic_instr *instr)
    case nir_intrinsic_load_cull_small_primitives_enabled_amd:
    case nir_intrinsic_load_cull_any_enabled_amd:
    case nir_intrinsic_load_cull_small_prim_precision_amd:
+   case nir_intrinsic_load_user_data_amd:
+   case nir_intrinsic_load_tess_level_inner_default:
+   case nir_intrinsic_load_tess_level_outer_default:
       is_divergent = false;
       break;
 
@@ -167,7 +171,7 @@ visit_intrinsic(nir_shader *shader, nir_intrinsic_instr *instr)
          is_divergent |= !(options & nir_divergence_single_prim_per_subgroup);
       else if (stage == MESA_SHADER_TESS_EVAL)
          is_divergent |= !(options & nir_divergence_single_patch_per_tes_subgroup);
-      else
+      else if (stage != MESA_SHADER_MESH)
          is_divergent = true;
       break;
    case nir_intrinsic_load_per_vertex_input:
@@ -186,18 +190,33 @@ visit_intrinsic(nir_shader *shader, nir_intrinsic_instr *instr)
       is_divergent |= !(options & nir_divergence_single_prim_per_subgroup);
       break;
    case nir_intrinsic_load_output:
-      assert(stage == MESA_SHADER_TESS_CTRL || stage == MESA_SHADER_FRAGMENT);
       is_divergent = instr->src[0].ssa->divergent;
-      if (stage == MESA_SHADER_TESS_CTRL)
+      switch (stage) {
+      case MESA_SHADER_TESS_CTRL:
          is_divergent |= !(options & nir_divergence_single_patch_per_tcs_subgroup);
-      else
+         break;
+      case MESA_SHADER_FRAGMENT:
          is_divergent = true;
+         break;
+      case MESA_SHADER_TASK:
+      case MESA_SHADER_MESH:
+         /* Divergent if src[0] is, so nothing else to do. */
+         break;
+      default:
+         unreachable("Invalid stage for load_output");
+      }
       break;
    case nir_intrinsic_load_per_vertex_output:
-      assert(stage == MESA_SHADER_TESS_CTRL);
+      assert(stage == MESA_SHADER_TESS_CTRL || stage == MESA_SHADER_MESH);
       is_divergent = instr->src[0].ssa->divergent ||
                      instr->src[1].ssa->divergent ||
-                     !(options & nir_divergence_single_patch_per_tcs_subgroup);
+                     (stage == MESA_SHADER_TESS_CTRL &&
+                      !(options & nir_divergence_single_patch_per_tcs_subgroup));
+      break;
+   case nir_intrinsic_load_per_primitive_output:
+      assert(stage == MESA_SHADER_MESH);
+      is_divergent = instr->src[0].ssa->divergent ||
+                     instr->src[1].ssa->divergent;
       break;
    case nir_intrinsic_load_layer_id:
    case nir_intrinsic_load_front_face:
@@ -245,8 +264,9 @@ visit_intrinsic(nir_shader *shader, nir_intrinsic_instr *instr)
       break;
 
    case nir_intrinsic_load_workgroup_id:
-      assert(stage == MESA_SHADER_COMPUTE);
-      is_divergent |= (options & nir_divergence_multiple_workgroup_per_compute_subgroup);
+      assert(gl_shader_stage_uses_workgroup(stage));
+      if (stage == MESA_SHADER_COMPUTE)
+         is_divergent |= (options & nir_divergence_multiple_workgroup_per_compute_subgroup);
       break;
 
    /* Clustered reductions are uniform if cluster_size == subgroup_size or
@@ -370,6 +390,7 @@ visit_intrinsic(nir_shader *shader, nir_intrinsic_instr *instr)
    case nir_intrinsic_load_line_coord:
    case nir_intrinsic_load_frag_coord:
    case nir_intrinsic_load_sample_pos:
+   case nir_intrinsic_load_sample_pos_or_center:
    case nir_intrinsic_load_vertex_id_zero_base:
    case nir_intrinsic_load_vertex_id:
    case nir_intrinsic_load_instance_id:
@@ -424,6 +445,8 @@ visit_intrinsic(nir_shader *shader, nir_intrinsic_instr *instr)
    case nir_intrinsic_image_deref_atomic_fadd:
    case nir_intrinsic_image_deref_atomic_fmin:
    case nir_intrinsic_image_deref_atomic_fmax:
+   case nir_intrinsic_image_deref_atomic_inc_wrap:
+   case nir_intrinsic_image_deref_atomic_dec_wrap:
    case nir_intrinsic_image_atomic_add:
    case nir_intrinsic_image_atomic_imin:
    case nir_intrinsic_image_atomic_umin:
@@ -513,8 +536,10 @@ visit_intrinsic(nir_shader *shader, nir_intrinsic_instr *instr)
    case nir_intrinsic_has_input_vertex_amd:
    case nir_intrinsic_has_input_primitive_amd:
    case nir_intrinsic_load_packed_passthrough_primitive_amd:
-   case nir_intrinsic_load_initial_edgeflag_amd:
+   case nir_intrinsic_load_initial_edgeflags_amd:
    case nir_intrinsic_gds_atomic_add_amd:
+   case nir_intrinsic_load_rt_arg_scratch_offset_amd:
+   case nir_intrinsic_load_intersection_opaque_amd:
       is_divergent = true;
       break;
 
@@ -584,6 +609,7 @@ nir_variable_mode_is_uniform(nir_variable_mode mode) {
    case nir_var_mem_ssbo:
    case nir_var_mem_shared:
    case nir_var_mem_global:
+   case nir_var_image:
       return true;
    default:
       return false;
@@ -997,3 +1023,19 @@ bool nir_update_instr_divergence(nir_shader *shader, nir_instr *instr)
    return true;
 }
 
+
+bool
+nir_has_divergent_loop(nir_shader *shader)
+{
+   bool divergent_loop = false;
+   nir_function_impl *func = nir_shader_get_entrypoint(shader);
+
+   foreach_list_typed(nir_cf_node, node, node, &func->body) {
+      if (node->type == nir_cf_node_loop && nir_cf_node_as_loop(node)->divergent) {
+         divergent_loop = true;
+         break;
+      }
+   }
+
+   return divergent_loop;
+}
