@@ -137,7 +137,8 @@ lower_system_value_instr(nir_builder *b, nir_instr *instr, void *_state)
          assert(deref->deref_type == nir_deref_type_var);
          assert(deref->var->data.location == SYSTEM_VALUE_SAMPLE_MASK_IN ||
                 deref->var->data.location == SYSTEM_VALUE_RAY_OBJECT_TO_WORLD ||
-                deref->var->data.location == SYSTEM_VALUE_RAY_WORLD_TO_OBJECT);
+                deref->var->data.location == SYSTEM_VALUE_RAY_WORLD_TO_OBJECT ||
+                deref->var->data.location == SYSTEM_VALUE_MESH_VIEW_INDICES);
       }
       nir_variable *var = deref->var;
 
@@ -197,6 +198,21 @@ lower_system_value_instr(nir_builder *b, nir_instr *instr, void *_state)
          return nir_load_barycentric(b, nir_intrinsic_load_barycentric_model,
                                      INTERP_MODE_NONE);
 
+      case SYSTEM_VALUE_HELPER_INVOCATION: {
+         /* When demote operation is used, reading the HelperInvocation
+          * needs to use Volatile memory access semantics to provide the
+          * correct (dynamic) value.  See OpDemoteToHelperInvocation.
+          */
+         if (nir_intrinsic_access(intrin) & ACCESS_VOLATILE)
+            return nir_is_helper_invocation(b, 1);
+         break;
+      }
+
+      case SYSTEM_VALUE_MESH_VIEW_INDICES:
+         return nir_load_mesh_view_indices(b, intrin->dest.ssa.num_components,
+               bit_size, column, .base = 0,
+               .range = intrin->dest.ssa.num_components * bit_size / 8);
+
       default:
          break;
       }
@@ -275,7 +291,8 @@ lower_compute_system_value_instr(nir_builder *b,
       /* If lower_cs_local_id_from_index is true, then we derive the local
        * index from the local id.
        */
-      if (b->shader->options->lower_cs_local_id_from_index) {
+      if (b->shader->options->lower_cs_local_id_from_index ||
+          (options && options->lower_cs_local_id_from_index)) {
          /* We lower gl_LocalInvocationID from gl_LocalInvocationIndex based
           * on this formula:
           *
@@ -387,6 +404,21 @@ lower_compute_system_value_instr(nir_builder *b,
 
          return nir_vec3(b, x, y, z);
       }
+
+      /* If a workgroup size dimension is 1, then the local invocation id must be zero. */
+      nir_component_mask_t is_zero = 0;
+      is_zero |= b->shader->info.workgroup_size[0] == 1 ? 0x1 : 0x0;
+      is_zero |= b->shader->info.workgroup_size[1] == 1 ? 0x2 : 0x0;
+      is_zero |= b->shader->info.workgroup_size[2] == 1 ? 0x4 : 0x0;
+      if (!b->shader->info.workgroup_size_variable && is_zero) {
+         nir_ssa_def *defs[3];
+         for (unsigned i = 0; i < 3; i++) {
+            defs[i] = is_zero & (1 << i) ? nir_imm_zero(b, 1, 32) :
+                                           nir_channel(b, &intrin->dest.ssa, i);
+         }
+         return nir_vec(b, defs, 3);
+      }
+
       return NULL;
 
    case nir_intrinsic_load_local_invocation_index:
@@ -502,8 +534,7 @@ bool
 nir_lower_compute_system_values(nir_shader *shader,
                                 const nir_lower_compute_system_values_options *options)
 {
-   if (shader->info.stage != MESA_SHADER_COMPUTE &&
-       shader->info.stage != MESA_SHADER_KERNEL)
+   if (!gl_shader_stage_uses_workgroup(shader->info.stage))
       return false;
 
    struct lower_sysval_state state;
