@@ -289,7 +289,7 @@ sample_plane(nir_builder *b, nir_tex_instr *tex, int plane,
    nir_tex_instr *plane_tex =
       nir_tex_instr_create(b->shader, tex->num_srcs + 1);
    for (unsigned i = 0; i < tex->num_srcs; i++) {
-      nir_src_copy(&plane_tex->src[i].src, &tex->src[i].src, plane_tex);
+      nir_src_copy(&plane_tex->src[i].src, &tex->src[i].src);
       plane_tex->src[i].src_type = tex->src[i].src_type;
    }
    plane_tex->src[tex->num_srcs].src = nir_src_for_ssa(nir_imm_int(b, plane));
@@ -770,7 +770,7 @@ lower_tex_to_txd(nir_builder *b, nir_tex_instr *tex)
 
    /* reuse existing srcs */
    for (unsigned i = 0; i < tex->num_srcs; i++) {
-      nir_src_copy(&txd->src[i].src, &tex->src[i].src, txd);
+      nir_src_copy(&txd->src[i].src, &tex->src[i].src);
       txd->src[i].src_type = tex->src[i].src_type;
    }
    int coord = nir_tex_instr_src_index(tex, nir_tex_src_coord);
@@ -807,7 +807,7 @@ lower_txb_to_txl(nir_builder *b, nir_tex_instr *tex)
    /* reuse all but bias src */
    for (int i = 0; i < 2; i++) {
       if (tex->src[i].src_type != nir_tex_src_bias) {
-         nir_src_copy(&txl->src[i].src, &tex->src[i].src, txl);
+         nir_src_copy(&txl->src[i].src, &tex->src[i].src);
          txl->src[i].src_type = tex->src[i].src_type;
       }
    }
@@ -1097,7 +1097,7 @@ lower_tg4_offsets(nir_builder *b, nir_tex_instr *tex)
       tex_copy->dest_type = tex->dest_type;
 
       for (unsigned j = 0; j < tex->num_srcs; ++j) {
-         nir_src_copy(&tex_copy->src[j].src, &tex->src[j].src, tex_copy);
+         nir_src_copy(&tex_copy->src[j].src, &tex->src[j].src);
          tex_copy->src[j].src_type = tex->src[j].src_type;
       }
 
@@ -1172,6 +1172,121 @@ nir_lower_txs_lod(nir_builder *b, nir_tex_instr *tex)
    return true;
 }
 
+static void
+nir_lower_txs_cube_array(nir_builder *b, nir_tex_instr *tex)
+{
+   assert(tex->sampler_dim == GLSL_SAMPLER_DIM_CUBE && tex->is_array);
+   tex->sampler_dim = GLSL_SAMPLER_DIM_2D;
+
+   b->cursor = nir_after_instr(&tex->instr);
+
+   assert(tex->dest.is_ssa);
+   assert(tex->dest.ssa.num_components == 3);
+   nir_ssa_def *size = &tex->dest.ssa;
+   size = nir_vec3(b, nir_channel(b, size, 0),
+                      nir_channel(b, size, 1),
+                      nir_idiv(b, nir_channel(b, size, 2),
+                                  nir_imm_int(b, 6)));
+
+   nir_ssa_def_rewrite_uses_after(&tex->dest.ssa, size, size->parent_instr);
+}
+
+static void
+nir_lower_ms_txf_to_fragment_fetch(nir_builder *b, nir_tex_instr *tex)
+{
+   lower_offset(b, tex);
+
+   b->cursor = nir_before_instr(&tex->instr);
+
+   /* Create FMASK fetch. */
+   assert(tex->texture_index == 0);
+   nir_tex_instr *fmask_fetch = nir_tex_instr_create(b->shader, tex->num_srcs - 1);
+   fmask_fetch->op = nir_texop_fragment_mask_fetch_amd;
+   fmask_fetch->coord_components = tex->coord_components;
+   fmask_fetch->sampler_dim = tex->sampler_dim;
+   fmask_fetch->is_array = tex->is_array;
+   fmask_fetch->texture_non_uniform = tex->texture_non_uniform;
+   fmask_fetch->dest_type = nir_type_uint32;
+   nir_ssa_dest_init(&fmask_fetch->instr, &fmask_fetch->dest, 1, 32, NULL);
+
+   fmask_fetch->num_srcs = 0;
+   for (unsigned i = 0; i < tex->num_srcs; i++) {
+      if (tex->src[i].src_type == nir_tex_src_ms_index)
+         continue;
+      nir_tex_src *src = &fmask_fetch->src[fmask_fetch->num_srcs++];
+      src->src = nir_src_for_ssa(tex->src[i].src.ssa);
+      src->src_type = tex->src[i].src_type;
+   }
+
+   nir_builder_instr_insert(b, &fmask_fetch->instr);
+
+   /* Obtain new sample index. */
+   int ms_index = nir_tex_instr_src_index(tex, nir_tex_src_ms_index);
+   assert(ms_index >= 0);
+   nir_src sample = tex->src[ms_index].src;
+   nir_ssa_def *new_sample = NULL;
+   if (nir_src_is_const(sample) && (nir_src_as_uint(sample) == 0 || nir_src_as_uint(sample) == 7)) {
+      if (nir_src_as_uint(sample) == 7)
+         new_sample = nir_ushr(b, &fmask_fetch->dest.ssa, nir_imm_int(b, 28));
+      else
+         new_sample = nir_iand_imm(b, &fmask_fetch->dest.ssa, 0xf);
+   } else {
+      new_sample = nir_ubitfield_extract(b, &fmask_fetch->dest.ssa,
+                                         nir_imul_imm(b, sample.ssa, 4), nir_imm_int(b, 4));
+   }
+
+   /* Update instruction. */
+   tex->op = nir_texop_fragment_fetch_amd;
+   nir_instr_rewrite_src_ssa(&tex->instr, &tex->src[ms_index].src, new_sample);
+}
+
+static void
+nir_lower_samples_identical_to_fragment_fetch(nir_builder *b, nir_tex_instr *tex)
+{
+   b->cursor = nir_after_instr(&tex->instr);
+
+   nir_tex_instr *fmask_fetch = nir_instr_as_tex(nir_instr_clone(b->shader, &tex->instr));
+   fmask_fetch->op = nir_texop_fragment_mask_fetch_amd;
+   fmask_fetch->dest_type = nir_type_uint32;
+   nir_ssa_dest_init(&fmask_fetch->instr, &fmask_fetch->dest, 1, 32, NULL);
+   nir_builder_instr_insert(b, &fmask_fetch->instr);
+
+   nir_ssa_def_rewrite_uses(&tex->dest.ssa, nir_ieq_imm(b, &fmask_fetch->dest.ssa, 0));
+   nir_instr_remove_v(&tex->instr);
+}
+
+static void
+nir_lower_lod_zero_width(nir_builder *b, nir_tex_instr *tex)
+{
+   int coord_index = nir_tex_instr_src_index(tex, nir_tex_src_coord);
+   assert(coord_index >= 0);
+
+   b->cursor = nir_after_instr(&tex->instr);
+
+   nir_ssa_def *is_zero = nir_imm_bool(b, true);
+   for (unsigned i = 0; i < tex->coord_components; i++) {
+      nir_ssa_def *coord = nir_channel(b, tex->src[coord_index].src.ssa, i);
+
+      /* Compute the sum of the absolute values of derivatives. */
+      nir_ssa_def *dfdx = nir_fddx(b, coord);
+      nir_ssa_def *dfdy = nir_fddy(b, coord);
+      nir_ssa_def *fwidth = nir_fadd(b, nir_fabs(b, dfdx), nir_fabs(b, dfdy));
+
+      /* Check if the sum is 0. */
+      is_zero = nir_iand(b, is_zero, nir_feq(b, fwidth, nir_imm_float(b, 0.0)));
+   }
+
+   /* Replace the raw LOD by -FLT_MAX if the sum is 0 for all coordinates. */
+   nir_ssa_def *adjusted_lod =
+      nir_bcsel(b, is_zero, nir_imm_float(b, -FLT_MAX),
+                   nir_channel(b, &tex->dest.ssa, 1));
+
+   nir_ssa_def *def =
+      nir_vec2(b, nir_channel(b, &tex->dest.ssa, 0), adjusted_lod);
+
+   nir_ssa_def_rewrite_uses_after(&tex->dest.ssa, def, def->parent_instr);
+}
+
 static bool
 nir_lower_tex_block(nir_block *block, nir_builder *b,
                     const nir_lower_tex_options *options,
@@ -1206,7 +1321,9 @@ nir_lower_tex_block(nir_block *block, nir_builder *b,
       if ((tex->op == nir_texop_txf && options->lower_txf_offset) ||
           (sat_mask && nir_tex_instr_src_index(tex, nir_tex_src_coord) >= 0) ||
           (tex->sampler_dim == GLSL_SAMPLER_DIM_RECT &&
-           options->lower_rect_offset)) {
+           options->lower_rect_offset) ||
+          (options->lower_offset_filter &&
+           options->lower_offset_filter(instr, options->callback_data))) {
          progress = lower_offset(b, tex) || progress;
       }
 
@@ -1228,7 +1345,7 @@ nir_lower_tex_block(nir_block *block, nir_builder *b,
          nir_deref_instr *deref = nir_src_as_deref(tex->src[tex_index].src);
          nir_variable *var = nir_deref_instr_get_variable(deref);
          texture_index = var ? var->data.binding : 0;
-         texture_mask = var ? (1u << texture_index) : 0u;
+         texture_mask = var && texture_index < 32 ? (1u << texture_index) : 0u;
       }
 
       if (texture_mask & options->lower_y_uv_external) {
@@ -1354,13 +1471,8 @@ nir_lower_tex_block(nir_block *block, nir_builder *b,
        * derivatives.  Lower those opcodes which use implicit derivatives to
        * use an explicit LOD of 0.
        */
-      bool shader_supports_implicit_lod =
-         b->shader->info.stage == MESA_SHADER_FRAGMENT ||
-         (b->shader->info.stage == MESA_SHADER_COMPUTE &&
-          b->shader->info.cs.derivative_group != DERIVATIVE_GROUP_NONE);
-
       if (nir_tex_instr_has_implicit_derivative(tex) &&
-          !shader_supports_implicit_lod) {
+          !nir_shader_supports_implicit_lod(b->shader)) {
          lower_zero_lod(b, tex);
          progress = true;
       }
@@ -1370,6 +1482,13 @@ nir_lower_tex_block(nir_block *block, nir_builder *b,
          continue;
       }
 
+      if (options->lower_txs_cube_array && tex->op == nir_texop_txs &&
+          tex->sampler_dim == GLSL_SAMPLER_DIM_CUBE && tex->is_array) {
+         nir_lower_txs_cube_array(b, tex);
+         progress = true;
+         continue;
+      }
+
       /* has to happen after all the other lowerings as the original tg4 gets
        * replaced by 4 tg4 instructions.
        */
@@ -1379,6 +1498,24 @@ nir_lower_tex_block(nir_block *block, nir_builder *b,
          progress |= lower_tg4_offsets(b, tex);
          continue;
       }
+
+      if (options->lower_to_fragment_fetch_amd && tex->op == nir_texop_txf_ms) {
+         nir_lower_ms_txf_to_fragment_fetch(b, tex);
+         progress = true;
+         continue;
+      }
+
+      if (options->lower_to_fragment_fetch_amd && tex->op == nir_texop_samples_identical) {
+         nir_lower_samples_identical_to_fragment_fetch(b, tex);
+         progress = true;
+         continue;
+      }
+
+      if (options->lower_lod_zero_width && tex->op == nir_texop_lod) {
+         nir_lower_lod_zero_width(b, tex);
+         progress = true;
+         continue;
+      }
    }
 
    return progress;
