@@ -115,6 +115,12 @@ ir3_should_double_threadsize(struct ir3_shader_variant *v, unsigned regs_count)
 {
    const struct ir3_compiler *compiler = v->shader->compiler;
 
+   /* If the user forced a particular wavesize respect that. */
+   if (v->shader->real_wavesize == IR3_SINGLE_ONLY)
+      return false;
+   if (v->shader->real_wavesize == IR3_DOUBLE_ONLY)
+      return true;
+
    /* We can't support more than compiler->branchstack_size diverging threads
     * in a wave. Thus, doubling the threadsize is only possible if we don't
     * exceed the branchstack size limit.
@@ -125,6 +131,7 @@ ir3_should_double_threadsize(struct ir3_shader_variant *v, unsigned regs_count)
    }
 
    switch (v->type) {
+   case MESA_SHADER_KERNEL:
    case MESA_SHADER_COMPUTE: {
       unsigned threads_per_wg =
          v->local_size[0] * v->local_size[1] * v->local_size[2];
@@ -135,7 +142,7 @@ ir3_should_double_threadsize(struct ir3_shader_variant *v, unsigned regs_count)
        * fit. For smaller workgroup sizes, we follow the blob and use the
        * smaller threadsize.
        */
-      if (compiler->gpu_id < 600) {
+      if (compiler->gen < 6) {
          return v->local_size_variable ||
                 threads_per_wg >
                    compiler->threadsize_base * compiler->max_waves;
@@ -176,23 +183,6 @@ ir3_get_reg_independent_max_waves(struct ir3_shader_variant *v,
    const struct ir3_compiler *compiler = v->shader->compiler;
    unsigned max_waves = compiler->max_waves;
 
-   /* If this is a compute shader, compute the limit based on shared size */
-   if (v->type == MESA_SHADER_COMPUTE) {
-      /* Shared is allocated in chunks of 1k */
-      unsigned shared_per_wg = ALIGN_POT(v->shared_size, 1024);
-      if (shared_per_wg > 0 && !v->local_size_variable) {
-         unsigned wgs_per_core = compiler->local_mem_size / shared_per_wg;
-         unsigned threads_per_wg =
-            v->local_size[0] * v->local_size[1] * v->local_size[2];
-         unsigned waves_per_wg =
-            DIV_ROUND_UP(threads_per_wg, compiler->threadsize_base *
-                                            (double_threadsize ? 2 : 1) *
-                                            compiler->wave_granularity);
-         max_waves = MIN2(max_waves, waves_per_wg * wgs_per_core *
-                                        compiler->wave_granularity);
-      }
-   }
-
    /* Compute the limit based on branchstack */
    if (v->branchstack > 0) {
       unsigned branchstack_max_waves = compiler->branchstack_size /
@@ -201,6 +191,42 @@ ir3_get_reg_independent_max_waves(struct ir3_shader_variant *v,
       max_waves = MIN2(max_waves, branchstack_max_waves);
    }
 
+   /* If this is a compute shader, compute the limit based on shared size */
+   if ((v->type == MESA_SHADER_COMPUTE) ||
+       (v->type == MESA_SHADER_KERNEL)) {
+      unsigned threads_per_wg =
+         v->local_size[0] * v->local_size[1] * v->local_size[2];
+      unsigned waves_per_wg =
+         DIV_ROUND_UP(threads_per_wg, compiler->threadsize_base *
+                                         (double_threadsize ? 2 : 1) *
+                                         compiler->wave_granularity);
+
+      /* Shared is allocated in chunks of 1k */
+      unsigned shared_per_wg = ALIGN_POT(v->shared_size, 1024);
+      if (shared_per_wg > 0 && !v->local_size_variable) {
+         unsigned wgs_per_core = compiler->local_mem_size / shared_per_wg;
+
+         max_waves = MIN2(max_waves, waves_per_wg * wgs_per_core *
+                                        compiler->wave_granularity);
+      }
+
+      /* If we have a compute shader that has a big workgroup, a barrier, and
+       * a branchstack which limits max_waves - this may result in a situation
+       * when we cannot run concurrently all waves of the workgroup, which
+       * would lead to a hang.
+       *
+       * TODO: Could we spill branchstack or is there other way around?
+       * Blob just explodes in such case.
+       */
+      if (v->has_barrier && (max_waves < waves_per_wg)) {
+         mesa_loge(
+            "Compute shader (%s:%s) which has workgroup barrier cannot be used "
+            "because it's impossible to have enough concurrent waves.",
+            v->shader->nir->info.name, v->shader->nir->info.label);
+         exit(1);
+      }
+   }
+
    return max_waves;
 }
 
@@ -247,7 +273,7 @@ ir3_collect_info(struct ir3_shader_variant *v)
    info->sizedwords = info->size / 4;
 
    foreach_block (block, &shader->block_list) {
-      int sfu_delay = 0;
+      int sfu_delay = 0, mem_delay = 0;
 
       foreach_instr (instr, &block->instr_list) {
 
@@ -262,14 +288,19 @@ ir3_collect_info(struct ir3_shader_variant *v)
          }
 
          if ((instr->opc == OPC_STP || instr->opc == OPC_LDP)) {
-            struct ir3_register *base =
-               (instr->opc == OPC_STP) ? instr->srcs[2] : instr->srcs[1];
-            if (base->iim_val * type_size(instr->cat6.type) > 32) {
+            unsigned components = instr->srcs[2]->uim_val;
+            if (components * type_size(instr->cat6.type) > 32) {
                info->multi_dword_ldp_stp = true;
             }
+
+            if (instr->opc == OPC_STP)
+               info->stp_count += components;
+            else
+               info->ldp_count += components;
          }
 
-         if ((instr->opc == OPC_BARY_F) && (instr->dsts[0]->flags & IR3_REG_EI))
+         if ((instr->opc == OPC_BARY_F || instr->opc == OPC_FLAT_B) &&
+             (instr->dsts[0]->flags & IR3_REG_EI))
             info->last_baryf = info->instrs_count;
 
          unsigned instrs_count = 1 + instr->repeat + instr->nop;
@@ -300,15 +331,25 @@ ir3_collect_info(struct ir3_shader_variant *v)
             sfu_delay = 0;
          }
 
-         if (instr->flags & IR3_INSTR_SY)
+         if (instr->flags & IR3_INSTR_SY) {
             info->sy++;
+            info->systall += mem_delay;
+            mem_delay = 0;
+         }
 
-         if (is_sfu(instr)) {
-            sfu_delay = 10;
+         if (is_ss_producer(instr)) {
+            sfu_delay = soft_ss_delay(instr);
          } else {
             int n = MIN2(sfu_delay, 1 + instr->repeat + instr->nop);
             sfu_delay -= n;
          }
+
+         if (is_sy_producer(instr)) {
+            mem_delay = soft_sy_delay(instr, shader);
+         } else {
+            int n = MIN2(mem_delay, 1 + instr->repeat + instr->nop);
+            mem_delay -= n;
+         }
       }
    }
 
@@ -317,7 +358,7 @@ ir3_collect_info(struct ir3_shader_variant *v)
     */
    unsigned regs_count =
       info->max_reg + 1 +
-      (compiler->gpu_id >= 600 ? ((info->max_half_reg + 2) / 2) : 0);
+      (compiler->gen >= 6 ? ((info->max_half_reg + 2) / 2) : 0);
 
    info->double_threadsize = ir3_should_double_threadsize(v, regs_count);
    unsigned reg_independent_max_waves =
@@ -393,6 +434,22 @@ ir3_block_remove_predecessor(struct ir3_block *block, struct ir3_block *pred)
    }
 }
 
+void
+ir3_block_remove_physical_predecessor(struct ir3_block *block, struct ir3_block *pred)
+{
+   for (unsigned i = 0; i < block->physical_predecessors_count; i++) {
+      if (block->physical_predecessors[i] == pred) {
+         if (i < block->physical_predecessors_count - 1) {
+            block->physical_predecessors[i] =
+               block->physical_predecessors[block->physical_predecessors_count - 1];
+         }
+
+         block->physical_predecessors_count--;
+         return;
+      }
+   }
+}
+
 unsigned
 ir3_block_get_pred_index(struct ir3_block *block, struct ir3_block *pred)
 {
@@ -470,6 +527,11 @@ ir3_instr_clone(struct ir3_instruction *instr)
       *new_reg = *reg;
    }
 
+   if (instr->address) {
+      assert(instr->srcs_count > 0);
+      new_instr->address = new_instr->srcs[instr->srcs_count - 1];
+   }
+
    return new_instr;
 }
 
@@ -684,6 +746,9 @@ ir3_set_dst_type(struct ir3_instruction *instr, bool half)
 void
 ir3_fixup_src_type(struct ir3_instruction *instr)
 {
+   if (instr->srcs_count == 0)
+      return;
+
    switch (opc_cat(instr->opc)) {
    case 1: /* move instructions */
       if (instr->srcs[0]->flags & IR3_REG_HALF) {
@@ -781,7 +846,7 @@ ir3_valid_flags(struct ir3_instruction *instr, unsigned n, unsigned flags)
        * same block (since we can't propagate address register values
        * across blocks currently)
        */
-      if (compiler->gpu_id < 600)
+      if (compiler->gen < 6)
          return false;
 
       /* NOTE in the special try_swap_mad_two_srcs() case we can be
@@ -833,6 +898,11 @@ ir3_valid_flags(struct ir3_instruction *instr, unsigned n, unsigned flags)
       if (flags & ~valid_flags)
          return false;
 
+      /* Allow an immediate src1 for flat.b, since it's ignored */
+      if (instr->opc == OPC_FLAT_B &&
+          n == 1 && flags == IR3_REG_IMMED)
+         return true;
+
       if (flags & (IR3_REG_CONST | IR3_REG_IMMED | IR3_REG_SHARED)) {
          unsigned m = n ^ 1;
          /* cannot deal w/ const or shared in both srcs:
@@ -852,12 +922,29 @@ ir3_valid_flags(struct ir3_instruction *instr, unsigned n, unsigned flags)
       valid_flags =
          ir3_cat3_absneg(instr->opc) | IR3_REG_RELATIV | IR3_REG_SHARED;
 
-      if (instr->opc == OPC_SHLG_B16) {
+      switch (instr->opc) {
+      case OPC_SHRM:
+      case OPC_SHLM:
+      case OPC_SHRG:
+      case OPC_SHLG:
+      case OPC_ANDG: {
          valid_flags |= IR3_REG_IMMED;
-         /* shlg.b16 can be RELATIV+CONST but not CONST: */
+         /* Can be RELATIV+CONST but not CONST: */
          if (flags & IR3_REG_RELATIV)
             valid_flags |= IR3_REG_CONST;
-      } else {
+         break;
+      }
+      case OPC_WMM:
+      case OPC_WMM_ACCU: {
+         valid_flags = IR3_REG_SHARED;
+         if (n == 2)
+            valid_flags = IR3_REG_CONST;
+         break;
+      }
+      case OPC_DP2ACC:
+      case OPC_DP4ACC:
+         break;
+      default:
          valid_flags |= IR3_REG_CONST;
       }
 
@@ -921,10 +1008,11 @@ ir3_valid_flags(struct ir3_instruction *instr, unsigned n, unsigned flags)
          /* disallow immediates in anything but the SSBO slot argument for
           * cat6 instructions:
           */
-         if (is_atomic(instr->opc) && (n != 0))
+         if (is_global_a3xx_atomic(instr->opc) && (n != 0))
             return false;
 
-         if (is_atomic(instr->opc) && !(instr->flags & IR3_INSTR_G))
+         if (is_local_atomic(instr->opc) || is_global_a6xx_atomic(instr->opc) ||
+             is_bindless_atomic(instr->opc))
             return false;
 
          if (instr->opc == OPC_STG && (n == 2))
@@ -933,13 +1021,18 @@ ir3_valid_flags(struct ir3_instruction *instr, unsigned n, unsigned flags)
          if (instr->opc == OPC_STG_A && (n == 4))
             return false;
 
+         if (instr->opc == OPC_LDG && (n == 0))
+            return false;
+
+         if (instr->opc == OPC_LDG_A && (n < 2))
+            return false;
+
          /* as with atomics, these cat6 instrs can only have an immediate
           * for SSBO/IBO slot argument
           */
          switch (instr->opc) {
          case OPC_LDIB:
          case OPC_STIB:
-         case OPC_LDC:
          case OPC_RESINFO:
             if (n != 0)
                return false;
@@ -954,3 +1047,39 @@ ir3_valid_flags(struct ir3_instruction *instr, unsigned n, unsigned flags)
 
    return true;
 }
+
+bool
+ir3_valid_immediate(struct ir3_instruction *instr, int32_t immed)
+{
+   if (instr->opc == OPC_MOV || is_meta(instr))
+      return true;
+
+   if (is_mem(instr)) {
+      switch (instr->opc) {
+      /* Some load/store instructions have a 13-bit offset and size which must
+       * always be an immediate and the rest of the sources cannot be
+       * immediates, so the frontend is responsible for checking the size:
+       */
+      case OPC_LDL:
+      case OPC_STL:
+      case OPC_LDP:
+      case OPC_STP:
+      case OPC_LDG:
+      case OPC_STG:
+      case OPC_SPILL_MACRO:
+      case OPC_RELOAD_MACRO:
+      case OPC_LDG_A:
+      case OPC_STG_A:
+      case OPC_LDLW:
+      case OPC_STLW:
+      case OPC_LDLV:
+         return true;
+      default:
+         /* most cat6 src immediates can only encode 8 bits: */
+         return !(immed & ~0xff);
+      }
+   }
+
+   /* Other than cat1 (mov) we can only encode up to 10 bits, sign-extended: */
+   return !(immed & ~0x1ff) || !(-immed & ~0x1ff);
+}
