@@ -90,6 +90,8 @@ enum pipe_error cso_set_vertex_elements(struct cso_context *ctx,
 
 void cso_set_vertex_buffers(struct cso_context *ctx,
                             unsigned start_slot, unsigned count,
+                            unsigned unbind_trailing_count,
+                            bool take_ownership,
                             const struct pipe_vertex_buffer *buffers);
 
 void cso_set_stream_outputs(struct cso_context *ctx,
@@ -98,6 +100,15 @@ void cso_set_stream_outputs(struct cso_context *ctx,
                             const unsigned *offsets);
 
 
+enum cso_unbind_flags {
+   CSO_UNBIND_FS_SAMPLERVIEWS = (1 << 0),
+   CSO_UNBIND_FS_SAMPLERVIEW0 = (1 << 1),
+   CSO_UNBIND_FS_IMAGE0 = (1 << 2),
+   CSO_UNBIND_VS_CONSTANTS = (1 << 3),
+   CSO_UNBIND_FS_CONSTANTS = (1 << 4),
+   CSO_UNBIND_VERTEX_BUFFER0 = (1 << 5),
+};
+
 /*
  * We don't provide shader caching in CSO.  Most of the time the api provides
  * object semantics for shaders anyway, and the cases where it doesn't
@@ -165,7 +176,7 @@ void cso_set_render_condition(struct cso_context *cso,
 #define CSO_BIT_COMPUTE_SAMPLERS (1<<1)
 
 void cso_save_state(struct cso_context *cso, unsigned state_mask);
-void cso_restore_state(struct cso_context *cso);
+void cso_restore_state(struct cso_context *cso, unsigned unbind);
 
 void cso_save_compute_state(struct cso_context *cso, unsigned state_mask);
 void cso_restore_compute_state(struct cso_context *cso);
