@@ -140,7 +140,8 @@ struct draw_vertex_shader {
                        const unsigned const_size[PIPE_MAX_CONSTANT_BUFFERS],
 		       unsigned count,
 		       unsigned input_stride,
-		       unsigned output_stride );
+		       unsigned output_stride,
+		       const unsigned *fetch_elts);
 
 
    void (*delete)( struct draw_vertex_shader * );
@@ -163,7 +164,7 @@ draw_create_vs_exec(struct draw_context *draw,
 struct draw_vs_variant_key;
 struct draw_vertex_shader;
 
-#if HAVE_LLVM
+#ifdef DRAW_LLVM_AVAILABLE
 struct draw_vertex_shader *
 draw_create_vs_llvm(struct draw_context *draw,
 		    const struct pipe_shader_state *state);
