@@ -29,7 +29,9 @@
 #include "util/u_memory.h"
 #include "util/u_inlines.h"
 #include "util/format/u_format.h"
+#include "util/u_helpers.h"
 #include "util/u_upload_mgr.h"
+#include "util/u_threaded_context.h"
 #include "noop_public.h"
 
 DEBUG_GET_ONCE_BOOL_OPTION(noop, "GALLIUM_NOOP", false)
@@ -39,12 +41,14 @@ void noop_init_state_functions(struct pipe_context *ctx);
 struct noop_pipe_screen {
    struct pipe_screen	pscreen;
    struct pipe_screen	*oscreen;
+   struct slab_parent_pool pool_transfers;
 };
 
 /*
  * query
  */
 struct noop_query {
+   struct threaded_query b;
    unsigned	query;
 };
 static struct pipe_query *noop_create_query(struct pipe_context *ctx, unsigned query_type, unsigned index)
@@ -90,7 +94,7 @@ noop_set_active_query_state(struct pipe_context *pipe, bool enable)
  * resource
  */
 struct noop_resource {
-   struct pipe_resource	base;
+   struct threaded_resource b;
    unsigned		size;
    char			*data;
    struct sw_displaytarget	*dt;
@@ -107,16 +111,34 @@ static struct pipe_resource *noop_resource_create(struct pipe_screen *screen,
       return NULL;
 
    stride = util_format_get_stride(templ->format, templ->width0);
-   nresource->base = *templ;
-   nresource->base.screen = screen;
+   nresource->b.b = *templ;
+   nresource->b.b.screen = screen;
    nresource->size = stride * templ->height0 * templ->depth0;
    nresource->data = MALLOC(nresource->size);
-   pipe_reference_init(&nresource->base.reference, 1);
+   pipe_reference_init(&nresource->b.b.reference, 1);
    if (nresource->data == NULL) {
       FREE(nresource);
       return NULL;
    }
-   return &nresource->base;
+   threaded_resource_init(&nresource->b.b, false, 0);
+   return &nresource->b.b;
+}
+
+static struct pipe_resource *
+noop_resource_create_with_modifiers(struct pipe_screen *screen,
+                                    const struct pipe_resource *templ,
+                                    const uint64_t *modifiers, int count)
+{
+   struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)screen;
+   struct pipe_screen *oscreen = noop_screen->oscreen;
+   struct pipe_resource *result;
+   struct pipe_resource *noop_resource;
+
+   result = oscreen->resource_create_with_modifiers(oscreen, templ,
+                                                    modifiers, count);
+   noop_resource = noop_resource_create(screen, result);
+   pipe_resource_reference(&result, NULL);
+   return noop_resource;
 }
 
 static struct pipe_resource *noop_resource_from_handle(struct pipe_screen *screen,
@@ -187,6 +209,7 @@ static void noop_resource_destroy(struct pipe_screen *screen,
 {
    struct noop_resource *nresource = (struct noop_resource *)resource;
 
+   threaded_resource_deinit(resource);
    FREE(nresource->data);
    FREE(resource);
 }
@@ -205,7 +228,7 @@ static void *noop_transfer_map(struct pipe_context *pipe,
    struct pipe_transfer *transfer;
    struct noop_resource *nresource = (struct noop_resource *)resource;
 
-   transfer = CALLOC_STRUCT(pipe_transfer);
+   transfer = (struct pipe_transfer*)CALLOC_STRUCT(threaded_transfer);
    if (!transfer)
       return NULL;
    pipe_resource_reference(&transfer->resource, resource);
@@ -310,8 +333,13 @@ static void noop_flush(struct pipe_context *ctx,
                        struct pipe_fence_handle **fence,
                        unsigned flags)
 {
-   if (fence)
-      *fence = NULL;
+   if (fence) {
+      struct pipe_reference *f = MALLOC_STRUCT(pipe_reference);
+      f->count = 1;
+
+      ctx->screen->fence_reference(ctx->screen, fence, NULL);
+      *fence = (struct pipe_fence_handle*)f;
+   }
 }
 
 static void noop_destroy_context(struct pipe_context *ctx)
@@ -319,6 +347,7 @@ static void noop_destroy_context(struct pipe_context *ctx)
    if (ctx->stream_uploader)
       u_upload_destroy(ctx->stream_uploader);
 
+   p_atomic_dec(&ctx->screen->num_contexts);
    FREE(ctx);
 }
 
@@ -348,6 +377,32 @@ static void noop_set_frontend_noop(struct pipe_context *ctx, bool enable)
 {
 }
 
+static void noop_replace_buffer_storage(struct pipe_context *ctx,
+                                        struct pipe_resource *dst,
+                                        struct pipe_resource *src,
+                                        unsigned num_rebinds,
+                                        uint32_t rebind_mask,
+                                        uint32_t delete_buffer_id)
+{
+}
+
+static struct pipe_fence_handle *
+noop_create_fence(struct pipe_context *ctx,
+                  struct tc_unflushed_batch_token *tc_token)
+{
+   struct pipe_reference *f = MALLOC_STRUCT(pipe_reference);
+
+   f->count = 1;
+   return (struct pipe_fence_handle*)f;
+}
+
+static bool noop_is_resource_busy(struct pipe_screen *screen,
+                                  struct pipe_resource *resource,
+                                  unsigned usage)
+{
+   return false;
+}
+
 static struct pipe_context *noop_create_context(struct pipe_screen *screen,
                                                 void *priv, unsigned flags)
 {
@@ -393,7 +448,25 @@ static struct pipe_context *noop_create_context(struct pipe_screen *screen,
    ctx->set_frontend_noop = noop_set_frontend_noop;
    noop_init_state_functions(ctx);
 
-   return ctx;
+   p_atomic_inc(&screen->num_contexts);
+
+   if (!(flags & PIPE_CONTEXT_PREFER_THREADED))
+      return ctx;
+
+   struct pipe_context *tc =
+      threaded_context_create(ctx,
+                              &((struct noop_pipe_screen*)screen)->pool_transfers,
+                              noop_replace_buffer_storage,
+                              &(struct threaded_context_options) {
+                                 .create_fence = noop_create_fence,
+                                 .is_resource_busy = noop_is_resource_busy,
+                              },
+                              NULL);
+
+   if (tc && tc != ctx)
+      threaded_context_init_bytes_mapped_limit((struct threaded_context *)tc, 4);
+
+   return tc;
 }
 
 
@@ -481,6 +554,7 @@ static void noop_destroy_screen(struct pipe_screen *screen)
    struct pipe_screen *oscreen = noop_screen->oscreen;
 
    oscreen->destroy(oscreen);
+   slab_destroy_parent(&noop_screen->pool_transfers);
    FREE(screen);
 }
 
@@ -488,6 +562,11 @@ static void noop_fence_reference(struct pipe_screen *screen,
                           struct pipe_fence_handle **ptr,
                           struct pipe_fence_handle *fence)
 {
+   if (pipe_reference((struct pipe_reference*)*ptr,
+                      (struct pipe_reference*)fence))
+      FREE(*ptr);
+
+   *ptr = fence;
 }
 
 static bool noop_fence_finish(struct pipe_screen *screen,
@@ -523,11 +602,118 @@ static const void *noop_get_compiler_options(struct pipe_screen *pscreen,
    return screen->get_compiler_options(screen, ir, shader);
 }
 
-static void noop_finalize_nir(struct pipe_screen *pscreen, void *nir, bool optimize)
+static char *noop_finalize_nir(struct pipe_screen *pscreen, void *nir)
 {
    struct pipe_screen *screen = ((struct noop_pipe_screen*)pscreen)->oscreen;
 
-   screen->finalize_nir(screen, nir, optimize);
+   return screen->finalize_nir(screen, nir);
+}
+
+static bool noop_check_resource_capability(struct pipe_screen *screen,
+                                           struct pipe_resource *resource,
+                                           unsigned bind)
+{
+   return true;
+}
+
+static void noop_set_max_shader_compiler_threads(struct pipe_screen *screen,
+                                                 unsigned max_threads)
+{
+}
+
+static bool noop_is_parallel_shader_compilation_finished(struct pipe_screen *screen,
+                                                         void *shader,
+                                                         unsigned shader_type)
+{
+   return true;
+}
+
+static bool noop_is_dmabuf_modifier_supported(struct pipe_screen *screen,
+                                              uint64_t modifier, enum pipe_format format,
+                                              bool *external_only)
+{
+   struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)screen;
+   struct pipe_screen *oscreen = noop_screen->oscreen;
+
+   return oscreen->is_dmabuf_modifier_supported(oscreen, modifier, format, external_only);
+}
+
+static unsigned int noop_get_dmabuf_modifier_planes(struct pipe_screen *screen,
+                                                    uint64_t modifier,
+                                                    enum pipe_format format)
+{
+   struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)screen;
+   struct pipe_screen *oscreen = noop_screen->oscreen;
+
+   return oscreen->get_dmabuf_modifier_planes(oscreen, modifier, format);
+}
+
+static void noop_get_driver_uuid(struct pipe_screen *screen, char *uuid)
+{
+   struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)screen;
+   struct pipe_screen *oscreen = noop_screen->oscreen;
+
+   oscreen->get_driver_uuid(oscreen, uuid);
+}
+
+static void noop_get_device_uuid(struct pipe_screen *screen, char *uuid)
+{
+   struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)screen;
+   struct pipe_screen *oscreen = noop_screen->oscreen;
+
+   oscreen->get_device_uuid(oscreen, uuid);
+}
+
+static int noop_get_sparse_texture_virtual_page_size(struct pipe_screen *screen,
+                                                     enum pipe_texture_target target,
+                                                     bool multi_sample,
+                                                     enum pipe_format format,
+                                                     unsigned offset, unsigned size,
+                                                     int *x, int *y, int *z)
+{
+   struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)screen;
+   struct pipe_screen *oscreen = noop_screen->oscreen;
+
+   return oscreen->get_sparse_texture_virtual_page_size(screen, target, multi_sample,
+                                                        format, offset, size, x, y, z);
+}
+
+static void noop_query_dmabuf_modifiers(struct pipe_screen *screen,
+                                        enum pipe_format format, int max,
+                                        uint64_t *modifiers,
+                                        unsigned int *external_only, int *count)
+{
+   struct noop_pipe_screen *noop_screen = (struct noop_pipe_screen*)screen;
+   struct pipe_screen *oscreen = noop_screen->oscreen;
+
+   oscreen->query_dmabuf_modifiers(oscreen, format, max, modifiers,
+                                   external_only, count);
+}
+
+static struct pipe_vertex_state *
+noop_create_vertex_state(struct pipe_screen *screen,
+                         struct pipe_vertex_buffer *buffer,
+                         const struct pipe_vertex_element *elements,
+                         unsigned num_elements,
+                         struct pipe_resource *indexbuf,
+                         uint32_t full_velem_mask)
+{
+   struct pipe_vertex_state *state = CALLOC_STRUCT(pipe_vertex_state);
+
+   if (!state)
+      return NULL;
+
+   util_init_pipe_vertex_state(screen, buffer, elements, num_elements, indexbuf,
+                               full_velem_mask, state);
+   return state;
+}
+
+static void noop_vertex_state_destroy(struct pipe_screen *screen,
+                                      struct pipe_vertex_state *state)
+{
+   pipe_vertex_buffer_unreference(&state->input.vbuffer);
+   pipe_resource_reference(&state->input.indexbuf, NULL);
+   FREE(state);
 }
 
 struct pipe_screen *noop_screen_create(struct pipe_screen *oscreen)
@@ -570,6 +756,22 @@ struct pipe_screen *noop_screen_create(struct pipe_screen *oscreen)
    screen->get_disk_shader_cache = noop_get_disk_shader_cache;
    screen->get_compiler_options = noop_get_compiler_options;
    screen->finalize_nir = noop_finalize_nir;
+   screen->check_resource_capability = noop_check_resource_capability;
+   screen->set_max_shader_compiler_threads = noop_set_max_shader_compiler_threads;
+   screen->is_parallel_shader_compilation_finished = noop_is_parallel_shader_compilation_finished;
+   screen->is_dmabuf_modifier_supported = noop_is_dmabuf_modifier_supported;
+   screen->get_dmabuf_modifier_planes = noop_get_dmabuf_modifier_planes;
+   screen->get_driver_uuid = noop_get_driver_uuid;
+   screen->get_device_uuid = noop_get_device_uuid;
+   screen->query_dmabuf_modifiers = noop_query_dmabuf_modifiers;
+   screen->resource_create_with_modifiers = noop_resource_create_with_modifiers;
+   screen->create_vertex_state = noop_create_vertex_state;
+   screen->vertex_state_destroy = noop_vertex_state_destroy;
+   if (oscreen->get_sparse_texture_virtual_page_size)
+      screen->get_sparse_texture_virtual_page_size = noop_get_sparse_texture_virtual_page_size;
+
+   slab_create_parent(&noop_screen->pool_transfers,
+                      sizeof(struct pipe_transfer), 64);
 
    return screen;
 }
