@@ -77,6 +77,16 @@ rbug_screen_get_device_vendor(struct pipe_screen *_screen)
    return screen->get_device_vendor(screen);
 }
 
+static const void *
+rbug_screen_get_compiler_options(struct pipe_screen *_screen,
+                                 enum pipe_shader_ir ir,
+                                 enum pipe_shader_type shader)
+{
+   struct pipe_screen *screen = rbug_screen(_screen)->screen;
+
+   return screen->get_compiler_options(screen, ir, shader);
+}
+
 static struct disk_cache *
 rbug_screen_get_disk_shader_cache(struct pipe_screen *_screen)
 {
@@ -155,6 +165,47 @@ rbug_screen_query_dmabuf_modifiers(struct pipe_screen *_screen,
                                   count);
 }
 
+static bool
+rbug_screen_is_dmabuf_modifier_supported(struct pipe_screen *_screen,
+                                         uint64_t modifier,
+                                         enum pipe_format format,
+                                         bool *external_only)
+{
+   struct rbug_screen *rb_screen = rbug_screen(_screen);
+   struct pipe_screen *screen = rb_screen->screen;
+
+   return screen->is_dmabuf_modifier_supported(screen,
+                                               modifier,
+                                               format,
+                                               external_only);
+}
+
+static unsigned int
+rbug_screen_get_dmabuf_modifier_planes(struct pipe_screen *_screen,
+                                       uint64_t modifier,
+                                       enum pipe_format format)
+{
+   struct rbug_screen *rb_screen = rbug_screen(_screen);
+   struct pipe_screen *screen = rb_screen->screen;
+
+   return screen->get_dmabuf_modifier_planes(screen, modifier, format);
+}
+
+static int
+rbug_screen_get_sparse_texture_virtual_page_size(struct pipe_screen *_screen,
+                                                 enum pipe_texture_target target,
+                                                 bool multi_sample,
+                                                 enum pipe_format format,
+                                                 unsigned offset, unsigned size,
+                                                 int *x, int *y, int *z)
+{
+   struct rbug_screen *rb_screen = rbug_screen(_screen);
+   struct pipe_screen *screen = rb_screen->screen;
+
+   return screen->get_sparse_texture_virtual_page_size(screen, target, multi_sample,
+                                                       format, offset, size, x, y, z);
+}
+
 static struct pipe_context *
 rbug_screen_context_create(struct pipe_screen *_screen,
                            void *priv, unsigned flags)
@@ -320,6 +371,7 @@ rbug_screen_resource_destroy(struct pipe_screen *screen,
 
 static void
 rbug_screen_flush_frontbuffer(struct pipe_screen *_screen,
+                              struct pipe_context *_ctx,
                               struct pipe_resource *_resource,
                               unsigned level, unsigned layer,
                               void *context_private, struct pipe_box *sub_box)
@@ -328,8 +380,10 @@ rbug_screen_flush_frontbuffer(struct pipe_screen *_screen,
    struct rbug_resource *rb_resource = rbug_resource(_resource);
    struct pipe_screen *screen = rb_screen->screen;
    struct pipe_resource *resource = rb_resource->resource;
+   struct pipe_context *ctx = _ctx ? rbug_context(_ctx)->pipe : NULL;
 
    screen->flush_frontbuffer(screen,
+                             ctx,
                              resource,
                              level, layer,
                              context_private, sub_box);
@@ -371,12 +425,12 @@ rbug_screen_fence_get_fd(struct pipe_screen *_screen,
    return screen->fence_get_fd(screen, fence);
 }
 
-static void
-rbug_screen_finalize_nir(struct pipe_screen *_screen, void *nir, bool optimize)
+static char *
+rbug_screen_finalize_nir(struct pipe_screen *_screen, void *nir)
 {
    struct pipe_screen *screen = rbug_screen(_screen)->screen;
 
-   screen->finalize_nir(screen, nir, optimize);
+   return screen->finalize_nir(screen, nir);
 }
 
 bool
@@ -409,6 +463,7 @@ rbug_screen_create(struct pipe_screen *screen)
    rb_screen->base.destroy = rbug_screen_destroy;
    rb_screen->base.get_name = rbug_screen_get_name;
    rb_screen->base.get_vendor = rbug_screen_get_vendor;
+   SCR_INIT(get_compiler_options);
    SCR_INIT(get_disk_shader_cache);
    rb_screen->base.get_device_vendor = rbug_screen_get_device_vendor;
    rb_screen->base.get_param = rbug_screen_get_param;
@@ -416,6 +471,8 @@ rbug_screen_create(struct pipe_screen *screen)
    rb_screen->base.get_paramf = rbug_screen_get_paramf;
    rb_screen->base.is_format_supported = rbug_screen_is_format_supported;
    SCR_INIT(query_dmabuf_modifiers);
+   SCR_INIT(is_dmabuf_modifier_supported);
+   SCR_INIT(get_dmabuf_modifier_planes);
    rb_screen->base.context_create = rbug_screen_context_create;
    SCR_INIT(can_create_resource);
    rb_screen->base.resource_create = rbug_screen_resource_create;
@@ -432,6 +489,7 @@ rbug_screen_create(struct pipe_screen *screen)
    rb_screen->base.fence_finish = rbug_screen_fence_finish;
    rb_screen->base.fence_get_fd = rbug_screen_fence_get_fd;
    SCR_INIT(finalize_nir);
+   SCR_INIT(get_sparse_texture_virtual_page_size);
 
    rb_screen->screen = screen;
 
