@@ -77,8 +77,6 @@
  * - stream_output_target_destroy
  * - transfer_map (only unsychronized buffer mappings)
  * - get_query_result (when threaded_query::flushed == true)
- *
- * Create calls causing a sync that can't be async due to driver limitations:
  * - create_stream_output_target
  *
  *
@@ -106,6 +104,11 @@
  *    TC_TRANSFER_MAP_NO_INVALIDATE into transfer_map and buffer_subdata to
  *    indicate this. Ignoring the flag will lead to failures.
  *    The threaded context uses its own buffer invalidation mechanism.
+ *    Do NOT use pipe_buffer_write, as this may trigger invalidation;
+ *    use tc_buffer_write instead.
+ *
+ * 4) PIPE_MAP_ONCE can no longer be used to infer that a buffer will not be mapped
+ *    a second time before it is unmapped.
  *
  *
  * Rules for fences
@@ -144,53 +147,76 @@
  *    the threaded context wants to replace a resource's backing storage with
  *    another resource's backing storage. The threaded context uses it to
  *    implement buffer invalidation. This call is always queued.
+ *    Note that 'minimum_num_rebinds' specifies only the minimum number of rebinds
+ *    which must be managed by the driver; if a buffer is bound multiple times in
+ *    the same binding point (e.g., vertex buffer slots 0,1,2), this will be counted
+ *    as a single rebind.
  *
  *
- * Performance gotchas
- * -------------------
+ * Optional resource busy callbacks for better performance
+ * -------------------------------------------------------
  *
- * Buffer invalidations are done unconditionally - they don't check whether
- * the buffer is busy. This can cause drivers to have more live allocations
- * and CPU mappings than necessary.
+ * This adds checking whether a resource is used by the GPU and whether
+ * a resource is referenced by an unflushed command buffer. If neither is true,
+ * the threaded context will map the buffer as UNSYNCHRONIZED without flushing
+ * or synchronizing the thread and will skip any buffer invalidations
+ * (reallocations) because invalidating an idle buffer has no benefit.
+ *
+ * There are 1 driver callback and 1 TC callback:
+ *
+ * 1) is_resource_busy: It returns true when a resource is busy. If this is NULL,
+ *    the resource is considered always busy.
+ *
+ * 2) tc_driver_internal_flush_notify: If the driver set
+ *    driver_calls_flush_notify = true in threaded_context_create, it should
+ *    call this after every internal driver flush. The threaded context uses it
+ *    to track internal driver flushes for the purpose of tracking which
+ *    buffers are referenced by an unflushed command buffer.
+ *
+ * If is_resource_busy is set, threaded_resource::buffer_id_unique must be
+ * generated by the driver, and the replace_buffer_storage callback should
+ * delete the buffer ID passed to it. The driver should use
+ * util_idalloc_mt_init_tc.
  *
  *
  * How it works (queue architecture)
  * ---------------------------------
  *
- * There is a multithreaded queue consisting of batches, each batch consisting
- * of call slots. Each call slot consists of an 8-byte header (call ID +
- * call size + constant 32-bit marker for integrity checking) and an 8-byte
- * body for per-call data. That is 16 bytes per call slot.
- *
- * Simple calls such as bind_xx_state(CSO) occupy only one call slot. Bigger
- * calls occupy multiple call slots depending on the size needed by call
- * parameters. That means that calls can have a variable size in the batch.
- * For example, set_vertex_buffers(count = any, buffers = NULL) occupies only
- * 1 call slot, but set_vertex_buffers(count = 5) occupies 6 call slots.
- * Even though the first call slot can use only 8 bytes for data, additional
- * call slots used by the same call can use all 16 bytes for data.
- * For example, a call using 2 call slots has 24 bytes of space for data.
+ * There is a multithreaded queue consisting of batches, each batch containing
+ * 8-byte slots. Calls can occupy 1 or more slots.
  *
  * Once a batch is full and there is no space for the next call, it's flushed,
  * meaning that it's added to the queue for execution in the other thread.
  * The batches are ordered in a ring and reused once they are idle again.
  * The batching is necessary for low queue/mutex overhead.
- *
  */
 
 #ifndef U_THREADED_CONTEXT_H
 #define U_THREADED_CONTEXT_H
 
+#include "c11/threads.h"
 #include "pipe/p_context.h"
 #include "pipe/p_state.h"
+#include "util/bitset.h"
 #include "util/u_inlines.h"
+#include "util/u_memory.h"
 #include "util/u_queue.h"
 #include "util/u_range.h"
+#include "util/u_thread.h"
 #include "util/slab.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 struct threaded_context;
 struct tc_unflushed_batch_token;
 
+/* 0 = disabled, 1 = assertions, 2 = printfs, 3 = logging */
+#define TC_DEBUG 0
+
+/* This is an internal flag not sent to the driver. */
+#define TC_TRANSFER_MAP_UPLOAD_CPU_STORAGE   (1u << 28)
 /* These are map flags sent to drivers. */
 /* Never infer whether it's safe to use unsychronized mappings: */
 #define TC_TRANSFER_MAP_NO_INFER_UNSYNCHRONIZED (1u << 29)
@@ -220,7 +246,20 @@ struct tc_unflushed_batch_token;
  * The idea is to have batches as small as possible but large enough so that
  * the queuing and mutex overhead is negligible.
  */
-#define TC_CALLS_PER_BATCH    768
+#define TC_SLOTS_PER_BATCH    1536
+
+/* The buffer list queue is much deeper than the batch queue because buffer
+ * lists need to stay around until the driver internally flushes its command
+ * buffer.
+ */
+#define TC_MAX_BUFFER_LISTS   (TC_MAX_BATCHES * 4)
+
+/* This mask is used to get a hash of a buffer ID. It's also the bit size of
+ * the buffer list - 1. It must be 2^n - 1. The size should be as low as
+ * possible to minimize memory usage, but high enough to minimize hash
+ * collisions.
+ */
+#define TC_BUFFER_ID_MASK      BITFIELD_MASK(14)
 
 /* Threshold for when to use the queue or sync. */
 #define TC_MAX_STRING_MARKER_BYTES  512
@@ -232,15 +271,49 @@ struct tc_unflushed_batch_token;
  */
 #define TC_MAX_SUBDATA_BYTES        320
 
+enum tc_binding_type {
+   TC_BINDING_VERTEX_BUFFER,
+   TC_BINDING_STREAMOUT_BUFFER,
+   TC_BINDING_UBO_VS,
+   TC_BINDING_UBO_FS,
+   TC_BINDING_UBO_GS,
+   TC_BINDING_UBO_TCS,
+   TC_BINDING_UBO_TES,
+   TC_BINDING_UBO_CS,
+   TC_BINDING_SAMPLERVIEW_VS,
+   TC_BINDING_SAMPLERVIEW_FS,
+   TC_BINDING_SAMPLERVIEW_GS,
+   TC_BINDING_SAMPLERVIEW_TCS,
+   TC_BINDING_SAMPLERVIEW_TES,
+   TC_BINDING_SAMPLERVIEW_CS,
+   TC_BINDING_SSBO_VS,
+   TC_BINDING_SSBO_FS,
+   TC_BINDING_SSBO_GS,
+   TC_BINDING_SSBO_TCS,
+   TC_BINDING_SSBO_TES,
+   TC_BINDING_SSBO_CS,
+   TC_BINDING_IMAGE_VS,
+   TC_BINDING_IMAGE_FS,
+   TC_BINDING_IMAGE_GS,
+   TC_BINDING_IMAGE_TCS,
+   TC_BINDING_IMAGE_TES,
+   TC_BINDING_IMAGE_CS,
+};
+
 typedef void (*tc_replace_buffer_storage_func)(struct pipe_context *ctx,
                                                struct pipe_resource *dst,
-                                               struct pipe_resource *src);
+                                               struct pipe_resource *src,
+                                               unsigned minimum_num_rebinds,
+                                               uint32_t rebind_mask,
+                                               uint32_t delete_buffer_id);
 typedef struct pipe_fence_handle *(*tc_create_fence_func)(struct pipe_context *ctx,
                                                           struct tc_unflushed_batch_token *token);
+typedef bool (*tc_is_resource_busy)(struct pipe_screen *screen,
+                                    struct pipe_resource *resource,
+                                    unsigned usage);
 
 struct threaded_resource {
    struct pipe_resource b;
-   const struct u_resource_vtbl *vtbl;
 
    /* Since buffer invalidations are queued, we can't use the base resource
     * for unsychronized mappings. This points to the latest version of
@@ -249,6 +322,13 @@ struct threaded_resource {
     */
    struct pipe_resource *latest;
 
+   /* Optional CPU storage of the buffer. When we get partial glBufferSubData(implemented by
+    * copy_buffer) + glDrawElements, we don't want to drain the gfx pipeline before executing
+    * the copy. For ideal pipelining, we upload to this CPU storage and then reallocate
+    * the GPU storage completely and reupload everything without copy_buffer.
+    */
+   void *cpu_storage;
+
    /* The buffer range which is initialized (with a write transfer, streamout,
     * or writable shader resources). The remainder of the buffer is considered
     * invalid and can be mapped unsynchronized.
@@ -262,28 +342,21 @@ struct threaded_resource {
     */
    struct util_range valid_buffer_range;
 
-   /* If "this" is not the base instance of the buffer, but it's one of its
-    * reallocations (set in "latest" of the base instance), this points to
-    * the valid range of the base instance. It's used for transfers after
-    * a buffer invalidation, because such transfers operate on "latest", not
-    * the base instance. Initially it's set to &valid_buffer_range.
-    */
-   struct util_range *base_valid_buffer_range;
-
    /* Drivers are required to update this for shared resources and user
     * pointers. */
    bool	is_shared;
    bool is_user_ptr;
 
-   /* If positive, prefer DISCARD_RANGE with a staging buffer over any other
-    * method of CPU access when map flags allow it. Useful for buffers that
-    * are too large for the visible VRAM window.
+   /* Unique buffer ID. Drivers must set it to non-zero for buffers and it must
+    * be unique. Textures must set 0. Low bits are used as a hash of the ID.
+    * Use util_idalloc_mt to generate these IDs.
     */
-   int max_forced_staging_uploads;
+   uint32_t buffer_id_unique;
 
    /* If positive, then a staging transfer is in progress.
     */
    int pending_staging_uploads;
+
    /* If staging uploads are pending, this will hold the union of the mapped
     * ranges.
     */
@@ -296,9 +369,15 @@ struct threaded_transfer {
    /* Staging buffer for DISCARD_RANGE transfers. */
    struct pipe_resource *staging;
 
-   /* Offset into the staging buffer, because the backing buffer is
-    * sub-allocated. */
-   unsigned offset;
+   /* If b.resource is not the base instance of the buffer, but it's one of its
+    * reallocations (set in "latest" of the base instance), this points to
+    * the valid range of the base instance. It's used for transfers after
+    * a buffer invalidation, because such transfers operate on "latest", not
+    * the base instance. Initially it's set to &b.resource->valid_buffer_range.
+    */
+   struct util_range *valid_buffer_range;
+
+   bool cpu_storage_mapped;
 };
 
 struct threaded_query {
@@ -309,25 +388,12 @@ struct threaded_query {
    bool flushed;
 };
 
-/* This is the second half of tc_call containing call data.
- * Most calls will typecast this to the type they need, typically larger
- * than 8 bytes.
- */
-union tc_payload {
-   struct pipe_query *query;
-   struct pipe_resource *resource;
-   struct pipe_transfer *transfer;
-   struct pipe_fence_handle *fence;
-   uint64_t handle;
-   bool boolean;
-};
-
-/* Each call slot should be aligned to its own size for optimal cache usage. */
-struct ALIGN16 tc_call {
-   unsigned sentinel;
-   ushort num_call_slots;
+struct tc_call_base {
+#if !defined(NDEBUG) && TC_DEBUG >= 1
+   uint32_t sentinel;
+#endif
+   ushort num_slots;
    ushort call_id;
-   union tc_payload payload;
 };
 
 /**
@@ -341,12 +407,40 @@ struct tc_unflushed_batch_token {
 };
 
 struct tc_batch {
-   struct pipe_context *pipe;
+   struct threaded_context *tc;
+#if !defined(NDEBUG) && TC_DEBUG >= 1
    unsigned sentinel;
-   unsigned num_total_call_slots;
-   struct tc_unflushed_batch_token *token;
+#endif
+   uint16_t num_total_slots;
+   uint16_t buffer_list_index;
    struct util_queue_fence fence;
-   struct tc_call call[TC_CALLS_PER_BATCH];
+   struct tc_unflushed_batch_token *token;
+   uint64_t slots[TC_SLOTS_PER_BATCH];
+};
+
+struct tc_buffer_list {
+   /* Signalled by the driver after it flushes its internal command buffer. */
+   struct util_queue_fence driver_flushed_fence;
+
+   /* Buffer list where bit N means whether ID hash N is in the list. */
+   BITSET_DECLARE(buffer_list, TC_BUFFER_ID_MASK + 1);
+};
+
+/**
+ * Optional TC parameters/callbacks.
+ */
+struct threaded_context_options {
+   tc_create_fence_func create_fence;
+   tc_is_resource_busy is_resource_busy;
+   bool driver_calls_flush_notify;
+
+   /**
+    * If true, ctx->get_device_reset_status() will be called without
+    * synchronizing with driver thread.  Drivers can enable this to avoid
+    * TC syncs if their implementation of get_device_reset_status() is
+    * safe to call without synchronizing with driver thread.
+    */
+   bool unsynchronized_get_device_reset_status;
 };
 
 struct threaded_context {
@@ -354,7 +448,7 @@ struct threaded_context {
    struct pipe_context *pipe;
    struct slab_child_pool pool_transfers;
    tc_replace_buffer_storage_func replace_buffer_storage;
-   tc_create_fence_func create_fence;
+   struct threaded_context_options options;
    unsigned map_buffer_alignment;
    unsigned ubo_alignment;
 
@@ -366,6 +460,8 @@ struct threaded_context {
    unsigned num_syncs;
 
    bool use_forced_staging_uploads;
+   bool add_all_gfx_bindings_to_buffer_list;
+   bool add_all_compute_bindings_to_buffer_list;
 
    /* Estimation of how much vram/gtt bytes are mmap'd in
     * the current tc_batch.
@@ -376,26 +472,83 @@ struct threaded_context {
    struct util_queue queue;
    struct util_queue_fence *fence;
 
-   unsigned last, next;
+#ifndef NDEBUG
+   /**
+    * The driver thread is normally the queue thread, but
+    * there are cases where the queue is flushed directly
+    * from the frontend thread
+    */
+   thread_id driver_thread;
+#endif
+
+   bool seen_tcs;
+   bool seen_tes;
+   bool seen_gs;
+
+   bool seen_streamout_buffers;
+   bool seen_shader_buffers[PIPE_SHADER_TYPES];
+   bool seen_image_buffers[PIPE_SHADER_TYPES];
+   bool seen_sampler_buffers[PIPE_SHADER_TYPES];
+
+   unsigned max_vertex_buffers;
+   unsigned max_const_buffers;
+   unsigned max_shader_buffers;
+   unsigned max_images;
+   unsigned max_samplers;
+
+   unsigned last, next, next_buf_list;
+
+   /* The list fences that the driver should signal after the next flush.
+    * If this is empty, all driver command buffers have been flushed.
+    */
+   struct util_queue_fence *signal_fences_next_flush[TC_MAX_BUFFER_LISTS];
+   unsigned num_signal_fences_next_flush;
+
+   /* Bound buffers are tracked here using threaded_resource::buffer_id_hash.
+    * 0 means unbound.
+    */
+   uint32_t vertex_buffers[PIPE_MAX_ATTRIBS];
+   uint32_t streamout_buffers[PIPE_MAX_SO_BUFFERS];
+   uint32_t const_buffers[PIPE_SHADER_TYPES][PIPE_MAX_CONSTANT_BUFFERS];
+   uint32_t shader_buffers[PIPE_SHADER_TYPES][PIPE_MAX_SHADER_BUFFERS];
+   uint32_t image_buffers[PIPE_SHADER_TYPES][PIPE_MAX_SHADER_IMAGES];
+   uint32_t shader_buffers_writeable_mask[PIPE_SHADER_TYPES];
+   uint32_t image_buffers_writeable_mask[PIPE_SHADER_TYPES];
+   /* Don't use PIPE_MAX_SHADER_SAMPLER_VIEWS because it's too large. */
+   uint32_t sampler_buffers[PIPE_SHADER_TYPES][PIPE_MAX_SAMPLERS];
+
    struct tc_batch batch_slots[TC_MAX_BATCHES];
+   struct tc_buffer_list buffer_lists[TC_MAX_BUFFER_LISTS];
 };
 
-void threaded_resource_init(struct pipe_resource *res);
+void threaded_resource_init(struct pipe_resource *res, bool allow_cpu_storage,
+                            unsigned map_buffer_alignment);
 void threaded_resource_deinit(struct pipe_resource *res);
 struct pipe_context *threaded_context_unwrap_sync(struct pipe_context *pipe);
+void tc_driver_internal_flush_notify(struct threaded_context *tc);
 
 struct pipe_context *
 threaded_context_create(struct pipe_context *pipe,
                         struct slab_parent_pool *parent_transfer_pool,
                         tc_replace_buffer_storage_func replace_buffer,
-                        tc_create_fence_func create_fence,
+                        const struct threaded_context_options *options,
                         struct threaded_context **out);
 
+void
+threaded_context_init_bytes_mapped_limit(struct threaded_context *tc, unsigned divisor);
+
 void
 threaded_context_flush(struct pipe_context *_pipe,
                        struct tc_unflushed_batch_token *token,
                        bool prefer_async);
 
+void
+tc_draw_vbo(struct pipe_context *_pipe, const struct pipe_draw_info *info,
+            unsigned drawid_offset,
+            const struct pipe_draw_indirect_info *indirect,
+            const struct pipe_draw_start_count_bias *draws,
+            unsigned num_draws);
+
 static inline struct threaded_context *
 threaded_context(struct pipe_context *pipe)
 {
@@ -429,4 +582,51 @@ tc_unflushed_batch_token_reference(struct tc_unflushed_batch_token **dst,
    *dst = src;
 }
 
+/**
+ * Helper for !NDEBUG builds to assert that it is called from driver
+ * thread.  This is to help drivers ensure that various code-paths
+ * are not hit indirectly from pipe entry points that are called from
+ * front-end/state-tracker thread.
+ */
+static inline void
+tc_assert_driver_thread(struct threaded_context *tc)
+{
+   if (!tc)
+      return;
+#ifndef NDEBUG
+   assert(util_thread_id_equal(tc->driver_thread, util_get_thread_id()));
+#endif
+}
+
+/**
+ * This is called before GPU stores to disable the CPU storage because
+ * the CPU storage doesn't mirror the GPU storage.
+ *
+ * Drivers should also call it before exporting a DMABUF of a buffer.
+ */
+static inline void
+tc_buffer_disable_cpu_storage(struct pipe_resource *buf)
+{
+   struct threaded_resource *tres = threaded_resource(buf);
+
+   if (tres->cpu_storage) {
+      align_free(tres->cpu_storage);
+      tres->cpu_storage = NULL;
+   }
+}
+
+static inline void
+tc_buffer_write(struct pipe_context *pipe,
+                struct pipe_resource *buf,
+                unsigned offset,
+                unsigned size,
+                const void *data)
+{
+   pipe->buffer_subdata(pipe, buf, PIPE_MAP_WRITE | TC_TRANSFER_MAP_NO_INVALIDATE, offset, size, data);
+}
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
