@@ -47,6 +47,7 @@ translate_cull_face(unsigned cull_face, unsigned front_ccw)
 {
    switch (cull_face) {
    case PIPE_FACE_NONE:
+   case PIPE_FACE_FRONT_AND_BACK: /* handled in draw_vbo */
       return VIVS_PA_CONFIG_CULL_FACE_MODE_OFF;
    case PIPE_FACE_BACK:
       return front_ccw ? VIVS_PA_CONFIG_CULL_FACE_MODE_CW
@@ -113,26 +114,6 @@ translate_stencil_op(unsigned stencil_op)
    }
 }
 
-static inline uint32_t
-translate_blend(unsigned blend)
-{
-   switch (blend) {
-   case PIPE_BLEND_ADD:
-      return BLEND_EQ_ADD;
-   case PIPE_BLEND_SUBTRACT:
-      return BLEND_EQ_SUBTRACT;
-   case PIPE_BLEND_REVERSE_SUBTRACT:
-      return BLEND_EQ_REVERSE_SUBTRACT;
-   case PIPE_BLEND_MIN:
-      return BLEND_EQ_MIN;
-   case PIPE_BLEND_MAX:
-      return BLEND_EQ_MAX;
-   default:
-      DBG("Unhandled blend: %i", blend);
-      return ETNA_NO_MATCH;
-   }
-}
-
 static inline uint32_t
 translate_blend_factor(unsigned blend_factor)
 {
@@ -227,7 +208,6 @@ translate_texture_filter(unsigned filter)
       return TEXTURE_FILTER_NEAREST;
    case PIPE_TEX_FILTER_LINEAR:
       return TEXTURE_FILTER_LINEAR;
-   /* What about anisotropic? */
    default:
       DBG("Unhandled texture filter: %i", filter);
       return ETNA_NO_MATCH;
