@@ -154,7 +154,7 @@ emit_image_tex(struct fd_ringbuffer *ring, unsigned slot, struct fd5_image *img,
    OUT_RING(ring, A5XX_TEX_CONST_1_WIDTH(img->width) |
                      A5XX_TEX_CONST_1_HEIGHT(img->height));
    OUT_RING(ring,
-            COND(img->buffer, A5XX_TEX_CONST_2_UNK4 | A5XX_TEX_CONST_2_UNK31) |
+            COND(img->buffer, A5XX_TEX_CONST_2_BUFFER) |
                A5XX_TEX_CONST_2_TYPE(img->type) |
                A5XX_TEX_CONST_2_PITCH(img->pitch));
    OUT_RING(ring, A5XX_TEX_CONST_3_ARRAY_PITCH(img->array_pitch));
@@ -182,8 +182,8 @@ emit_image_ssbo(struct fd_ringbuffer *ring, unsigned slot,
                      CP_LOAD_STATE4_0_STATE_SRC(SS4_DIRECT) |
                      CP_LOAD_STATE4_0_STATE_BLOCK(imgsb[shader]) |
                      CP_LOAD_STATE4_0_NUM_UNIT(1));
-   OUT_RING(ring,
-            CP_LOAD_STATE4_1_STATE_TYPE(1) | CP_LOAD_STATE4_1_EXT_SRC_ADDR(0));
+   OUT_RING(ring, CP_LOAD_STATE4_1_STATE_TYPE(ST4_CONSTANTS) |
+                     CP_LOAD_STATE4_1_EXT_SRC_ADDR(0));
    OUT_RING(ring, CP_LOAD_STATE4_2_EXT_SRC_ADDR_HI(0));
    OUT_RING(ring,
             A5XX_SSBO_1_0_FMT(img->fmt) | A5XX_SSBO_1_0_WIDTH(img->width));
@@ -195,8 +195,8 @@ emit_image_ssbo(struct fd_ringbuffer *ring, unsigned slot,
                      CP_LOAD_STATE4_0_STATE_SRC(SS4_DIRECT) |
                      CP_LOAD_STATE4_0_STATE_BLOCK(imgsb[shader]) |
                      CP_LOAD_STATE4_0_NUM_UNIT(1));
-   OUT_RING(ring,
-            CP_LOAD_STATE4_1_STATE_TYPE(2) | CP_LOAD_STATE4_1_EXT_SRC_ADDR(0));
+   OUT_RING(ring, CP_LOAD_STATE4_1_STATE_TYPE(ST4_UBO) |
+                     CP_LOAD_STATE4_1_EXT_SRC_ADDR(0));
    OUT_RING(ring, CP_LOAD_STATE4_2_EXT_SRC_ADDR_HI(0));
    if (img->bo) {
       OUT_RELOC(ring, img->bo, img->offset, 0, 0);
