@@ -76,21 +76,28 @@ fd6_screen_is_format_supported(struct pipe_screen *pscreen,
       return false;
 
    if ((usage & PIPE_BIND_VERTEX_BUFFER) &&
-       (fd6_pipe2vtx(format) != FMT6_NONE)) {
+       (fd6_vertex_format(format) != FMT6_NONE)) {
       retval |= PIPE_BIND_VERTEX_BUFFER;
    }
 
+   bool has_color = fd6_color_format(format, TILE6_LINEAR) != FMT6_NONE;
+   bool has_tex = fd6_texture_format(format, TILE6_LINEAR) != FMT6_NONE;
+
    if ((usage & (PIPE_BIND_SAMPLER_VIEW | PIPE_BIND_SHADER_IMAGE)) &&
-       (fd6_pipe2tex(format) != FMT6_NONE) &&
+       has_tex &&
        (target == PIPE_BUFFER || util_format_get_blocksize(format) != 12)) {
       retval |= usage & (PIPE_BIND_SAMPLER_VIEW | PIPE_BIND_SHADER_IMAGE);
    }
 
+   if (usage & PIPE_BIND_SHADER_IMAGE) {
+      if (sample_count > 1)
+         return false;
+   }
+
    if ((usage &
         (PIPE_BIND_RENDER_TARGET | PIPE_BIND_DISPLAY_TARGET |
          PIPE_BIND_SCANOUT | PIPE_BIND_SHARED | PIPE_BIND_COMPUTE_RESOURCE)) &&
-       (fd6_pipe2color(format) != FMT6_NONE) &&
-       (fd6_pipe2tex(format) != FMT6_NONE)) {
+       has_color && has_tex) {
       retval |= usage & (PIPE_BIND_RENDER_TARGET | PIPE_BIND_DISPLAY_TARGET |
                          PIPE_BIND_SCANOUT | PIPE_BIND_SHARED |
                          PIPE_BIND_COMPUTE_RESOURCE);
@@ -102,8 +109,7 @@ fd6_screen_is_format_supported(struct pipe_screen *pscreen,
    }
 
    if ((usage & PIPE_BIND_DEPTH_STENCIL) &&
-       (fd6_pipe2depth(format) != (enum a6xx_depth_format) ~0) &&
-       (fd6_pipe2tex(format) != FMT6_NONE)) {
+       (fd6_pipe2depth(format) != (enum a6xx_depth_format) ~0) && has_tex) {
       retval |= PIPE_BIND_DEPTH_STENCIL;
    }
 
@@ -121,6 +127,24 @@ fd6_screen_is_format_supported(struct pipe_screen *pscreen,
    return retval == usage;
 }
 
+/* clang-format off */
+static const uint8_t primtypes[] = {
+   [PIPE_PRIM_POINTS]                      = DI_PT_POINTLIST,
+   [PIPE_PRIM_LINES]                       = DI_PT_LINELIST,
+   [PIPE_PRIM_LINE_STRIP]                  = DI_PT_LINESTRIP,
+   [PIPE_PRIM_LINE_LOOP]                   = DI_PT_LINELOOP,
+   [PIPE_PRIM_TRIANGLES]                   = DI_PT_TRILIST,
+   [PIPE_PRIM_TRIANGLE_STRIP]              = DI_PT_TRISTRIP,
+   [PIPE_PRIM_TRIANGLE_FAN]                = DI_PT_TRIFAN,
+   [PIPE_PRIM_LINES_ADJACENCY]             = DI_PT_LINE_ADJ,
+   [PIPE_PRIM_LINE_STRIP_ADJACENCY]        = DI_PT_LINESTRIP_ADJ,
+   [PIPE_PRIM_TRIANGLES_ADJACENCY]         = DI_PT_TRI_ADJ,
+   [PIPE_PRIM_TRIANGLE_STRIP_ADJACENCY]    = DI_PT_TRISTRIP_ADJ,
+   [PIPE_PRIM_PATCHES]                     = DI_PT_PATCHES0,
+   [PIPE_PRIM_MAX]                         = DI_PT_RECTLIST,  /* internal clear blits */
+};
+/* clang-format on */
+
 void
 fd6_screen_init(struct pipe_screen *pscreen)
 {
@@ -128,6 +152,10 @@ fd6_screen_init(struct pipe_screen *pscreen)
 
    screen->max_rts = A6XX_MAX_RENDER_TARGETS;
 
+   screen->ccu_offset_bypass = screen->info->num_ccu * A6XX_CCU_DEPTH_SIZE;
+   screen->ccu_offset_gmem = (screen->gmemsize_bytes -
+         screen->info->num_ccu * A6XX_CCU_GMEM_COLOR_SIZE);
+
    /* Currently only FB_READ forces GMEM path, mostly because we'd have to
     * deal with cmdstream patching otherwise..
     */
@@ -143,4 +171,6 @@ fd6_screen_init(struct pipe_screen *pscreen)
    fd6_resource_screen_init(pscreen);
    fd6_emit_init_screen(pscreen);
    ir3_screen_init(pscreen);
+
+   screen->primtypes = primtypes;
 }
