@@ -80,9 +80,11 @@ struct fd_screen {
 
    uint64_t gmem_base;
    uint32_t gmemsize_bytes;
-   uint32_t device_id;
+
+   const struct fd_dev_id *dev_id;
+   uint8_t gen;      /* GPU (major) generation */
    uint32_t gpu_id;  /* 220, 305, etc */
-   uint32_t chip_id; /* coreid:8 majorrev:8 minorrev:8 patch:8 */
+   uint64_t chip_id; /* coreid:8 majorrev:8 minorrev:8 patch:8 */
    uint32_t max_freq;
    uint32_t ram_size;
    uint32_t max_rts; /* max # of render targets */
@@ -146,6 +148,23 @@ struct fd_screen {
    const uint64_t *supported_modifiers;
 
    struct renderonly *ro;
+
+   /* the blob seems to always use 8K factor and 128K param sizes, copy them */
+#define FD6_TESS_FACTOR_SIZE (8 * 1024)
+#define FD6_TESS_PARAM_SIZE (128 * 1024)
+#define FD6_TESS_BO_SIZE (FD6_TESS_FACTOR_SIZE + FD6_TESS_PARAM_SIZE)
+   struct fd_bo *tess_bo;
+
+   /* table with PIPE_PRIM_MAX+1 entries mapping PIPE_PRIM_x to
+    * DI_PT_x value to use for draw initiator.  There are some
+    * slight differences between generation.
+    *
+    * Note that primtypes[PRIM_TYPE_MAX] is used to map to the
+    * internal RECTLIST primtype, if available, used for blits/
+    * clears.
+    */
+   const uint8_t *primtypes;
+   uint32_t primtypes_mask;
 };
 
 static inline struct fd_screen *
@@ -179,7 +198,8 @@ struct fd_bo *fd_screen_bo_from_handle(struct pipe_screen *pscreen,
                                        struct winsys_handle *whandle);
 
 struct pipe_screen *fd_screen_create(struct fd_device *dev,
-                                     struct renderonly *ro);
+                                     struct renderonly *ro,
+                                     const struct pipe_screen_config *config);
 
 static inline boolean
 is_a20x(struct fd_screen *screen)
@@ -190,7 +210,7 @@ is_a20x(struct fd_screen *screen)
 static inline boolean
 is_a2xx(struct fd_screen *screen)
 {
-   return (screen->gpu_id >= 200) && (screen->gpu_id < 300);
+   return screen->gen == 2;
 }
 
 /* is a3xx patch revision 0? */
@@ -204,25 +224,25 @@ is_a3xx_p0(struct fd_screen *screen)
 static inline boolean
 is_a3xx(struct fd_screen *screen)
 {
-   return (screen->gpu_id >= 300) && (screen->gpu_id < 400);
+   return screen->gen == 3;
 }
 
 static inline boolean
 is_a4xx(struct fd_screen *screen)
 {
-   return (screen->gpu_id >= 400) && (screen->gpu_id < 500);
+   return screen->gen == 4;
 }
 
 static inline boolean
 is_a5xx(struct fd_screen *screen)
 {
-   return (screen->gpu_id >= 500) && (screen->gpu_id < 600);
+   return screen->gen == 5;
 }
 
 static inline boolean
 is_a6xx(struct fd_screen *screen)
 {
-   return (screen->gpu_id >= 600) && (screen->gpu_id < 700);
+   return screen->gen == 6;
 }
 
 /* is it using the ir3 compiler (shader isa introduced with a3xx)? */
