@@ -546,7 +546,11 @@ i915_create_fs_state(struct pipe_context *pipe,
    ifs->draw_data = draw_create_fragment_shader(i915->draw, templ);
 
    if (templ->type == PIPE_SHADER_IR_NIR) {
-      ifs->state.tokens = nir_to_tgsi(templ->ir.nir, pipe->screen);
+      nir_shader *s = templ->ir.nir;
+
+      NIR_PASS_V(s, i915_nir_lower_sincos);
+
+      ifs->state.tokens = nir_to_tgsi(s, pipe->screen);
    } else {
       assert(templ->type == PIPE_SHADER_IR_TGSI);
       /* we need to keep a local copy of the tokens */
@@ -576,6 +580,10 @@ i915_bind_fs_state(struct pipe_context *pipe, void *shader)
    draw_bind_fragment_shader(i915->draw,
                              (i915->fs ? i915->fs->draw_data : NULL));
 
+   /* Tell draw if we need to do point sprites so we can get PNTC. */
+   if (i915->fs)
+      draw_wide_point_sprites(i915->draw, i915->fs->reads_pntc);
+
    i915->dirty |= I915_NEW_FS;
 }
 
@@ -600,7 +608,7 @@ i915_create_vs_state(struct pipe_context *pipe,
 {
    struct i915_context *i915 = i915_context(pipe);
 
-   struct pipe_shader_state from_nir;
+   struct pipe_shader_state from_nir = { PIPE_SHADER_IR_TGSI };
    if (templ->type == PIPE_SHADER_IR_NIR) {
       nir_shader *s = templ->ir.nir;
 
@@ -611,7 +619,6 @@ i915_create_vs_state(struct pipe_context *pipe,
        * per-stage, and i915 FS can't do native integers.  So, convert to TGSI,
        * where the draw path *does* support non-native-integers.
        */
-      from_nir.type = PIPE_SHADER_IR_TGSI;
       from_nir.tokens = nir_to_tgsi(s, pipe->screen);
       templ = &from_nir;
    }
@@ -710,6 +717,7 @@ static void
 i915_set_sampler_views(struct pipe_context *pipe, enum pipe_shader_type shader,
                        unsigned start, unsigned num,
                        unsigned unbind_num_trailing_slots,
+                       bool take_ownership,
                        struct pipe_sampler_view **views)
 {
    if (shader != PIPE_SHADER_FRAGMENT) {
@@ -728,11 +736,23 @@ i915_set_sampler_views(struct pipe_context *pipe, enum pipe_shader_type shader,
    /* Check for no-op */
    if (views && num == i915->num_fragment_sampler_views &&
        !memcmp(i915->fragment_sampler_views, views,
-               num * sizeof(struct pipe_sampler_view *)))
+               num * sizeof(struct pipe_sampler_view *))) {
+      if (take_ownership) {
+         for (unsigned i = 0; i < num; i++) {
+            struct pipe_sampler_view *view = views[i];
+            pipe_sampler_view_reference(&view, NULL);
+         }
+      }
       return;
+   }
 
    for (i = 0; i < num; i++) {
-      pipe_sampler_view_reference(&i915->fragment_sampler_views[i], views[i]);
+      if (take_ownership) {
+         pipe_sampler_view_reference(&i915->fragment_sampler_views[i], NULL);
+         i915->fragment_sampler_views[i] = views[i];
+      } else {
+         pipe_sampler_view_reference(&i915->fragment_sampler_views[i], views[i]);
+      }
    }
 
    for (i = num; i < i915->num_fragment_sampler_views; i++)
@@ -811,6 +831,8 @@ i915_set_framebuffer_state(struct pipe_context *pipe,
       pipe_surface_reference(&i915->framebuffer.cbufs[0], NULL);
    }
    pipe_surface_reference(&i915->framebuffer.zsbuf, fb->zsbuf);
+   if (fb->zsbuf)
+      draw_set_zs_format(i915->draw, fb->zsbuf->format);
 
    i915->dirty |= I915_NEW_FRAMEBUFFER;
 }
