@@ -23,6 +23,7 @@
  */
 
 #include "util/ralloc.h"
+#include "util/u_cpu_detect.h"
 
 #include <err.h>
 #include <stdio.h>
@@ -166,7 +167,7 @@ main(int argc, char **argv)
    }
 
    char *filename = NULL;
-   filename = argv[n];
+   filename = argv[argc - 1];
 
    uint32_t size = 0;
    uint32_t *prog = extract_shader_binary(filename, &size, &is_frag);
@@ -175,6 +176,9 @@ main(int argc, char **argv)
       return -1;
    }
 
+   /* Needed by _mesa_half_to_float() */
+   util_cpu_detect();
+
    if (is_frag) {
       assert((size & 0x3) == 0);
       size >>= 2;
@@ -183,13 +187,13 @@ main(int argc, char **argv)
       do {
          ppir_codegen_ctrl *ctrl = (ppir_codegen_ctrl *)bin;
          printf("@%6d: ", offset);
-         ppir_disassemble_instr(bin, offset);
+         ppir_disassemble_instr(bin, offset, stdout);
          bin += ctrl->count;
          offset += ctrl->count;
          size -= ctrl->count;
       } while (size);
    } else {
-      gpir_disassemble_program((gpir_codegen_instr *)prog, size / (sizeof(gpir_codegen_instr)));
+      gpir_disassemble_program((gpir_codegen_instr *)prog, size / (sizeof(gpir_codegen_instr)), stdout);
    }
 
    ralloc_free(prog);
