@@ -61,6 +61,7 @@ llvmpipe_create_query(struct pipe_context *pipe,
 
    if (pq) {
       pq->type = type;
+      pq->index = index;
    }
 
    return (struct pipe_query *) pq;
@@ -138,6 +139,17 @@ llvmpipe_get_query_result(struct pipe_context *pipe,
          }
       }
       break;
+   case PIPE_QUERY_TIME_ELAPSED: {
+      uint64_t start = (uint64_t)-1, end = 0;
+      for (i = 0; i < num_threads; i++) {
+         if (pq->start[i] && pq->start[i] < start)
+            start = pq->start[i];
+         if (pq->end[i] && pq->end[i] > end)
+            end = pq->end[i];
+      }
+      *result = end - start;
+      break;
+   }
    case PIPE_QUERY_TIMESTAMP_DISJOINT: {
       struct pipe_query_data_timestamp_disjoint *td =
          (struct pipe_query_data_timestamp_disjoint *)vresult;
@@ -150,20 +162,24 @@ llvmpipe_get_query_result(struct pipe_context *pipe,
       vresult->b = true;
       break;
    case PIPE_QUERY_PRIMITIVES_GENERATED:
-      *result = pq->num_primitives_generated;
+      *result = pq->num_primitives_generated[0];
       break;
    case PIPE_QUERY_PRIMITIVES_EMITTED:
-      *result = pq->num_primitives_written;
+      *result = pq->num_primitives_written[0];
+      break;
+   case PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE:
+      vresult->b = false;
+      for (unsigned s = 0; s < PIPE_MAX_VERTEX_STREAMS; s++)
+         vresult->b |= pq->num_primitives_generated[s] > pq->num_primitives_written[s];
       break;
    case PIPE_QUERY_SO_OVERFLOW_PREDICATE:
-   case PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE:
-      vresult->b = pq->num_primitives_generated > pq->num_primitives_written;
+      vresult->b = pq->num_primitives_generated[0] > pq->num_primitives_written[0];
       break;
    case PIPE_QUERY_SO_STATISTICS: {
       struct pipe_query_data_so_statistics *stats =
          (struct pipe_query_data_so_statistics *)vresult;
-      stats->num_primitives_written = pq->num_primitives_written;
-      stats->primitives_storage_needed = pq->num_primitives_generated;
+      stats->num_primitives_written = pq->num_primitives_written[0];
+      stats->primitives_storage_needed = pq->num_primitives_generated[0];
    }
       break;
    case PIPE_QUERY_PIPELINE_STATISTICS: {
@@ -198,19 +214,22 @@ llvmpipe_get_query_result_resource(struct pipe_context *pipe,
    unsigned num_threads = MAX2(1, screen->num_threads);
    struct llvmpipe_query *pq = llvmpipe_query(q);
    struct llvmpipe_resource *lpr = llvmpipe_resource(resource);
-   bool unflushed = false;
    bool unsignalled = false;
    if (pq->fence) {
       /* only have a fence if there was a scene */
       if (!lp_fence_signalled(pq->fence)) {
-         unsignalled = true;
          if (!lp_fence_issued(pq->fence))
-            unflushed = true;
+            llvmpipe_flush(pipe, NULL, __FUNCTION__);
+
+         if (wait)
+            lp_fence_wait(pq->fence);
       }
+      unsignalled = !lp_fence_signalled(pq->fence);
    }
 
 
-   uint64_t value = 0;
+   uint64_t value = 0, value2 = 0;
+   unsigned num_values = 1;
    if (index == -1)
       if (unsignalled)
          value = 0;
@@ -219,15 +238,6 @@ llvmpipe_get_query_result_resource(struct pipe_context *pipe,
    else {
       unsigned i;
 
-      if (unflushed) {
-         llvmpipe_flush(pipe, NULL, __FUNCTION__);
-
-         if (!wait)
-            return;
-
-         lp_fence_wait(pq->fence);
-      }
-
       switch (pq->type) {
       case PIPE_QUERY_OCCLUSION_COUNTER:
          for (i = 0; i < num_threads; i++) {
@@ -242,10 +252,10 @@ llvmpipe_get_query_result_resource(struct pipe_context *pipe,
          }
          break;
       case PIPE_QUERY_PRIMITIVES_GENERATED:
-         value = pq->num_primitives_generated;
+         value = pq->num_primitives_generated[0];
          break;
       case PIPE_QUERY_PRIMITIVES_EMITTED:
-         value = pq->num_primitives_written;
+         value = pq->num_primitives_written[0];
          break;
       case PIPE_QUERY_TIMESTAMP:
          for (i = 0; i < num_threads; i++) {
@@ -254,9 +264,29 @@ llvmpipe_get_query_result_resource(struct pipe_context *pipe,
             }
          }
          break;
-      case PIPE_QUERY_SO_OVERFLOW_PREDICATE:
+      case PIPE_QUERY_TIME_ELAPSED: {
+         uint64_t start = (uint64_t)-1, end = 0;
+         for (i = 0; i < num_threads; i++) {
+            if (pq->start[i] && pq->start[i] < start)
+               start = pq->start[i];
+            if (pq->end[i] && pq->end[i] > end)
+               end = pq->end[i];
+         }
+         value = end - start;
+         break;
+      }
+      case PIPE_QUERY_SO_STATISTICS:
+         value = pq->num_primitives_written[0];
+         value2 = pq->num_primitives_generated[0];
+         num_values = 2;
+         break;
       case PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE:
-         value = !!(pq->num_primitives_generated > pq->num_primitives_written);
+         value = 0;
+         for (unsigned s = 0; s < PIPE_MAX_VERTEX_STREAMS; s++)
+            value |= !!(pq->num_primitives_generated[s] > pq->num_primitives_written[s]);
+         break;
+      case PIPE_QUERY_SO_OVERFLOW_PREDICATE:
+         value = !!(pq->num_primitives_generated[0] > pq->num_primitives_written[0]);
          break;
       case PIPE_QUERY_PIPELINE_STATISTICS:
          switch ((enum pipe_statistics_query_index)index) {
@@ -306,33 +336,42 @@ llvmpipe_get_query_result_resource(struct pipe_context *pipe,
    }
 
    void *dst = (uint8_t *)lpr->data + offset;
-   switch (result_type) {
-   case PIPE_QUERY_TYPE_I32: {
-      int32_t *iptr = (int32_t *)dst;
-      if (value > 0x7fffffff)
-         *iptr = 0x7fffffff;
-      else
-         *iptr = (int32_t)value;
-      break;
-   }
-   case PIPE_QUERY_TYPE_U32: {
-      uint32_t *uptr = (uint32_t *)dst;
-      if (value > 0xffffffff)
-         *uptr = 0xffffffff;
-      else
-         *uptr = (uint32_t)value;
-      break;
-   }
-   case PIPE_QUERY_TYPE_I64: {
-      int64_t *iptr = (int64_t *)dst;
-      *iptr = (int64_t)value;
-      break;
-   }
-   case PIPE_QUERY_TYPE_U64: {
-      uint64_t *uptr = (uint64_t *)dst;
-      *uptr = (uint64_t)value;
-      break;
-   }
+
+   for (unsigned i = 0; i < num_values; i++) {
+
+      if (i == 1) {
+         value = value2;
+         dst = (char *)dst + ((result_type == PIPE_QUERY_TYPE_I64 ||
+                               result_type == PIPE_QUERY_TYPE_U64) ? 8 : 4);
+      }
+      switch (result_type) {
+      case PIPE_QUERY_TYPE_I32: {
+         int32_t *iptr = (int32_t *)dst;
+         if (value > 0x7fffffff)
+            *iptr = 0x7fffffff;
+         else
+            *iptr = (int32_t)value;
+         break;
+      }
+      case PIPE_QUERY_TYPE_U32: {
+         uint32_t *uptr = (uint32_t *)dst;
+         if (value > 0xffffffff)
+            *uptr = 0xffffffff;
+         else
+            *uptr = (uint32_t)value;
+         break;
+      }
+      case PIPE_QUERY_TYPE_I64: {
+         int64_t *iptr = (int64_t *)dst;
+         *iptr = (int64_t)value;
+         break;
+      }
+      case PIPE_QUERY_TYPE_U64: {
+         uint64_t *uptr = (uint64_t *)dst;
+         *uptr = (uint64_t)value;
+         break;
+      }
+      }
    }
 }
 
@@ -357,20 +396,25 @@ llvmpipe_begin_query(struct pipe_context *pipe, struct pipe_query *q)
 
    switch (pq->type) {
    case PIPE_QUERY_PRIMITIVES_EMITTED:
-      pq->num_primitives_written = llvmpipe->so_stats.num_primitives_written;
+      pq->num_primitives_written[0] = llvmpipe->so_stats[pq->index].num_primitives_written;
       break;
    case PIPE_QUERY_PRIMITIVES_GENERATED:
-      pq->num_primitives_generated = llvmpipe->so_stats.primitives_storage_needed;
+      pq->num_primitives_generated[0] = llvmpipe->so_stats[pq->index].primitives_storage_needed;
       llvmpipe->active_primgen_queries++;
       break;
    case PIPE_QUERY_SO_STATISTICS:
-      pq->num_primitives_written = llvmpipe->so_stats.num_primitives_written;
-      pq->num_primitives_generated = llvmpipe->so_stats.primitives_storage_needed;
+      pq->num_primitives_written[0] = llvmpipe->so_stats[pq->index].num_primitives_written;
+      pq->num_primitives_generated[0] = llvmpipe->so_stats[pq->index].primitives_storage_needed;
+      break;
+   case PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE:
+      for (unsigned s = 0; s < PIPE_MAX_VERTEX_STREAMS; s++) {
+         pq->num_primitives_written[s] = llvmpipe->so_stats[s].num_primitives_written;
+         pq->num_primitives_generated[s] = llvmpipe->so_stats[s].primitives_storage_needed;
+      }
       break;
    case PIPE_QUERY_SO_OVERFLOW_PREDICATE:
-   case PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE:
-      pq->num_primitives_written = llvmpipe->so_stats.num_primitives_written;
-      pq->num_primitives_generated = llvmpipe->so_stats.primitives_storage_needed;
+      pq->num_primitives_written[0] = llvmpipe->so_stats[pq->index].num_primitives_written;
+      pq->num_primitives_generated[0] = llvmpipe->so_stats[pq->index].primitives_storage_needed;
       break;
    case PIPE_QUERY_PIPELINE_STATISTICS:
       /* reset our cache */
@@ -405,27 +449,34 @@ llvmpipe_end_query(struct pipe_context *pipe, struct pipe_query *q)
    switch (pq->type) {
 
    case PIPE_QUERY_PRIMITIVES_EMITTED:
-      pq->num_primitives_written =
-         llvmpipe->so_stats.num_primitives_written - pq->num_primitives_written;
+      pq->num_primitives_written[0] =
+         llvmpipe->so_stats[pq->index].num_primitives_written - pq->num_primitives_written[0];
       break;
    case PIPE_QUERY_PRIMITIVES_GENERATED:
       assert(llvmpipe->active_primgen_queries);
       llvmpipe->active_primgen_queries--;
-      pq->num_primitives_generated =
-         llvmpipe->so_stats.primitives_storage_needed - pq->num_primitives_generated;
+      pq->num_primitives_generated[0] =
+         llvmpipe->so_stats[pq->index].primitives_storage_needed - pq->num_primitives_generated[0];
       break;
    case PIPE_QUERY_SO_STATISTICS:
-      pq->num_primitives_written =
-         llvmpipe->so_stats.num_primitives_written - pq->num_primitives_written;
-      pq->num_primitives_generated =
-         llvmpipe->so_stats.primitives_storage_needed - pq->num_primitives_generated;
+      pq->num_primitives_written[0] =
+         llvmpipe->so_stats[pq->index].num_primitives_written - pq->num_primitives_written[0];
+      pq->num_primitives_generated[0] =
+         llvmpipe->so_stats[pq->index].primitives_storage_needed - pq->num_primitives_generated[0];
+      break;
+   case PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE:
+      for (unsigned s = 0; s < PIPE_MAX_VERTEX_STREAMS; s++) {
+         pq->num_primitives_written[s] =
+            llvmpipe->so_stats[s].num_primitives_written - pq->num_primitives_written[s];
+         pq->num_primitives_generated[s] =
+            llvmpipe->so_stats[s].primitives_storage_needed - pq->num_primitives_generated[s];
+      }
       break;
    case PIPE_QUERY_SO_OVERFLOW_PREDICATE:
-   case PIPE_QUERY_SO_OVERFLOW_ANY_PREDICATE:
-      pq->num_primitives_written =
-         llvmpipe->so_stats.num_primitives_written - pq->num_primitives_written;
-      pq->num_primitives_generated =
-         llvmpipe->so_stats.primitives_storage_needed - pq->num_primitives_generated;
+      pq->num_primitives_written[0] =
+         llvmpipe->so_stats[pq->index].num_primitives_written - pq->num_primitives_written[0];
+      pq->num_primitives_generated[0] =
+         llvmpipe->so_stats[pq->index].primitives_storage_needed - pq->num_primitives_generated[0];
       break;
    case PIPE_QUERY_PIPELINE_STATISTICS:
       pq->stats.ia_vertices =
@@ -446,6 +497,10 @@ llvmpipe_end_query(struct pipe_context *pipe, struct pipe_query *q)
          llvmpipe->pipeline_statistics.ps_invocations - pq->stats.ps_invocations;
       pq->stats.cs_invocations =
          llvmpipe->pipeline_statistics.cs_invocations - pq->stats.cs_invocations;
+      pq->stats.hs_invocations =
+         llvmpipe->pipeline_statistics.hs_invocations - pq->stats.hs_invocations;
+      pq->stats.ds_invocations =
+         llvmpipe->pipeline_statistics.ds_invocations - pq->stats.ds_invocations;
       llvmpipe->active_statistics_queries--;
       break;
    case PIPE_QUERY_OCCLUSION_COUNTER:
@@ -469,6 +524,10 @@ llvmpipe_check_render_cond(struct llvmpipe_context *lp)
    boolean b, wait;
    uint64_t result;
 
+   if (lp->render_cond_buffer) {
+      uint32_t data = *(uint32_t *)((char *)lp->render_cond_buffer->data + lp->render_cond_offset);
+      return (!data) == lp->render_cond_cond;
+   }
    if (!lp->render_cond_query)
       return TRUE; /* no query predicate, draw normally */
 
