@@ -34,7 +34,8 @@
 static void
 panfrost_blitter_save(
         struct panfrost_context *ctx,
-        struct blitter_context *blitter)
+        struct blitter_context *blitter,
+        bool render_cond)
 {
 
         util_blitter_save_vertex_buffer_slot(blitter, ctx->vertex_buffers);
@@ -48,7 +49,7 @@ panfrost_blitter_save(
         util_blitter_save_depth_stencil_alpha(blitter, ctx->depth_stencil);
         util_blitter_save_stencil_ref(blitter, &ctx->stencil_ref);
         util_blitter_save_so_targets(blitter, 0, NULL);
-        util_blitter_save_sample_mask(blitter, ctx->sample_mask);
+        util_blitter_save_sample_mask(blitter, ctx->sample_mask, ctx->min_samples);
 
         util_blitter_save_framebuffer(blitter, &ctx->pipe_framebuffer);
         util_blitter_save_fragment_sampler_states(blitter,
@@ -59,35 +60,28 @@ panfrost_blitter_save(
                         (struct pipe_sampler_view **)&ctx->sampler_views[PIPE_SHADER_FRAGMENT]);
         util_blitter_save_fragment_constant_buffer_slot(blitter,
                         ctx->constant_buffer[PIPE_SHADER_FRAGMENT].cb);
-}
 
-static bool
-panfrost_u_blitter_blit(struct pipe_context *pipe,
-                        const struct pipe_blit_info *info)
-{
-        struct panfrost_context *ctx = pan_context(pipe);
+        if (!render_cond) {
+                util_blitter_save_render_condition(blitter,
+                                (struct pipe_query *) ctx->cond_query,
+                                ctx->cond_cond, ctx->cond_mode);
+        }
 
-        if (!util_blitter_is_blit_supported(ctx->blitter, info))
-                unreachable("Unsupported blit\n");
-
-        /* TODO: Scissor */
-
-        panfrost_blitter_save(ctx, ctx->blitter);
-        util_blitter_blit(ctx->blitter, info);
-
-        return true;
 }
 
 void
 panfrost_blit(struct pipe_context *pipe,
               const struct pipe_blit_info *info)
 {
-        /* We don't have a hardware blit, so we just fake it with
-         * u_blitter. We could do a little better by culling
-         * vertex jobs, though. */
+        struct panfrost_context *ctx = pan_context(pipe);
 
-        if (panfrost_u_blitter_blit(pipe, info))
+        if (info->render_condition_enable &&
+            !panfrost_render_condition_check(ctx))
                 return;
 
-        return;
+        if (!util_blitter_is_blit_supported(ctx->blitter, info))
+                unreachable("Unsupported blit\n");
+
+        panfrost_blitter_save(ctx, ctx->blitter, info->render_condition_enable);
+        util_blitter_blit(ctx->blitter, info);
 }
