@@ -45,6 +45,8 @@
 
 #include "radeon_program_pair.h"
 
+#include "util/compiler.h"
+
 #define PROG_CODE \
 	struct r500_fragment_program_code *code = &c->code->code.r500
 
@@ -101,9 +103,9 @@ static unsigned int translate_rgb_op(struct r300_fragment_program_compiler *c, r
 	case RC_OPCODE_FRC: return R500_ALU_RGBA_OP_FRC;
 	default:
 		error("translate_rgb_op: unknown opcode %s\n", rc_get_opcode_info(opcode)->Name);
-		/* fall through */
+		FALLTHROUGH;
 	case RC_OPCODE_NOP:
-		/* fall through */
+		FALLTHROUGH;
 	case RC_OPCODE_MAD: return R500_ALU_RGBA_OP_MAD;
 	case RC_OPCODE_MAX: return R500_ALU_RGBA_OP_MAX;
 	case RC_OPCODE_MIN: return R500_ALU_RGBA_OP_MIN;
@@ -126,9 +128,9 @@ static unsigned int translate_alpha_op(struct r300_fragment_program_compiler *c,
 	case RC_OPCODE_LG2: return R500_ALPHA_OP_LN2;
 	default:
 		error("translate_alpha_op: unknown opcode %s\n", rc_get_opcode_info(opcode)->Name);
-		/* fall through */
+		FALLTHROUGH;
 	case RC_OPCODE_NOP:
-		/* fall through */
+		FALLTHROUGH;
 	case RC_OPCODE_MAD: return R500_ALPHA_OP_MAD;
 	case RC_OPCODE_MAX: return R500_ALPHA_OP_MAX;
 	case RC_OPCODE_MIN: return R500_ALPHA_OP_MIN;
