@@ -26,6 +26,8 @@
  *
  */
 
+#include "util/u_math.h"
+
 #include "radeon_dataflow.h"
 
 #include "radeon_compiler.h"
@@ -496,14 +498,18 @@ static int presub_helper(
 	return 1;
 }
 
-/* This function assumes that inst_add->U.I.SrcReg[0] and
- * inst_add->U.I.SrcReg[1] aren't both negative. */
 static void presub_replace_add(
 	struct rc_instruction * inst_add,
 	struct rc_instruction * inst_reader,
 	unsigned int src_index)
 {
 	rc_presubtract_op presub_opcode;
+
+	/* This function assumes that inst_add->U.I.SrcReg[0] and
+	 * inst_add->U.I.SrcReg[1] aren't both negative.
+	 */
+	assert(!(inst_add->U.I.SrcReg[1].Negate && inst_add->U.I.SrcReg[0].Negate));
+
 	if (inst_add->U.I.SrcReg[1].Negate || inst_add->U.I.SrcReg[0].Negate)
 		presub_opcode = RC_PRESUB_SUB;
 	else
@@ -653,11 +659,12 @@ static int peephole_add_presub_inv(
 	/* XXX It would be nice to use is_src_uniform_constant here, but that
 	 * function only works if the register's file is RC_FILE_NONE */
 	for(i = 0; i < 4; i++ ) {
+		if (!(inst_add->U.I.DstReg.WriteMask & (1 << i)))
+			continue;
+
 		swz = GET_SWZ(inst_add->U.I.SrcReg[0].Swizzle, i);
-		if(((1 << i) & inst_add->U.I.DstReg.WriteMask)
-						&& swz != RC_SWIZZLE_ONE) {
+		if (swz != RC_SWIZZLE_ONE || inst_add->U.I.SrcReg[0].Negate & (1 << i))
 			return 0;
-		}
 	}
 
 	/* Check src1. */
@@ -832,8 +839,15 @@ static int peephole_mul_omod(
 		return 0;
 	}
 
-	/* Rewrite the instructions */
 	writemask_sum = rc_variable_writemask_sum(writer_list->Item);
+
+	/* rc_normal_rewrite_writemask can't expand a previous writemask to store
+	 * more channels replicated.
+	 */
+	if (util_bitcount(writemask_sum) < util_bitcount(inst_mul->U.I.DstReg.WriteMask))
+		return 0;
+
+	/* Rewrite the instructions */
 	for (var = writer_list->Item; var; var = var->Friend) {
 		struct rc_variable * writer = var;
 		unsigned conversion_swizzle = rc_make_conversion_swizzle(
@@ -876,7 +890,6 @@ static int peephole(struct radeon_compiler * c, struct rc_instruction * inst)
 void rc_optimize(struct radeon_compiler * c, void *user)
 {
 	struct rc_instruction * inst = c->Program.Instructions.Next;
-	struct rc_list * var_list;
 	while(inst != &c->Program.Instructions) {
 		struct rc_instruction * cur = inst;
 		inst = inst->Next;
@@ -897,12 +910,15 @@ void rc_optimize(struct radeon_compiler * c, void *user)
 	}
 
 	inst = c->Program.Instructions.Next;
+	struct rc_list * var_list = NULL;
 	while(inst != &c->Program.Instructions) {
 		struct rc_instruction * cur = inst;
 		inst = inst->Next;
 		if (cur->U.I.Opcode == RC_OPCODE_MUL) {
-			var_list = rc_get_variables(c);
-			peephole_mul_omod(c, cur, var_list);
+			if (!var_list)
+				var_list = rc_get_variables(c);
+			if (peephole_mul_omod(c, cur, var_list))
+				var_list = NULL;
 		}
 	}
 }
