@@ -226,16 +226,6 @@ static struct rc_dst_register try_to_reuse_dst(struct radeon_compiler *c,
 	return dstregtmpmask(tmp, inst->U.I.DstReg.WriteMask);
 }
 
-static void transform_ABS(struct radeon_compiler* c,
-	struct rc_instruction* inst)
-{
-	struct rc_src_register src = inst->U.I.SrcReg[0];
-	src.Abs = 1;
-	src.Negate = RC_MASK_NONE;
-	emit1(c, inst->Prev, RC_OPCODE_MOV, &inst->U.I, inst->U.I.DstReg, src);
-	rc_remove_instruction(inst);
-}
-
 static void transform_CEIL(struct radeon_compiler* c,
 	struct rc_instruction* inst)
 {
@@ -250,28 +240,12 @@ static void transform_CEIL(struct radeon_compiler* c,
 	 */
 
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
-	emit1(c, inst->Prev, RC_OPCODE_FRC, 0, dst, negate(inst->U.I.SrcReg[0]));
+	emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, dst, negate(inst->U.I.SrcReg[0]));
 	emit2(c, inst->Prev, RC_OPCODE_ADD, &inst->U.I, inst->U.I.DstReg,
 		inst->U.I.SrcReg[0], srcreg(RC_FILE_TEMPORARY, dst.Index));
 	rc_remove_instruction(inst);
 }
 
-static void transform_CLAMP(struct radeon_compiler *c,
-	struct rc_instruction *inst)
-{
-	/* CLAMP dst, src, min, max
-	 *    into:
-	 * MIN tmp, src, max
-	 * MAX dst, tmp, min
-	 */
-	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
-	emit2(c, inst->Prev, RC_OPCODE_MIN, 0, dst,
-		inst->U.I.SrcReg[0], inst->U.I.SrcReg[2]);
-	emit2(c, inst->Prev, RC_OPCODE_MAX, &inst->U.I, inst->U.I.DstReg,
-		srcreg(RC_FILE_TEMPORARY, dst.Index), inst->U.I.SrcReg[1]);
-	rc_remove_instruction(inst);
-}
-
 static void transform_DP2(struct radeon_compiler* c,
 	struct rc_instruction* inst)
 {
@@ -287,17 +261,6 @@ static void transform_DP2(struct radeon_compiler* c,
 	rc_remove_instruction(inst);
 }
 
-static void transform_DPH(struct radeon_compiler* c,
-	struct rc_instruction* inst)
-{
-	struct rc_src_register src0 = inst->U.I.SrcReg[0];
-	src0.Negate &= ~RC_MASK_W;
-	src0.Swizzle &= ~(7 << (3 * 3));
-	src0.Swizzle |= RC_SWIZZLE_ONE << (3 * 3);
-	emit2(c, inst->Prev, RC_OPCODE_DP4, &inst->U.I, inst->U.I.DstReg, src0, inst->U.I.SrcReg[1]);
-	rc_remove_instruction(inst);
-}
-
 /**
  * [1, src0.y*src1.y, src0.z, src1.w]
  * So basically MUL with lotsa swizzling.
@@ -315,7 +278,7 @@ static void transform_FLR(struct radeon_compiler* c,
 	struct rc_instruction* inst)
 {
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
-	emit1(c, inst->Prev, RC_OPCODE_FRC, 0, dst, inst->U.I.SrcReg[0]);
+	emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, dst, inst->U.I.SrcReg[0]);
 	emit2(c, inst->Prev, RC_OPCODE_ADD, &inst->U.I, inst->U.I.DstReg,
 		inst->U.I.SrcReg[0], negate(srcreg(RC_FILE_TEMPORARY, dst.Index)));
 	rc_remove_instruction(inst);
@@ -331,8 +294,8 @@ static void transform_TRUNC(struct radeon_compiler* c,
 	 *   y * sgn(x) = (x < 0 ? -y : y)
 	 */
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
-	emit1(c, inst->Prev, RC_OPCODE_FRC, 0, dst, absolute(inst->U.I.SrcReg[0]));
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, dst, absolute(inst->U.I.SrcReg[0]),
+	emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, dst, absolute(inst->U.I.SrcReg[0]));
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, dst, absolute(inst->U.I.SrcReg[0]),
 	      negate(srcreg(RC_FILE_TEMPORARY, dst.Index)));
 	emit3(c, inst->Prev, RC_OPCODE_CMP, &inst->U.I, inst->U.I.DstReg, inst->U.I.SrcReg[0],
 	      negate(srcreg(RC_FILE_TEMPORARY, dst.Index)), srcreg(RC_FILE_TEMPORARY, dst.Index));
@@ -371,7 +334,7 @@ static void transform_LIT(struct radeon_compiler* c,
 		struct rc_instruction * inst_mov;
 
 		inst_mov = emit1(c, inst,
-			RC_OPCODE_MOV, 0, inst->U.I.DstReg,
+			RC_OPCODE_MOV, NULL, inst->U.I.DstReg,
 			srcreg(RC_FILE_TEMPORARY, rc_find_free_temporary(c)));
 
 		inst->U.I.DstReg.File = RC_FILE_TEMPORARY;
@@ -385,25 +348,25 @@ static void transform_LIT(struct radeon_compiler* c,
 	/* tmp.x = max(0.0, Src.x); */
 	/* tmp.y = max(0.0, Src.y); */
 	/* tmp.w = clamp(Src.z, -128+eps, 128-eps); */
-	emit2(c, inst->Prev, RC_OPCODE_MAX, 0,
+	emit2(c, inst->Prev, RC_OPCODE_MAX, NULL,
 		dstregtmpmask(temp, RC_MASK_XYW),
 		inst->U.I.SrcReg[0],
 		swizzle(srcreg(RC_FILE_CONSTANT, constant),
 			RC_SWIZZLE_ZERO, RC_SWIZZLE_ZERO, RC_SWIZZLE_ZERO, constant_swizzle&3));
-	emit2(c, inst->Prev, RC_OPCODE_MIN, 0,
+	emit2(c, inst->Prev, RC_OPCODE_MIN, NULL,
 		dstregtmpmask(temp, RC_MASK_Z),
 		swizzle_wwww(srctemp),
 		negate(srcregswz(RC_FILE_CONSTANT, constant, constant_swizzle)));
 
 	/* tmp.w = Pow(tmp.y, tmp.w) */
-	emit1(c, inst->Prev, RC_OPCODE_LG2, 0,
+	emit1(c, inst->Prev, RC_OPCODE_LG2, NULL,
 		dstregtmpmask(temp, RC_MASK_W),
 		swizzle_yyyy(srctemp));
-	emit2(c, inst->Prev, RC_OPCODE_MUL, 0,
+	emit2(c, inst->Prev, RC_OPCODE_MUL, NULL,
 		dstregtmpmask(temp, RC_MASK_W),
 		swizzle_wwww(srctemp),
 		swizzle_zzzz(srctemp));
-	emit1(c, inst->Prev, RC_OPCODE_EX2, 0,
+	emit1(c, inst->Prev, RC_OPCODE_EX2, NULL,
 		dstregtmpmask(temp, RC_MASK_W),
 		swizzle_wwww(srctemp));
 
@@ -427,12 +390,12 @@ static void transform_LRP(struct radeon_compiler* c,
 {
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
 
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0,
+	emit3(c, inst->Prev, RC_OPCODE_MAD, NULL,
 		dst,
-		inst->U.I.SrcReg[1], negate(inst->U.I.SrcReg[2]));
+		negate(inst->U.I.SrcReg[0]), inst->U.I.SrcReg[2], inst->U.I.SrcReg[2]);
 	emit3(c, inst->Prev, RC_OPCODE_MAD, &inst->U.I,
 		inst->U.I.DstReg,
-		inst->U.I.SrcReg[0], srcreg(RC_FILE_TEMPORARY, dst.Index), inst->U.I.SrcReg[2]);
+		inst->U.I.SrcReg[0], inst->U.I.SrcReg[1], srcreg(RC_FILE_TEMPORARY, dst.Index));
 
 	rc_remove_instruction(inst);
 }
@@ -445,8 +408,8 @@ static void transform_POW(struct radeon_compiler* c,
 	tempdst.WriteMask = RC_MASK_W;
 	tempsrc.Swizzle = RC_SWIZZLE_WWWW;
 
-	emit1(c, inst->Prev, RC_OPCODE_LG2, 0, tempdst, swizzle_xxxx(inst->U.I.SrcReg[0]));
-	emit2(c, inst->Prev, RC_OPCODE_MUL, 0, tempdst, tempsrc, swizzle_xxxx(inst->U.I.SrcReg[1]));
+	emit1(c, inst->Prev, RC_OPCODE_LG2, NULL, tempdst, swizzle_xxxx(inst->U.I.SrcReg[0]));
+	emit2(c, inst->Prev, RC_OPCODE_MUL, NULL, tempdst, tempsrc, swizzle_xxxx(inst->U.I.SrcReg[1]));
 	emit1(c, inst->Prev, RC_OPCODE_EX2, &inst->U.I, inst->U.I.DstReg, tempsrc);
 
 	rc_remove_instruction(inst);
@@ -472,7 +435,7 @@ static void transform_ROUND(struct radeon_compiler* c,
 	/* add = src + .5 */
 	add_index = rc_find_free_temporary(c);
 	add_dst = dstregtmpmask(add_index, mask);
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, add_dst, inst->U.I.SrcReg[0],
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, add_dst, inst->U.I.SrcReg[0],
 								builtin_half);
 	add_src = srcreg(RC_FILE_TEMPORARY, add_dst.Index);
 
@@ -480,11 +443,11 @@ static void transform_ROUND(struct radeon_compiler* c,
 	/* frac = FRC(add) */
 	frac_index = rc_find_free_temporary(c);
 	frac_dst = dstregtmpmask(frac_index, mask);
-	emit1(c, inst->Prev, RC_OPCODE_FRC, 0, frac_dst, add_src);
+	emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, frac_dst, add_src);
 	frac_src = srcreg(RC_FILE_TEMPORARY, frac_dst.Index);
 
 	/* dst = add - frac */
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, inst->U.I.DstReg,
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, inst->U.I.DstReg,
 						add_src, negate(frac_src));
 	rc_remove_instruction(inst);
 }
@@ -500,26 +463,19 @@ static void transform_SEQ(struct radeon_compiler* c,
 {
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
 
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, dst, inst->U.I.SrcReg[0], negate(inst->U.I.SrcReg[1]));
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, dst, inst->U.I.SrcReg[0], negate(inst->U.I.SrcReg[1]));
 	emit3(c, inst->Prev, RC_OPCODE_CMP, &inst->U.I, inst->U.I.DstReg,
 		negate(absolute(srcreg(RC_FILE_TEMPORARY, dst.Index))), builtin_zero, builtin_one);
 
 	rc_remove_instruction(inst);
 }
 
-static void transform_SFL(struct radeon_compiler* c,
-	struct rc_instruction* inst)
-{
-	emit1(c, inst->Prev, RC_OPCODE_MOV, &inst->U.I, inst->U.I.DstReg, builtin_zero);
-	rc_remove_instruction(inst);
-}
-
 static void transform_SGE(struct radeon_compiler* c,
 	struct rc_instruction* inst)
 {
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
 
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, dst, inst->U.I.SrcReg[0], negate(inst->U.I.SrcReg[1]));
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, dst, inst->U.I.SrcReg[0], negate(inst->U.I.SrcReg[1]));
 	emit3(c, inst->Prev, RC_OPCODE_CMP, &inst->U.I, inst->U.I.DstReg,
 		srcreg(RC_FILE_TEMPORARY, dst.Index), builtin_zero, builtin_one);
 
@@ -531,7 +487,7 @@ static void transform_SGT(struct radeon_compiler* c,
 {
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
 
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, dst, negate(inst->U.I.SrcReg[0]), inst->U.I.SrcReg[1]);
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, dst, negate(inst->U.I.SrcReg[0]), inst->U.I.SrcReg[1]);
 	emit3(c, inst->Prev, RC_OPCODE_CMP, &inst->U.I, inst->U.I.DstReg,
 		srcreg(RC_FILE_TEMPORARY, dst.Index), builtin_one, builtin_zero);
 
@@ -543,7 +499,7 @@ static void transform_SLE(struct radeon_compiler* c,
 {
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
 
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, dst, negate(inst->U.I.SrcReg[0]), inst->U.I.SrcReg[1]);
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, dst, negate(inst->U.I.SrcReg[0]), inst->U.I.SrcReg[1]);
 	emit3(c, inst->Prev, RC_OPCODE_CMP, &inst->U.I, inst->U.I.DstReg,
 		srcreg(RC_FILE_TEMPORARY, dst.Index), builtin_zero, builtin_one);
 
@@ -555,7 +511,7 @@ static void transform_SLT(struct radeon_compiler* c,
 {
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
 
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, dst, inst->U.I.SrcReg[0], negate(inst->U.I.SrcReg[1]));
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, dst, inst->U.I.SrcReg[0], negate(inst->U.I.SrcReg[1]));
 	emit3(c, inst->Prev, RC_OPCODE_CMP, &inst->U.I, inst->U.I.DstReg,
 		srcreg(RC_FILE_TEMPORARY, dst.Index), builtin_one, builtin_zero);
 
@@ -567,7 +523,7 @@ static void transform_SNE(struct radeon_compiler* c,
 {
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
 
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0, dst, inst->U.I.SrcReg[0], negate(inst->U.I.SrcReg[1]));
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL, dst, inst->U.I.SrcReg[0], negate(inst->U.I.SrcReg[1]));
 	emit3(c, inst->Prev, RC_OPCODE_CMP, &inst->U.I, inst->U.I.DstReg,
 		negate(absolute(srcreg(RC_FILE_TEMPORARY, dst.Index))), builtin_one, builtin_zero);
 
@@ -588,7 +544,7 @@ static void transform_SSG(struct radeon_compiler* c,
 
 	/* 0 < x */
 	dst0 = try_to_reuse_dst(c, inst);
-	emit3(c, inst->Prev, RC_OPCODE_CMP, 0,
+	emit3(c, inst->Prev, RC_OPCODE_CMP, NULL,
 	      dst0,
 	      negate(inst->U.I.SrcReg[0]),
 	      builtin_one,
@@ -596,7 +552,7 @@ static void transform_SSG(struct radeon_compiler* c,
 
 	/* x < 0 */
 	tmp1 = rc_find_free_temporary(c);
-	emit3(c, inst->Prev, RC_OPCODE_CMP, 0,
+	emit3(c, inst->Prev, RC_OPCODE_CMP, NULL,
 	      dstregtmpmask(tmp1, inst->U.I.DstReg.WriteMask),
 	      inst->U.I.SrcReg[0],
 	      builtin_one,
@@ -604,7 +560,7 @@ static void transform_SSG(struct radeon_compiler* c,
 
 	/* Either both are zero, or one of them is one and the other is zero. */
 	/* result = tmp0 - tmp1 */
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0,
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL,
 	      inst->U.I.DstReg,
 	      srcreg(RC_FILE_TEMPORARY, dst0.Index),
 	      negate(srcreg(RC_FILE_TEMPORARY, tmp1)));
@@ -619,35 +575,12 @@ static void transform_SUB(struct radeon_compiler* c,
 	inst->U.I.SrcReg[1] = negate(inst->U.I.SrcReg[1]);
 }
 
-static void transform_SWZ(struct radeon_compiler* c,
-	struct rc_instruction* inst)
-{
-	inst->U.I.Opcode = RC_OPCODE_MOV;
-}
-
-static void transform_XPD(struct radeon_compiler* c,
-	struct rc_instruction* inst)
-{
-	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
-
-	emit2(c, inst->Prev, RC_OPCODE_MUL, 0, dst,
-		swizzle(inst->U.I.SrcReg[0], RC_SWIZZLE_Z, RC_SWIZZLE_X, RC_SWIZZLE_Y, RC_SWIZZLE_W),
-		swizzle(inst->U.I.SrcReg[1], RC_SWIZZLE_Y, RC_SWIZZLE_Z, RC_SWIZZLE_X, RC_SWIZZLE_W));
-	emit3(c, inst->Prev, RC_OPCODE_MAD, &inst->U.I, inst->U.I.DstReg,
-		swizzle(inst->U.I.SrcReg[0], RC_SWIZZLE_Y, RC_SWIZZLE_Z, RC_SWIZZLE_X, RC_SWIZZLE_W),
-		swizzle(inst->U.I.SrcReg[1], RC_SWIZZLE_Z, RC_SWIZZLE_X, RC_SWIZZLE_Y, RC_SWIZZLE_W),
-		negate(srcreg(RC_FILE_TEMPORARY, dst.Index)));
-
-	rc_remove_instruction(inst);
-}
-
-
 /**
  * Can be used as a transformation for @ref radeonClauseLocalTransform,
  * no userData necessary.
  *
  * Eliminates the following ALU instructions:
- *  ABS, CEIL, DPH, DST, FLR, LIT, LRP, POW, SEQ, SFL, SGE, SGT, SLE, SLT, SNE, SUB, SWZ, XPD
+ *  CEIL, DST, FLR, LIT, LRP, POW, SEQ, SGE, SGT, SLE, SLT, SNE, SUB
  * using:
  *  MOV, ADD, MUL, MAD, FRC, DP3, LG2, EX2, CMP
  *
@@ -662,11 +595,8 @@ int radeonTransformALU(
 	void* unused)
 {
 	switch(inst->U.I.Opcode) {
-	case RC_OPCODE_ABS: transform_ABS(c, inst); return 1;
 	case RC_OPCODE_CEIL: transform_CEIL(c, inst); return 1;
-	case RC_OPCODE_CLAMP: transform_CLAMP(c, inst); return 1;
 	case RC_OPCODE_DP2: transform_DP2(c, inst); return 1;
-	case RC_OPCODE_DPH: transform_DPH(c, inst); return 1;
 	case RC_OPCODE_DST: transform_DST(c, inst); return 1;
 	case RC_OPCODE_FLR: transform_FLR(c, inst); return 1;
 	case RC_OPCODE_LIT: transform_LIT(c, inst); return 1;
@@ -675,7 +605,6 @@ int radeonTransformALU(
 	case RC_OPCODE_ROUND: transform_ROUND(c, inst); return 1;
 	case RC_OPCODE_RSQ: transform_RSQ(c, inst); return 1;
 	case RC_OPCODE_SEQ: transform_SEQ(c, inst); return 1;
-	case RC_OPCODE_SFL: transform_SFL(c, inst); return 1;
 	case RC_OPCODE_SGE: transform_SGE(c, inst); return 1;
 	case RC_OPCODE_SGT: transform_SGT(c, inst); return 1;
 	case RC_OPCODE_SLE: transform_SLE(c, inst); return 1;
@@ -683,24 +612,12 @@ int radeonTransformALU(
 	case RC_OPCODE_SNE: transform_SNE(c, inst); return 1;
 	case RC_OPCODE_SSG: transform_SSG(c, inst); return 1;
 	case RC_OPCODE_SUB: transform_SUB(c, inst); return 1;
-	case RC_OPCODE_SWZ: transform_SWZ(c, inst); return 1;
 	case RC_OPCODE_TRUNC: transform_TRUNC(c, inst); return 1;
-	case RC_OPCODE_XPD: transform_XPD(c, inst); return 1;
 	default:
 		return 0;
 	}
 }
 
-
-static void transform_r300_vertex_ABS(struct radeon_compiler* c,
-	struct rc_instruction* inst)
-{
-	/* Note: r500 can take absolute values, but r300 cannot. */
-	inst->U.I.Opcode = RC_OPCODE_MAX;
-	inst->U.I.SrcReg[1] = inst->U.I.SrcReg[0];
-	inst->U.I.SrcReg[1].Negate ^= RC_MASK_XYZW;
-}
-
 static void transform_r300_vertex_CMP(struct radeon_compiler* c,
 	struct rc_instruction* inst)
 {
@@ -717,13 +634,13 @@ static void transform_r300_vertex_CMP(struct radeon_compiler* c,
 	struct rc_dst_register dst = try_to_reuse_dst(c, inst);
 
 	/* SLT tmp0, src0, 0.0 */
-	emit2(c, inst->Prev, RC_OPCODE_SLT, 0,
+	emit2(c, inst->Prev, RC_OPCODE_SLT, NULL,
 		dst,
 		inst->U.I.SrcReg[0], builtin_zero);
 
 	/* LRP dst, tmp0, src1, src2 */
 	transform_LRP(c,
-		emit3(c, inst->Prev, RC_OPCODE_LRP, 0,
+		emit3(c, inst->Prev, RC_OPCODE_LRP, NULL,
 		      inst->U.I.DstReg,
 		      srcreg(RC_FILE_TEMPORARY, dst.Index), inst->U.I.SrcReg[1],  inst->U.I.SrcReg[2]));
 
@@ -764,12 +681,12 @@ static void transform_r300_vertex_fix_LIT(struct radeon_compiler* c,
 
 	/* MOV dst, src */
 	dst.WriteMask = RC_MASK_XYZW;
-	emit1(c, inst->Prev, RC_OPCODE_MOV, 0,
+	emit1(c, inst->Prev, RC_OPCODE_MOV, NULL,
 		dst,
 		inst->U.I.SrcReg[0]);
 
 	/* MAX dst.y, src, 0.00...001 */
-	emit2(c, inst->Prev, RC_OPCODE_MAX, 0,
+	emit2(c, inst->Prev, RC_OPCODE_MAX, NULL,
 		dstregtmpmask(dst.Index, RC_MASK_Y),
 		srcreg(RC_FILE_TEMPORARY, dst.Index),
 		srcregswz(RC_FILE_CONSTANT, constant, constant_swizzle));
@@ -784,19 +701,19 @@ static void transform_r300_vertex_SEQ(struct radeon_compiler *c,
 	int tmp = rc_find_free_temporary(c);
 
 	/* x <= y */
-	emit2(c, inst->Prev, RC_OPCODE_SGE, 0,
+	emit2(c, inst->Prev, RC_OPCODE_SGE, NULL,
 	      dstregtmpmask(tmp, inst->U.I.DstReg.WriteMask),
 	      inst->U.I.SrcReg[0],
 	      inst->U.I.SrcReg[1]);
 
 	/* y <= x */
-	emit2(c, inst->Prev, RC_OPCODE_SGE, 0,
+	emit2(c, inst->Prev, RC_OPCODE_SGE, NULL,
 	      inst->U.I.DstReg,
 	      inst->U.I.SrcReg[1],
 	      inst->U.I.SrcReg[0]);
 
 	/* x && y  =  x * y */
-	emit2(c, inst->Prev, RC_OPCODE_MUL, 0,
+	emit2(c, inst->Prev, RC_OPCODE_MUL, NULL,
 	      inst->U.I.DstReg,
 	      srcreg(RC_FILE_TEMPORARY, tmp),
 	      srcreg(inst->U.I.DstReg.File, inst->U.I.DstReg.Index));
@@ -811,19 +728,19 @@ static void transform_r300_vertex_SNE(struct radeon_compiler *c,
 	int tmp = rc_find_free_temporary(c);
 
 	/* x < y */
-	emit2(c, inst->Prev, RC_OPCODE_SLT, 0,
+	emit2(c, inst->Prev, RC_OPCODE_SLT, NULL,
 	      dstregtmpmask(tmp, inst->U.I.DstReg.WriteMask),
 	      inst->U.I.SrcReg[0],
 	      inst->U.I.SrcReg[1]);
 
 	/* y < x */
-	emit2(c, inst->Prev, RC_OPCODE_SLT, 0,
+	emit2(c, inst->Prev, RC_OPCODE_SLT, NULL,
 	      inst->U.I.DstReg,
 	      inst->U.I.SrcReg[1],
 	      inst->U.I.SrcReg[0]);
 
 	/* x || y  =  max(x, y) */
-	emit2(c, inst->Prev, RC_OPCODE_MAX, 0,
+	emit2(c, inst->Prev, RC_OPCODE_MAX, NULL,
 	      inst->U.I.DstReg,
 	      srcreg(RC_FILE_TEMPORARY, tmp),
 	      srcreg(inst->U.I.DstReg.File, inst->U.I.DstReg.Index));
@@ -858,26 +775,26 @@ static void transform_r300_vertex_SSG(struct radeon_compiler* c,
 	 *   SLT tmp1, x, 0;
 	 *   ADD result, tmp0, -tmp1;
 	 */
-	struct rc_dst_register dst0 = try_to_reuse_dst(c, inst);
+	struct rc_dst_register dst0;
 	unsigned tmp1;
 
 	/* 0 < x */
 	dst0 = try_to_reuse_dst(c, inst);
-	emit2(c, inst->Prev, RC_OPCODE_SLT, 0,
+	emit2(c, inst->Prev, RC_OPCODE_SLT, NULL,
 	      dst0,
 	      builtin_zero,
 	      inst->U.I.SrcReg[0]);
 
 	/* x < 0 */
 	tmp1 = rc_find_free_temporary(c);
-	emit2(c, inst->Prev, RC_OPCODE_SLT, 0,
+	emit2(c, inst->Prev, RC_OPCODE_SLT, NULL,
 	      dstregtmpmask(tmp1, inst->U.I.DstReg.WriteMask),
 	      inst->U.I.SrcReg[0],
 	      builtin_zero);
 
 	/* Either both are zero, or one of them is one and the other is zero. */
 	/* result = tmp0 - tmp1 */
-	emit2(c, inst->Prev, RC_OPCODE_ADD, 0,
+	emit2(c, inst->Prev, RC_OPCODE_ADD, NULL,
 	      inst->U.I.DstReg,
 	      srcreg(RC_FILE_TEMPORARY, dst0.Index),
 	      negate(srcreg(RC_FILE_TEMPORARY, tmp1)));
@@ -906,13 +823,10 @@ int r300_transform_vertex_alu(
 	void* unused)
 {
 	switch(inst->U.I.Opcode) {
-	case RC_OPCODE_ABS: transform_r300_vertex_ABS(c, inst); return 1;
 	case RC_OPCODE_CEIL: transform_CEIL(c, inst); return 1;
-	case RC_OPCODE_CLAMP: transform_CLAMP(c, inst); return 1;
 	case RC_OPCODE_CMP: transform_r300_vertex_CMP(c, inst); return 1;
 	case RC_OPCODE_DP2: transform_r300_vertex_DP2(c, inst); return 1;
 	case RC_OPCODE_DP3: transform_r300_vertex_DP3(c, inst); return 1;
-	case RC_OPCODE_DPH: transform_DPH(c, inst); return 1;
 	case RC_OPCODE_FLR: transform_FLR(c, inst); return 1;
 	case RC_OPCODE_LIT: transform_r300_vertex_fix_LIT(c, inst); return 1;
 	case RC_OPCODE_LRP: transform_LRP(c, inst); return 1;
@@ -922,7 +836,6 @@ int r300_transform_vertex_alu(
 			return 1;
 		}
 		return 0;
-	case RC_OPCODE_SFL: transform_SFL(c, inst); return 1;
 	case RC_OPCODE_SGT: transform_r300_vertex_SGT(c, inst); return 1;
 	case RC_OPCODE_SLE: transform_r300_vertex_SLE(c, inst); return 1;
 	case RC_OPCODE_SNE:
@@ -933,9 +846,7 @@ int r300_transform_vertex_alu(
 		return 0;
 	case RC_OPCODE_SSG: transform_r300_vertex_SSG(c, inst); return 1;
 	case RC_OPCODE_SUB: transform_SUB(c, inst); return 1;
-	case RC_OPCODE_SWZ: transform_SWZ(c, inst); return 1;
 	case RC_OPCODE_TRUNC: transform_vertex_TRUNC(c, inst); return 1;
-	case RC_OPCODE_XPD: transform_XPD(c, inst); return 1;
 	default:
 		return 0;
 	}
@@ -977,25 +888,25 @@ static void sin_approx(
 {
 	unsigned int tempreg = rc_find_free_temporary(c);
 
-	emit2(c, inst->Prev, RC_OPCODE_MUL, 0, dstregtmpmask(tempreg, RC_MASK_XY),
+	emit2(c, inst->Prev, RC_OPCODE_MUL, NULL, dstregtmpmask(tempreg, RC_MASK_XY),
 		swizzle_xxxx(src),
 		srcreg(RC_FILE_CONSTANT, constants[0]));
-	emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(tempreg, RC_MASK_X),
+	emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(tempreg, RC_MASK_X),
 		swizzle_yyyy(srcreg(RC_FILE_TEMPORARY, tempreg)),
 		absolute(swizzle_xxxx(src)),
 		swizzle_xxxx(srcreg(RC_FILE_TEMPORARY, tempreg)));
-	emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(tempreg, RC_MASK_Y),
+	emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(tempreg, RC_MASK_Y),
 		swizzle_xxxx(srcreg(RC_FILE_TEMPORARY, tempreg)),
 		absolute(swizzle_xxxx(srcreg(RC_FILE_TEMPORARY, tempreg))),
 		negate(swizzle_xxxx(srcreg(RC_FILE_TEMPORARY, tempreg))));
-	emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dst,
+	emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dst,
 		swizzle_yyyy(srcreg(RC_FILE_TEMPORARY, tempreg)),
 		swizzle_wwww(srcreg(RC_FILE_CONSTANT, constants[0])),
 		swizzle_xxxx(srcreg(RC_FILE_TEMPORARY, tempreg)));
 }
 
 /**
- * Translate the trigonometric functions COS, SIN, and SCS
+ * Translate the trigonometric functions COS and SIN
  * using only the basic instructions
  *  MOV, ADD, MUL, MAD, FRC
  */
@@ -1007,8 +918,7 @@ int r300_transform_trig_simple(struct radeon_compiler* c,
 	unsigned int tempreg;
 
 	if (inst->U.I.Opcode != RC_OPCODE_COS &&
-	    inst->U.I.Opcode != RC_OPCODE_SIN &&
-	    inst->U.I.Opcode != RC_OPCODE_SCS)
+	    inst->U.I.Opcode != RC_OPCODE_SIN)
 		return 0;
 
 	tempreg = rc_find_free_temporary(c);
@@ -1019,13 +929,13 @@ int r300_transform_trig_simple(struct radeon_compiler* c,
 		/* MAD tmp.x, src, 1/(2*PI), 0.75 */
 		/* FRC tmp.x, tmp.x */
 		/* MAD tmp.z, tmp.x, 2*PI, -PI */
-		emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(tempreg, RC_MASK_W),
+		emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(tempreg, RC_MASK_W),
 			swizzle_xxxx(inst->U.I.SrcReg[0]),
 			swizzle_zzzz(srcreg(RC_FILE_CONSTANT, constants[1])),
 			swizzle_xxxx(srcreg(RC_FILE_CONSTANT, constants[1])));
-		emit1(c, inst->Prev, RC_OPCODE_FRC, 0, dstregtmpmask(tempreg, RC_MASK_W),
+		emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, dstregtmpmask(tempreg, RC_MASK_W),
 			swizzle_wwww(srcreg(RC_FILE_TEMPORARY, tempreg)));
-		emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(tempreg, RC_MASK_W),
+		emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(tempreg, RC_MASK_W),
 			swizzle_wwww(srcreg(RC_FILE_TEMPORARY, tempreg)),
 			swizzle_wwww(srcreg(RC_FILE_CONSTANT, constants[1])),
 			negate(swizzle_zzzz(srcreg(RC_FILE_CONSTANT, constants[0]))));
@@ -1034,13 +944,13 @@ int r300_transform_trig_simple(struct radeon_compiler* c,
 			swizzle_wwww(srcreg(RC_FILE_TEMPORARY, tempreg)),
 			constants);
 	} else if (inst->U.I.Opcode == RC_OPCODE_SIN) {
-		emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(tempreg, RC_MASK_W),
+		emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(tempreg, RC_MASK_W),
 			swizzle_xxxx(inst->U.I.SrcReg[0]),
 			swizzle_zzzz(srcreg(RC_FILE_CONSTANT, constants[1])),
 			swizzle_yyyy(srcreg(RC_FILE_CONSTANT, constants[1])));
-		emit1(c, inst->Prev, RC_OPCODE_FRC, 0, dstregtmpmask(tempreg, RC_MASK_W),
+		emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, dstregtmpmask(tempreg, RC_MASK_W),
 			swizzle_wwww(srcreg(RC_FILE_TEMPORARY, tempreg)));
-		emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(tempreg, RC_MASK_W),
+		emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(tempreg, RC_MASK_W),
 			swizzle_wwww(srcreg(RC_FILE_TEMPORARY, tempreg)),
 			swizzle_wwww(srcreg(RC_FILE_CONSTANT, constants[1])),
 			negate(swizzle_zzzz(srcreg(RC_FILE_CONSTANT, constants[0]))));
@@ -1051,13 +961,13 @@ int r300_transform_trig_simple(struct radeon_compiler* c,
 	} else {
 		struct rc_dst_register dst;
 
-		emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(tempreg, RC_MASK_XY),
+		emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(tempreg, RC_MASK_XY),
 			swizzle_xxxx(inst->U.I.SrcReg[0]),
 			swizzle_zzzz(srcreg(RC_FILE_CONSTANT, constants[1])),
 			swizzle(srcreg(RC_FILE_CONSTANT, constants[1]), RC_SWIZZLE_X, RC_SWIZZLE_Y, RC_SWIZZLE_Z, RC_SWIZZLE_W));
-		emit1(c, inst->Prev, RC_OPCODE_FRC, 0, dstregtmpmask(tempreg, RC_MASK_XY),
+		emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, dstregtmpmask(tempreg, RC_MASK_XY),
 			srcreg(RC_FILE_TEMPORARY, tempreg));
-		emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(tempreg, RC_MASK_XY),
+		emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(tempreg, RC_MASK_XY),
 			srcreg(RC_FILE_TEMPORARY, tempreg),
 			swizzle_wwww(srcreg(RC_FILE_CONSTANT, constants[1])),
 			negate(swizzle_zzzz(srcreg(RC_FILE_CONSTANT, constants[0]))));
@@ -1080,7 +990,7 @@ int r300_transform_trig_simple(struct radeon_compiler* c,
 	return 1;
 }
 
-static void r300_transform_SIN_COS_SCS(struct radeon_compiler *c,
+static void r300_transform_SIN_COS(struct radeon_compiler *c,
 	struct rc_instruction *inst,
 	unsigned srctmp)
 {
@@ -1090,19 +1000,6 @@ static void r300_transform_SIN_COS_SCS(struct radeon_compiler *c,
 	} else if (inst->U.I.Opcode == RC_OPCODE_SIN) {
 		emit1(c, inst->Prev, RC_OPCODE_SIN, &inst->U.I,
 			inst->U.I.DstReg, srcregswz(RC_FILE_TEMPORARY, srctmp, RC_SWIZZLE_WWWW));
-	} else if (inst->U.I.Opcode == RC_OPCODE_SCS) {
-		struct rc_dst_register moddst = inst->U.I.DstReg;
-
-		if (inst->U.I.DstReg.WriteMask & RC_MASK_X) {
-			moddst.WriteMask = RC_MASK_X;
-			emit1(c, inst->Prev, RC_OPCODE_COS, &inst->U.I, moddst,
-				srcregswz(RC_FILE_TEMPORARY, srctmp, RC_SWIZZLE_WWWW));
-		}
-		if (inst->U.I.DstReg.WriteMask & RC_MASK_Y) {
-			moddst.WriteMask = RC_MASK_Y;
-			emit1(c, inst->Prev, RC_OPCODE_SIN, &inst->U.I, moddst,
-				srcregswz(RC_FILE_TEMPORARY, srctmp, RC_SWIZZLE_WWWW));
-		}
 	}
 
 	rc_remove_instruction(inst);
@@ -1110,10 +1007,9 @@ static void r300_transform_SIN_COS_SCS(struct radeon_compiler *c,
 
 
 /**
- * Transform the trigonometric functions COS, SIN, and SCS
+ * Transform the trigonometric functions COS and SIN
  * to include pre-scaling by 1/(2*PI) and taking the fractional
  * part, so that the input to COS and SIN is always in the range [0,1).
- * SCS is replaced by one COS and one SIN instruction.
  *
  * @warning This transformation implicitly changes the semantics of SIN and COS!
  */
@@ -1127,27 +1023,25 @@ int radeonTransformTrigScale(struct radeon_compiler* c,
 	unsigned int constant_swizzle;
 
 	if (inst->U.I.Opcode != RC_OPCODE_COS &&
-	    inst->U.I.Opcode != RC_OPCODE_SIN &&
-	    inst->U.I.Opcode != RC_OPCODE_SCS)
+	    inst->U.I.Opcode != RC_OPCODE_SIN)
 		return 0;
 
 	temp = rc_find_free_temporary(c);
 	constant = rc_constants_add_immediate_scalar(&c->Program.Constants, RCP_2PI, &constant_swizzle);
 
-	emit2(c, inst->Prev, RC_OPCODE_MUL, 0, dstregtmpmask(temp, RC_MASK_W),
+	emit2(c, inst->Prev, RC_OPCODE_MUL, NULL, dstregtmpmask(temp, RC_MASK_W),
 		swizzle_xxxx(inst->U.I.SrcReg[0]),
 		srcregswz(RC_FILE_CONSTANT, constant, constant_swizzle));
-	emit1(c, inst->Prev, RC_OPCODE_FRC, 0, dstregtmpmask(temp, RC_MASK_W),
+	emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, dstregtmpmask(temp, RC_MASK_W),
 		srcreg(RC_FILE_TEMPORARY, temp));
 
-	r300_transform_SIN_COS_SCS(c, inst, temp);
+	r300_transform_SIN_COS(c, inst, temp);
 	return 1;
 }
 
 /**
- * Transform the trigonometric functions COS, SIN, and SCS
+ * Transform the trigonometric functions COS and SIN
  * so that the input to COS and SIN is always in the range [-PI, PI].
- * SCS is replaced by one COS and one SIN instruction.
  */
 int r300_transform_trig_scale_vertex(struct radeon_compiler *c,
 	struct rc_instruction *inst,
@@ -1158,8 +1052,7 @@ int r300_transform_trig_scale_vertex(struct radeon_compiler *c,
 	unsigned int constant;
 
 	if (inst->U.I.Opcode != RC_OPCODE_COS &&
-	    inst->U.I.Opcode != RC_OPCODE_SIN &&
-	    inst->U.I.Opcode != RC_OPCODE_SCS)
+	    inst->U.I.Opcode != RC_OPCODE_SIN)
 		return 0;
 
 	/* Repeat x in the range [-PI, PI]:
@@ -1170,18 +1063,37 @@ int r300_transform_trig_scale_vertex(struct radeon_compiler *c,
 	temp = rc_find_free_temporary(c);
 	constant = rc_constants_add_immediate_vec4(&c->Program.Constants, cons);
 
-	emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(temp, RC_MASK_W),
+	emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(temp, RC_MASK_W),
 		swizzle_xxxx(inst->U.I.SrcReg[0]),
 		srcregswz(RC_FILE_CONSTANT, constant, RC_SWIZZLE_XXXX),
 		srcregswz(RC_FILE_CONSTANT, constant, RC_SWIZZLE_YYYY));
-	emit1(c, inst->Prev, RC_OPCODE_FRC, 0, dstregtmpmask(temp, RC_MASK_W),
+	emit1(c, inst->Prev, RC_OPCODE_FRC, NULL, dstregtmpmask(temp, RC_MASK_W),
 		srcreg(RC_FILE_TEMPORARY, temp));
-	emit3(c, inst->Prev, RC_OPCODE_MAD, 0, dstregtmpmask(temp, RC_MASK_W),
+	emit3(c, inst->Prev, RC_OPCODE_MAD, NULL, dstregtmpmask(temp, RC_MASK_W),
 		srcreg(RC_FILE_TEMPORARY, temp),
 		srcregswz(RC_FILE_CONSTANT, constant, RC_SWIZZLE_ZZZZ),
 		srcregswz(RC_FILE_CONSTANT, constant, RC_SWIZZLE_WWWW));
 
-	r300_transform_SIN_COS_SCS(c, inst, temp);
+	r300_transform_SIN_COS(c, inst, temp);
+	return 1;
+}
+
+/**
+ * Replaces DDX/DDY instructions with MOV 0 to avoid using dummy shaders on r300/r400.
+ *
+ * @warning This explicitly changes the form of DDX and DDY!
+ */
+
+int radeonStubDeriv(struct radeon_compiler* c,
+	struct rc_instruction* inst,
+	void* unused)
+{
+	if (inst->U.I.Opcode != RC_OPCODE_DDX && inst->U.I.Opcode != RC_OPCODE_DDY)
+		return 0;
+
+	inst->U.I.Opcode = RC_OPCODE_MOV;
+	inst->U.I.SrcReg[0].Swizzle = RC_SWIZZLE_0000;
+
 	return 1;
 }
 
@@ -1215,12 +1127,6 @@ int radeonTransformDeriv(struct radeon_compiler* c,
  *
  * === OR ===
  *
- * IF Temp[0].x -\
- * KILL         - > KIL -abs(Temp[0].x)
- * ENDIF        -/
- *
- * === OR ===
- *
  * IF Temp[0].x -> IF Temp[0].x
  * ...          -> ...
  * ELSE         -> ELSE
@@ -1265,21 +1171,6 @@ void rc_transform_KILL(struct radeon_compiler * c, void *user)
 			 * block, because -0.0 is considered negative. */
 			inst->U.I.SrcReg[0] =
 				negate(absolute(if_inst->U.I.SrcReg[0]));
-
-			if (inst->Prev->U.I.Opcode != RC_OPCODE_IF
-				&& inst->Next->U.I.Opcode != RC_OPCODE_ENDIF) {
-
-				/* Optimize the special case:
-				 * IF Temp[0].x
-				 * KILP
-				 * ENDIF
-				 */
-
-				/* Remove IF */
-				rc_remove_instruction(inst->Prev);
-				/* Remove ENDIF */
-				rc_remove_instruction(inst->Next);
-			}
 		}
 	}
 }
@@ -1298,7 +1189,7 @@ int rc_force_output_alpha_to_one(struct radeon_compiler *c,
 	tmp = rc_find_free_temporary(c);
 
 	/* Insert MOV after inst, set alpha to 1. */
-	emit1(c, inst, RC_OPCODE_MOV, 0, inst->U.I.DstReg,
+	emit1(c, inst, RC_OPCODE_MOV, NULL, inst->U.I.DstReg,
 	      srcregswz(RC_FILE_TEMPORARY, tmp, RC_SWIZZLE_XYZ1));
 
 	/* Re-route the destination of inst to the source of mov. */
